@@ -23,7 +23,6 @@
 #include "V3Broken.h"
 #include "V3String.h"
 
-#include <cstdarg>
 #include <iomanip>
 #include <memory>
 
@@ -61,17 +60,17 @@ std::ostream& operator<<(std::ostream& os, AstType rhs);
 
 void AstNode::init() {
     editCountInc();
-    m_fileline = NULL;
-    m_nextp = NULL;
-    m_backp = NULL;
+    m_fileline = nullptr;
+    m_nextp = nullptr;
+    m_backp = nullptr;
     m_headtailp = this;  // When made, we're a list of only a single element
-    m_op1p = NULL;
-    m_op2p = NULL;
-    m_op3p = NULL;
-    m_op4p = NULL;
-    m_iterpp = NULL;
-    m_dtypep = NULL;
-    m_clonep = NULL;
+    m_op1p = nullptr;
+    m_op2p = nullptr;
+    m_op3p = nullptr;
+    m_op4p = nullptr;
+    m_iterpp = nullptr;
+    m_dtypep = nullptr;
+    m_clonep = nullptr;
     m_cloneCnt = 0;
     // Attributes
     m_didWidth = false;
@@ -276,8 +275,8 @@ AstNode* AstNode::addNext(AstNode* nodep, AstNode* newp) {
         // New tail needs the head
         AstNode* newtailp = newp->m_headtailp;
         AstNode* headp = oldtailp->m_headtailp;
-        oldtailp->m_headtailp = NULL;  // May be written again as new head
-        newp->m_headtailp = NULL;  // May be written again as new tail
+        oldtailp->m_headtailp = nullptr;  // May be written again as new head
+        newp->m_headtailp = nullptr;  // May be written again as new tail
         newtailp->m_headtailp = headp;
         headp->m_headtailp = newtailp;
         newp->editCountInc();
@@ -322,17 +321,17 @@ void AstNode::addNextHere(AstNode* newp) {
     //    tail (oldheadtailp && !oldnextp)   // this was tail of list, might also
     //                                     be head of one-node list
     //
-    newp->m_headtailp = NULL;  // Not at head any longer
-    addlastp->m_headtailp = NULL;  // Presume middle of list
+    newp->m_headtailp = nullptr;  // Not at head any longer
+    addlastp->m_headtailp = nullptr;  // Presume middle of list
     // newp might happen to be head/tail after all, if so will be set again below
     if (oldheadtailp) {  // else in middle of list, no change
         if (oldheadtailp == this) {  // this was one node
             this->m_headtailp = addlastp;  // Was head/tail, now a tail
-            addlastp->m_headtailp = oldheadtailp;  // Tail needs to remember head (or NULL)
+            addlastp->m_headtailp = oldheadtailp;  // Tail needs to remember head (or nullptr)
         } else if (!oldnextp) {  // this was tail
-            this->m_headtailp = NULL;  // No longer a tail
+            this->m_headtailp = nullptr;  // No longer a tail
             oldheadtailp->m_headtailp = addlastp;  // Head gets new tail
-            addlastp->m_headtailp = oldheadtailp;  // Tail needs to remember head (or NULL)
+            addlastp->m_headtailp = oldheadtailp;  // Tail needs to remember head (or nullptr)
         }  // else is head, and we're inserting into the middle, so no other change
     }
 
@@ -471,7 +470,7 @@ AstNode* AstNode::unlinkFrBackWithNext(AstNRelinker* linkerp) {
         }
     }
     if (backp->m_nextp == oldp) {
-        backp->m_nextp = NULL;
+        backp->m_nextp = nullptr;
         // Old list gets truncated
         // New list becomes a list upon itself
         // Most common case is unlinking a entire operand tree
@@ -487,21 +486,21 @@ AstNode* AstNode::unlinkFrBackWithNext(AstNRelinker* linkerp) {
         oldp->m_headtailp = oldtailp;
         oldp->m_headtailp->m_headtailp = oldp;
     } else if (backp->m_op1p == oldp) {
-        backp->m_op1p = NULL;
+        backp->m_op1p = nullptr;
     } else if (backp->m_op2p == oldp) {
-        backp->m_op2p = NULL;
+        backp->m_op2p = nullptr;
     } else if (backp->m_op3p == oldp) {
-        backp->m_op3p = NULL;
+        backp->m_op3p = nullptr;
     } else if (backp->m_op4p == oldp) {
-        backp->m_op4p = NULL;
+        backp->m_op4p = nullptr;
     } else {
         this->v3fatalSrc("Unlink of node with back not pointing to it.");
     }
     // Relink
-    oldp->m_backp = NULL;
+    oldp->m_backp = nullptr;
     // Iterator fixup
-    if (oldp->m_iterpp) *(oldp->m_iterpp) = NULL;
-    oldp->m_iterpp = NULL;
+    if (oldp->m_iterpp) *(oldp->m_iterpp) = nullptr;
+    oldp->m_iterpp = nullptr;
     oldp->debugTreeChange("-unlinkWNextOut: ", __LINE__, true);
     return oldp;
 }
@@ -562,10 +561,10 @@ AstNode* AstNode::unlinkFrBack(AstNRelinker* linkerp) {
     // Iterator fixup
     if (oldp->m_iterpp) *(oldp->m_iterpp) = oldp->m_nextp;
     // Relink
-    oldp->m_nextp = NULL;
-    oldp->m_backp = NULL;
+    oldp->m_nextp = nullptr;
+    oldp->m_backp = nullptr;
     oldp->m_headtailp = this;
-    oldp->m_iterpp = NULL;
+    oldp->m_iterpp = nullptr;
     oldp->debugTreeChange("-unlinkFrBkOut: ", __LINE__, true);
     return oldp;
 }
@@ -599,7 +598,7 @@ void AstNode::relink(AstNRelinker* linkerp) {
     }
     // Relink
     newp->m_backp = backp;
-    linkerp->m_backp = NULL;
+    linkerp->m_backp = nullptr;
     // Iterator fixup
     if (linkerp->m_iterpp) {
         // If we're iterating over a next() link, we need to follow links off the
@@ -611,7 +610,7 @@ void AstNode::relink(AstNRelinker* linkerp) {
         newp->m_iterpp = linkerp->m_iterpp;
     }
     // Empty the linker so not used twice accidentally
-    linkerp->m_backp = NULL;
+    linkerp->m_backp = nullptr;
     this->debugTreeChange("-relinkOut: ", __LINE__, true);
 }
 
@@ -634,8 +633,8 @@ void AstNode::relinkOneLink(AstNode*& pointpr,  // Ref to pointer that gets set
         newlistlastp->m_nextp = pointpr;
         pointpr->m_backp = newlistlastp;
         // Head/tail
-        pointpr->m_headtailp = NULL;  // Old head
-        newlistlastp->m_headtailp = NULL;  // Old tail
+        pointpr->m_headtailp = nullptr;  // Old head
+        newlistlastp->m_headtailp = nullptr;  // Old tail
         newp->m_headtailp = oldlistlastp;  // Head points to tail
         oldlistlastp->m_headtailp = newp;  // Tail points to head
     }
@@ -669,7 +668,7 @@ AstNode* AstNode::cloneTreeIter() {
     if (this->m_op2p) newp->op2p(this->m_op2p->cloneTreeIterList());
     if (this->m_op3p) newp->op3p(this->m_op3p->cloneTreeIterList());
     if (this->m_op4p) newp->op4p(this->m_op4p->cloneTreeIterList());
-    newp->m_iterpp = NULL;
+    newp->m_iterpp = nullptr;
     newp->clonep(this);  // Save pointers to/from both to simplify relinking.
     this->clonep(newp);  // Save pointers to/from both to simplify relinking.
     return newp;
@@ -677,12 +676,12 @@ AstNode* AstNode::cloneTreeIter() {
 
 AstNode* AstNode::cloneTreeIterList() {
     // private: Clone list of nodes, set m_headtailp
-    AstNode* newheadp = NULL;
-    AstNode* newtailp = NULL;
-    // Audited to make sure this is never NULL
+    AstNode* newheadp = nullptr;
+    AstNode* newtailp = nullptr;
+    // Audited to make sure this is never nullptr
     for (AstNode* oldp = this; oldp; oldp = oldp->m_nextp) {
         AstNode* newp = oldp->cloneTreeIter();
-        newp->m_headtailp = NULL;
+        newp->m_headtailp = nullptr;
         newp->m_backp = newtailp;
         if (newtailp) newtailp->m_nextp = newp;
         if (!newheadp) newheadp = newp;
@@ -701,10 +700,10 @@ AstNode* AstNode::cloneTree(bool cloneNextLink) {
         newp = cloneTreeIterList();
     } else {
         newp = cloneTreeIter();
-        newp->m_nextp = NULL;
+        newp->m_nextp = nullptr;
         newp->m_headtailp = newp;
     }
-    newp->m_backp = NULL;
+    newp->m_backp = nullptr;
     newp->cloneRelinkTree();
     newp->debugTreeChange("-cloneOut: ", __LINE__, true);
     return newp;
@@ -727,7 +726,7 @@ void AstNode::deleteNode() {
     this->m_op4p = reinterpret_cast<AstNode*>(0x1);
     if (
 #if !defined(VL_DEBUG) || defined(VL_LEAK_CHECKS)
-        1
+        true
 #else
         !v3Global.opt.debugLeak()
 #endif
@@ -740,7 +739,7 @@ void AstNode::deleteNode() {
 
 void AstNode::deleteTreeIter() {
     // private: Delete list of nodes. Publicly call deleteTree() instead.
-    // Audited to make sure this is never NULL
+    // Audited to make sure this is never nullptr
     for (AstNode *nodep = this, *nnextp; nodep; nodep = nnextp) {
         nnextp = nodep->m_nextp;
         // MUST be depth first!
@@ -748,8 +747,8 @@ void AstNode::deleteTreeIter() {
         if (nodep->m_op2p) nodep->m_op2p->deleteTreeIter();
         if (nodep->m_op3p) nodep->m_op3p->deleteTreeIter();
         if (nodep->m_op4p) nodep->m_op4p->deleteTreeIter();
-        nodep->m_nextp = NULL;
-        nodep->m_backp = NULL;
+        nodep->m_nextp = nullptr;
+        nodep->m_backp = nullptr;
         nodep->deleteNode();
     }
 }
@@ -832,11 +831,11 @@ void AstNode::iterateAndNext(AstNVisitor& v) {
         niterp->m_iterpp = &niterp;
         niterp->accept(v);
         // accept may do a replaceNode and change niterp on us...
-        // niterp maybe NULL, so need cast if printing
+        // niterp maybe nullptr, so need cast if printing
         // if (niterp != nodep) UINFO(1,"iterateAndNext edited "<<cvtToHex(nodep)
         //                             <<" now into "<<cvtToHex(niterp)<<endl);
         if (!niterp) return;  // Perhaps node deleted inside accept
-        niterp->m_iterpp = NULL;
+        niterp->m_iterpp = nullptr;
         if (VL_UNLIKELY(niterp != nodep)) {  // Edited node inside accept
             nodep = niterp;
         } else {  // Unchanged node, just continue loop
@@ -854,7 +853,7 @@ void AstNode::iterateListBackwards(AstNVisitor& v) {
         if (nodep->backp()->m_nextp == nodep) {
             nodep = nodep->backp();
         } else {
-            nodep = NULL;
+            nodep = nullptr;
         }  // else: backp points up the tree.
     }
 }
@@ -901,7 +900,7 @@ AstNode* AstNode::iterateSubtreeReturnEdits(AstNVisitor& v) {
     } else {
         // Use back to determine who's pointing at us (IE assume new node
         // grafts into same place as old one)
-        AstNode** nextnodepp = NULL;
+        AstNode** nextnodepp = nullptr;
         if (this->m_backp->m_op1p == this) {
             nextnodepp = &(this->m_backp->m_op1p);
         } else if (this->m_backp->m_op2p == this) {
@@ -975,9 +974,7 @@ std::ostream& operator<<(std::ostream& os, const V3Hash& rhs) {
 
 V3Hash::V3Hash(const string& name) {
     uint32_t val = 0;
-    for (string::const_iterator it = name.begin(); it != name.end(); ++it) {
-        val = val * 31 + *it;
-    }
+    for (const auto& c : name) val = val * 31 + c;
     setBoth(1, val);
 }
 
@@ -1000,7 +997,7 @@ void AstNode::checkTreeIter(AstNode* backp) {
 
 void AstNode::checkTreeIterList(AstNode* backp) {
     // private: Check a (possible) list of nodes, this is always the head of the list
-    // Audited to make sure this is never NULL
+    // Audited to make sure this is never nullptr
     AstNode* headp = this;
     AstNode* tailp = this;
     for (AstNode* nodep = headp; nodep; nodep = nodep->nextp()) {
@@ -1053,7 +1050,7 @@ void AstNode::checkIter() const {
     if (m_iterpp) {
         dumpPtrs(cout);
         // Perhaps something forgot to clear m_iterpp?
-        this->v3fatalSrc("Iteration link should be NULL");
+        this->v3fatalSrc("Iteration link should be nullptr");
     }
 }
 
@@ -1109,7 +1106,7 @@ void AstNode::dumpTree(std::ostream& os, const string& indent, int maxDepth) con
 }
 
 void AstNode::dumpTreeAndNext(std::ostream& os, const string& indent, int maxDepth) const {
-    // Audited to make sure this is never NULL
+    // Audited to make sure this is never nullptr
     for (const AstNode* nodep = this; nodep; nodep = nodep->nextp()) {
         nodep->dumpTree(os, indent, maxDepth);
     }
@@ -1120,7 +1117,7 @@ void AstNode::dumpTreeFile(const string& filename, bool append, bool doDump) {
     if (doDump) {
         {  // Write log & close
             UINFO(2, "Dumping " << filename << endl);
-            const vl_unique_ptr<std::ofstream> logsp(V3File::new_ofstream(filename, append));
+            const std::unique_ptr<std::ofstream> logsp(V3File::new_ofstream(filename, append));
             if (logsp->fail()) v3fatal("Can't write " << filename);
             *logsp << "Verilator Tree Dump (format 0x3900) from <e" << std::dec << editCountLast();
             *logsp << "> to <e" << std::dec << editCountGbl() << ">" << endl;
@@ -1129,6 +1126,7 @@ void AstNode::dumpTreeFile(const string& filename, bool append, bool doDump) {
                 *logsp << "No changes since last dump!\n";
             } else {
                 dumpTree(*logsp);
+                editCountSetLast();  // Next dump can indicate start from here
             }
         }
     }
@@ -1139,13 +1137,11 @@ void AstNode::dumpTreeFile(const string& filename, bool append, bool doDump) {
         // set by other steps if it is called in the middle of other operations
         if (AstNetlist* netp = VN_CAST(this, Netlist)) V3Broken::brokenAll(netp);
     }
-    // Next dump can indicate start from here
-    editCountSetLast();
 }
 
 void AstNode::v3errorEndFatal(std::ostringstream& str) const {
     v3errorEnd(str);
-    assert(0);
+    assert(0);  // LCOV_EXCL_LINE
     VL_UNREACHABLE
 }
 
@@ -1154,10 +1150,10 @@ string AstNode::locationStr() const {
     const AstNode* backp = this;
     int itmax = 10000;  // Max iterations before giving up on location search
     while (backp) {
-        if (--itmax < 0) {
+        if (VL_UNCOVERABLE(--itmax < 0)) {
             // Likely some circular back link, and V3Ast is trying to report a low-level error
             UINFO(1, "Ran out of iterations finding locationStr on " << backp << endl);
-            return "";
+            return "";  // LCOV_EXCL_LINE
         }
         const AstScope* scopep;
         if ((scopep = VN_CAST_CONST(backp, Scope))) {
@@ -1269,8 +1265,6 @@ AstNodeDType* AstNode::findVoidDType() const {
 // AstNVisitor
 
 void AstNVisitor::doDeletes() {
-    for (std::vector<AstNode*>::iterator it = m_deleteps.begin(); it != m_deleteps.end(); ++it) {
-        (*it)->deleteTree();
-    }
+    for (AstNode* nodep : m_deleteps) nodep->deleteTree();
     m_deleteps.clear();
 }
