@@ -32,8 +32,7 @@
 #include "V3AstConstOnly.h"
 
 #include <algorithm>
-#include <cstdarg>
-#include VL_INCLUDE_UNORDERED_MAP
+#include <unordered_map>
 
 //######################################################################
 
@@ -42,7 +41,7 @@ class BrokenTable : public AstNVisitor {
 private:
     // MEMBERS
     //   For each node, we keep if it exists or not.
-    typedef vl_unordered_map<const AstNode*, int> NodeMap;  // Performance matters (when --debug)
+    typedef std::unordered_map<const AstNode*, int> NodeMap;  // Performance matters (when --debug)
     static NodeMap s_nodes;  // Set of all nodes that exist
     // BITMASK
     enum { FLAG_ALLOCATED = 0x01 };  // new() and not delete()ed
@@ -50,12 +49,13 @@ private:
     enum { FLAG_LINKABLE = 0x04 };  // Is in netlist tree, can be linked to
     enum { FLAG_LEAKED = 0x08 };  // Known to have been leaked
     enum { FLAG_UNDER_NOW = 0x10 };  // Is in tree as parent of current node
+
 public:
     // METHODS
     static void deleted(const AstNode* nodep) {
         // Called by operator delete on any node - only if VL_LEAK_CHECKS
         if (debug() >= 9) cout << "-nodeDel:  " << cvtToHex(nodep) << endl;
-        NodeMap::iterator iter = s_nodes.find(nodep);
+        const auto iter = s_nodes.find(nodep);
         UASSERT_OBJ(!(iter == s_nodes.end() || !(iter->second & FLAG_ALLOCATED)),
                     reinterpret_cast<const AstNode*>(nodep),
                     "Deleting AstNode object that was never tracked or already deleted");
@@ -68,7 +68,7 @@ public:
     static void addNewed(const AstNode* nodep) {
         // Called by operator new on any node - only if VL_LEAK_CHECKS
         if (debug() >= 9) cout << "-nodeNew:  " << cvtToHex(nodep) << endl;
-        NodeMap::iterator iter = s_nodes.find(nodep);
+        const auto iter = s_nodes.find(nodep);
         UASSERT_OBJ(!(iter != s_nodes.end() && (iter->second & FLAG_ALLOCATED)), nodep,
                     "Newing AstNode object that is already allocated");
         if (iter == s_nodes.end()) {
@@ -79,7 +79,7 @@ public:
     static void setUnder(const AstNode* nodep, bool flag) {
         // Called by BrokenCheckVisitor when each node entered/exited
         if (!okIfLinkedTo(nodep)) return;
-        NodeMap::iterator iter = s_nodes.find(nodep);
+        const auto iter = s_nodes.find(nodep);
         if (iter != s_nodes.end()) {
             iter->second &= ~FLAG_UNDER_NOW;
             if (flag) iter->second |= FLAG_UNDER_NOW;
@@ -90,7 +90,7 @@ public:
         // cppcheck-suppress knownConditionTrueFalse
         if (!linkable) return;  // save some time, else the map will get huge!
 #endif
-        NodeMap::iterator iter = s_nodes.find(nodep);
+        const auto iter = s_nodes.find(nodep);
         if (VL_UNCOVERABLE(iter == s_nodes.end())) {
 #ifdef VL_LEAK_CHECKS
             nodep->v3fatalSrc("AstNode is in tree, but not allocated");
@@ -114,7 +114,7 @@ public:
         // Some generic node has a pointer to this node.  Is it allocated?
         // Use this when might not be in tree; otherwise use okIfLinkedTo().
 #ifdef VL_LEAK_CHECKS
-        NodeMap::iterator iter = s_nodes.find(nodep);
+        const auto iter = s_nodes.find(nodep);
         if (iter == s_nodes.end()) return false;
         if (!(iter->second & FLAG_ALLOCATED)) return false;
 #endif
@@ -122,7 +122,7 @@ public:
     }
     static bool okIfLinkedTo(const AstNode* nodep) {
         // Some node in tree has a pointer to this node.  Is it kosher?
-        NodeMap::iterator iter = s_nodes.find(nodep);
+        const auto iter = s_nodes.find(nodep);
         if (iter == s_nodes.end()) return false;
 #ifdef VL_LEAK_CHECKS
         if (!(iter->second & FLAG_ALLOCATED)) return false;
@@ -131,10 +131,18 @@ public:
         if (!(iter->second & FLAG_LINKABLE)) return false;
         return true;
     }
+    static bool okIfAbove(const AstNode* nodep) {
+        // Must be linked to and below current node
+        if (!okIfLinkedTo(nodep)) return false;
+        const auto iter = s_nodes.find(nodep);
+        if (iter == s_nodes.end()) return false;
+        if ((iter->second & FLAG_UNDER_NOW)) return false;
+        return true;
+    }
     static bool okIfBelow(const AstNode* nodep) {
         // Must be linked to and below current node
         if (!okIfLinkedTo(nodep)) return false;
-        NodeMap::iterator iter = s_nodes.find(nodep);
+        const auto iter = s_nodes.find(nodep);
         if (iter == s_nodes.end()) return false;
         if (!(iter->second & FLAG_UNDER_NOW)) return false;
         return true;
@@ -142,19 +150,22 @@ public:
     static void prepForTree() {
 #ifndef VL_LEAK_CHECKS
         s_nodes.clear();
-#endif
+#else
         for (NodeMap::iterator it = s_nodes.begin(); it != s_nodes.end(); ++it) {
             it->second &= ~FLAG_IN_TREE;
             it->second &= ~FLAG_LINKABLE;
         }
+#endif
     }
     static void doneWithTree() {
         for (int backs = 0; backs < 2;
              backs++) {  // Those with backp() are probably under one leaking without
             for (NodeMap::iterator it = s_nodes.begin(); it != s_nodes.end(); ++it) {
-                if ((it->second & FLAG_ALLOCATED) && !(it->second & FLAG_IN_TREE)
-                    && !(it->second & FLAG_LEAKED)
-                    && (it->first->backp() ? backs == 1 : backs == 0)) {
+                // LCOV_EXCL_START
+                if (VL_UNCOVERABLE((it->second & FLAG_ALLOCATED) && !(it->second & FLAG_IN_TREE)
+                                   && !(it->second & FLAG_LEAKED)
+                                   && (it->first->backp() ? backs == 1 : backs == 0))) {
+
                     // Use only AstNode::dump instead of the virtual one, as there
                     // may be varp() and other cross links that are bad.
                     if (v3Global.opt.debugCheck()) {
@@ -173,14 +184,14 @@ public:
                     }
                     it->second |= FLAG_LEAKED;
                 }
+                // LCOV_EXCL_STOP
             }
         }
     }
 
-public:
     // CONSTRUCTORS
     BrokenTable() {}
-    virtual ~BrokenTable() {}
+    virtual ~BrokenTable() override {}
 };
 
 BrokenTable::NodeMap BrokenTable::s_nodes;
@@ -193,6 +204,10 @@ bool AstNode::brokeExistsAbove() const {
     // Called by node->broken() routines to do table lookup
     return BrokenTable::okIfBelow(this);
 }
+bool AstNode::brokeExistsBelow() const {
+    // Called by node->broken() routines to do table lookup
+    return BrokenTable::okIfAbove(this);
+}
 
 //######################################################################
 
@@ -208,7 +223,7 @@ private:
         iterateChildrenConst(nodep);
     }
     // VISITORS
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         // Process not just iterate
         processAndIterate(nodep);
     }
@@ -216,7 +231,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit BrokenMarkVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~BrokenMarkVisitor() {}
+    virtual ~BrokenMarkVisitor() override {}
 };
 
 //######################################################################
@@ -224,7 +239,7 @@ public:
 
 class BrokenCheckVisitor : public AstNVisitor {
 private:
-    void checkWidthMin(const AstNode* nodep) {
+    static void checkWidthMin(const AstNode* nodep) {
         UASSERT_OBJ(nodep->width() == nodep->widthMin()
                         || v3Global.widthMinUsage() != VWidthMinUsage::MATCHES_WIDTH,
                     nodep, "Width != WidthMin");
@@ -256,14 +271,14 @@ private:
         iterateChildrenConst(nodep);
         BrokenTable::setUnder(nodep, false);
     }
-    virtual void visit(AstNodeAssign* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeAssign* nodep) override {
         processAndIterate(nodep);
         UASSERT_OBJ(!(v3Global.assertDTypesResolved() && nodep->brokeLhsMustBeLvalue()
                       && VN_IS(nodep->lhsp(), NodeVarRef)
                       && !VN_CAST(nodep->lhsp(), NodeVarRef)->lvalue()),
                     nodep, "Assignment LHS is not an lvalue");
     }
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         // Process not just iterate
         processAndIterate(nodep);
     }
@@ -271,7 +286,7 @@ private:
 public:
     // CONSTRUCTORS
     explicit BrokenCheckVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~BrokenCheckVisitor() {}
+    virtual ~BrokenCheckVisitor() override {}
 };
 
 //######################################################################
