@@ -33,14 +33,11 @@
 #include "V3File.h"
 
 #include <algorithm>
-#include <cstdarg>
 #include <deque>
 #include <iomanip>
-#include <list>
 #include <memory>
-#include <vector>
 
-#define CDC_WEIGHT_ASYNC 0x1000  // Weight for edges that feed async logic
+constexpr int CDC_WEIGHT_ASYNC = 0x1000;  // Weight for edges that feed async logic
 
 //######################################################################
 
@@ -55,25 +52,23 @@ public:
 class CdcEitherVertex : public V3GraphVertex {
     AstScope* m_scopep;
     AstNode* m_nodep;
-    AstSenTree* m_srcDomainp;
-    AstSenTree* m_dstDomainp;
+    AstSenTree* m_srcDomainp = nullptr;
+    AstSenTree* m_dstDomainp = nullptr;
     bool m_srcDomainSet : 1;
     bool m_dstDomainSet : 1;
     bool m_asyncPath : 1;
 
 public:
     CdcEitherVertex(V3Graph* graphp, AstScope* scopep, AstNode* nodep)
-        : V3GraphVertex(graphp)
-        , m_scopep(scopep)
-        , m_nodep(nodep)
-        , m_srcDomainp(NULL)
-        , m_dstDomainp(NULL)
-        , m_srcDomainSet(false)
-        , m_dstDomainSet(false)
-        , m_asyncPath(false) {}
-    virtual ~CdcEitherVertex() {}
+        : V3GraphVertex{graphp}
+        , m_scopep{scopep}
+        , m_nodep{nodep}
+        , m_srcDomainSet{false}
+        , m_dstDomainSet{false}
+        , m_asyncPath{false} {}
+    virtual ~CdcEitherVertex() override {}
     // ACCESSORS
-    virtual FileLine* fileline() const { return nodep()->fileline(); }
+    virtual FileLine* fileline() const override { return nodep()->fileline(); }
     AstScope* scopep() const { return m_scopep; }
     AstNode* nodep() const { return m_nodep; }
     AstSenTree* srcDomainp() const { return m_srcDomainp; }
@@ -90,20 +85,18 @@ public:
 
 class CdcVarVertex : public CdcEitherVertex {
     AstVarScope* m_varScp;
-    int m_cntAsyncRst;
-    bool m_fromFlop;
+    int m_cntAsyncRst = 0;
+    bool m_fromFlop = false;
 
 public:
     CdcVarVertex(V3Graph* graphp, AstScope* scopep, AstVarScope* varScp)
-        : CdcEitherVertex(graphp, scopep, varScp)
-        , m_varScp(varScp)
-        , m_cntAsyncRst(0)
-        , m_fromFlop(false) {}
-    virtual ~CdcVarVertex() {}
+        : CdcEitherVertex{graphp, scopep, varScp}
+        , m_varScp{varScp} {}
+    virtual ~CdcVarVertex() override {}
     // ACCESSORS
     AstVarScope* varScp() const { return m_varScp; }
-    virtual string name() const { return (cvtToHex(m_varScp) + " " + varScp()->name()); }
-    virtual string dotColor() const {
+    virtual string name() const override { return (cvtToHex(m_varScp) + " " + varScp()->name()); }
+    virtual string dotColor() const override {
         return fromFlop() ? "green" : cntAsyncRst() ? "red" : "blue";
     }
     int cntAsyncRst() const { return m_cntAsyncRst; }
@@ -118,16 +111,18 @@ class CdcLogicVertex : public CdcEitherVertex {
 
 public:
     CdcLogicVertex(V3Graph* graphp, AstScope* scopep, AstNode* nodep, AstSenTree* sensenodep)
-        : CdcEitherVertex(graphp, scopep, nodep)
-        , m_hazard(false)
-        , m_isFlop(false) {
+        : CdcEitherVertex{graphp, scopep, nodep}
+        , m_hazard{false}
+        , m_isFlop{false} {
         srcDomainp(sensenodep);
         dstDomainp(sensenodep);
     }
-    virtual ~CdcLogicVertex() {}
+    virtual ~CdcLogicVertex() override {}
     // ACCESSORS
-    virtual string name() const { return (cvtToHex(nodep()) + "@" + scopep()->prettyName()); }
-    virtual string dotColor() const { return hazard() ? "black" : "yellow"; }
+    virtual string name() const override {
+        return (cvtToHex(nodep()) + "@" + scopep()->prettyName());
+    }
+    virtual string dotColor() const override { return hazard() ? "black" : "yellow"; }
     bool hazard() const { return m_hazard; }
     void setHazard(AstNode* nodep) {
         m_hazard = true;
@@ -148,7 +143,7 @@ private:
     std::ofstream* m_ofp;  // Output file
     string m_prefix;
 
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         *m_ofp << m_prefix;
         if (nodep->user3()) {
             *m_ofp << " %%";
@@ -170,22 +165,22 @@ private:
 
 public:
     // CONSTRUCTORS
-    CdcDumpVisitor(AstNode* nodep, std::ofstream* ofp, const string& prefix) {
-        m_ofp = ofp;
-        m_prefix = prefix;
+    CdcDumpVisitor(AstNode* nodep, std::ofstream* ofp, const string& prefix)
+        : m_ofp{ofp}
+        , m_prefix{prefix} {
         iterate(nodep);
     }
-    virtual ~CdcDumpVisitor() {}
+    virtual ~CdcDumpVisitor() override {}
 };
 
 //######################################################################
 
 class CdcWidthVisitor : public CdcBaseVisitor {
 private:
-    int m_maxLineno;
-    size_t m_maxFilenameLen;
+    int m_maxLineno = 0;
+    size_t m_maxFilenameLen = 0;
 
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         iterateChildren(nodep);
         // Keeping line+filename lengths separate is much faster than calling ascii().length()
         if (nodep->fileline()->lineno() >= m_maxLineno) {
@@ -198,14 +193,10 @@ private:
 
 public:
     // CONSTRUCTORS
-    explicit CdcWidthVisitor(AstNode* nodep) {
-        m_maxLineno = 0;
-        m_maxFilenameLen = 0;
-        iterate(nodep);
-    }
-    virtual ~CdcWidthVisitor() {}
+    explicit CdcWidthVisitor(AstNode* nodep) { iterate(nodep); }
+    virtual ~CdcWidthVisitor() override {}
     // ACCESSORS
-    int maxWidth() {
+    int maxWidth() const {
         size_t width = 1;
         width += m_maxFilenameLen;
         width += 1;  // The :
@@ -232,16 +223,16 @@ private:
 
     // STATE
     V3Graph m_graph;  // Scoreboard of var usages/dependencies
-    CdcLogicVertex* m_logicVertexp;  // Current statement being tracked, NULL=ignored
-    AstScope* m_scopep;  // Current scope being processed
-    AstNodeModule* m_modp;  // Current module
-    AstSenTree* m_domainp;  // Current sentree
-    bool m_inDly;  // In delayed assign
-    int m_inSenItem;  // Number of senitems
+    CdcLogicVertex* m_logicVertexp = nullptr;  // Current statement being tracked, nullptr=ignored
+    AstScope* m_scopep = nullptr;  // Current scope being processed
+    AstNodeModule* m_modp = nullptr;  // Current module
+    AstSenTree* m_domainp = nullptr;  // Current sentree
+    bool m_inDly = false;  // In delayed assign
+    int m_inSenItem = 0;  // Number of senitems
     string m_ofFilename;  // Output filename
     std::ofstream* m_ofp;  // Output file
-    uint32_t m_userGeneration;  // Generation count to avoid slow userClearVertices
-    int m_filelineWidth;  // Characters in longest fileline
+    uint32_t m_userGeneration = 0;  // Generation count to avoid slow userClearVertices
+    int m_filelineWidth = 0;  // Characters in longest fileline
 
     // METHODS
     void iterateNewStmt(AstNode* nodep) {
@@ -256,9 +247,9 @@ private:
                 m_logicVertexp->dstDomainSet(true);
             }
             iterateChildren(nodep);
-            m_logicVertexp = NULL;
+            m_logicVertexp = nullptr;
 
-            if (0 && debug() >= 9) {
+            if (false && debug() >= 9) {
                 UINFO(9, "Trace Logic:\n");
                 nodep->dumpTree(cout, "-log1: ");
             }
@@ -276,7 +267,7 @@ private:
                 // Create IO vertex - note it's relative to the pointed to var, not where we are
                 // now This allows reporting to easily print the input statement
                 CdcLogicVertex* ioVertexp
-                    = new CdcLogicVertex(&m_graph, varscp->scopep(), varscp->varp(), NULL);
+                    = new CdcLogicVertex(&m_graph, varscp->scopep(), varscp->varp(), nullptr);
                 if (varscp->varp()->isWritable()) {
                     new V3GraphEdge(&m_graph, vertexp, ioVertexp, 1);
                 } else {
@@ -298,7 +289,7 @@ private:
         static bool told_file = false;
         nodep->v3warnCode(code, msg);
         if (!told_file) {
-            told_file = 1;
+            told_file = true;
             std::cerr << V3Error::msgPrefix() << "     See details in " << m_ofFilename << endl;
         }
         *m_ofp << "%Warning-" << code.ascii() << ": " << nodep->fileline() << " " << msg << endl;
@@ -367,7 +358,7 @@ private:
                     UINFO(8, "   Trace One async: " << vvertexp << endl);
                     // Twice, as we need to detect, then propagate
                     CdcEitherVertex* markp = traceAsyncRecurse(vvertexp, false);
-                    if (markp) {  // Mark is non-NULL if something bad on this path
+                    if (markp) {  // Mark is non-nullptr if something bad on this path
                         UINFO(9, "   Trace One bad! " << vvertexp << endl);
                         m_userGeneration++;  // Effectively a userClearVertices()
                         traceAsyncRecurse(vvertexp, true);
@@ -380,12 +371,12 @@ private:
     }
 
     CdcEitherVertex* traceAsyncRecurse(CdcEitherVertex* vertexp, bool mark) {
-        // First pass: Return vertex of any hazardous stuff attached, or NULL if OK
+        // First pass: Return vertex of any hazardous stuff attached, or nullptr if OK
         // If first pass returns true, second pass calls asyncPath() on appropriate nodes
-        if (vertexp->user() >= m_userGeneration) return NULL;  // Processed - prevent loop
+        if (vertexp->user() >= m_userGeneration) return nullptr;  // Processed - prevent loop
         vertexp->user(m_userGeneration);
 
-        CdcEitherVertex* mark_outp = NULL;
+        CdcEitherVertex* mark_outp = nullptr;
         UINFO(9, "      Trace: " << vertexp << endl);
 
         // Clear out in prep for marking next path
@@ -404,7 +395,7 @@ private:
                     CdcEitherVertex* eFromVertexp = static_cast<CdcEitherVertex*>(edgep->fromp());
                     eFromVertexp->asyncPath(true);
                 }
-                return NULL;
+                return nullptr;
             }
             // Also ok if from flop, but partially trace the flop so more obvious to users
             if (vvertexp->fromFlop()) {
@@ -412,7 +403,7 @@ private:
                     CdcEitherVertex* eFromVertexp = static_cast<CdcEitherVertex*>(edgep->fromp());
                     eFromVertexp->asyncPath(true);
                 }
-                return NULL;
+                return nullptr;
             }
         }
 
@@ -517,7 +508,7 @@ private:
         }
 
         string filename = v3Global.opt.makeDir() + "/" + v3Global.opt.prefix() + "__cdc_edges.txt";
-        const vl_unique_ptr<std::ofstream> ofp(V3File::new_ofstream(filename));
+        const std::unique_ptr<std::ofstream> ofp(V3File::new_ofstream(filename));
         if (ofp->fail()) v3fatal("Can't write " << filename);
         *ofp << "Edge Report for " << v3Global.opt.prefix() << endl;
 
@@ -552,9 +543,7 @@ private:
             }
         }
         stable_sort(report.begin(), report.end());
-        for (std::deque<string>::iterator it = report.begin(); it != report.end(); ++it) {
-            *ofp << *it;
-        }
+        for (const auto& line : report) *ofp << line;
     }
 
     void edgeDomainRecurse(CdcEitherVertex* vertexp, bool traceDests, int level) {
@@ -597,7 +586,7 @@ private:
         }
 
         // Convert list of senses into one sense node
-        AstSenTree* senoutp = NULL;
+        AstSenTree* senoutp = nullptr;
         bool senedited = false;
         for (SenSet::iterator it = senouts.begin(); it != senouts.end(); ++it) {
             if (!senoutp) {
@@ -636,22 +625,21 @@ private:
     }
 
     // VISITORS
-    virtual void visit(AstNodeModule* nodep) VL_OVERRIDE {
-        AstNodeModule* origModp = m_modp;
+    virtual void visit(AstNodeModule* nodep) override {
+        VL_RESTORER(m_modp);
         {
             m_modp = nodep;
             iterateChildren(nodep);
         }
-        m_modp = origModp;
     }
-    virtual void visit(AstScope* nodep) VL_OVERRIDE {
+    virtual void visit(AstScope* nodep) override {
         UINFO(4, " SCOPE " << nodep << endl);
         m_scopep = nodep;
-        m_logicVertexp = NULL;
+        m_logicVertexp = nullptr;
         iterateChildren(nodep);
-        m_scopep = NULL;
+        m_scopep = nullptr;
     }
-    virtual void visit(AstActive* nodep) VL_OVERRIDE {
+    virtual void visit(AstActive* nodep) override {
         // Create required blocks and add to module
         UINFO(4, "  BLOCK  " << nodep << endl);
         AstNode::user2ClearTree();
@@ -660,10 +648,10 @@ private:
             || m_domainp->hasClocked()) {  // IE not hasSettle/hasInitial
             iterateNewStmt(nodep);
         }
-        m_domainp = NULL;
+        m_domainp = nullptr;
         AstNode::user2ClearTree();
     }
-    virtual void visit(AstNodeVarRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeVarRef* nodep) override {
         if (m_scopep) {
             UASSERT_OBJ(m_logicVertexp, nodep, "Var ref not under a logic block");
             AstVarScope* varscp = nodep->varScopep();
@@ -691,71 +679,55 @@ private:
             }
         }
     }
-    virtual void visit(AstAssignDly* nodep) VL_OVERRIDE {
+    virtual void visit(AstAssignDly* nodep) override {
         m_inDly = true;
         iterateChildren(nodep);
         m_inDly = false;
     }
-    virtual void visit(AstSenItem* nodep) VL_OVERRIDE {
-        // Note we look at only AstSenItems, not AstSenGate's
-        // The gating term of a AstSenGate is normal logic
+    virtual void visit(AstSenItem* nodep) override {
         m_inSenItem = true;
         iterateChildren(nodep);
         m_inSenItem = false;
     }
-    virtual void visit(AstAlways* nodep) VL_OVERRIDE { iterateNewStmt(nodep); }
-    virtual void visit(AstAlwaysPublic* nodep) VL_OVERRIDE {
+    virtual void visit(AstAlways* nodep) override { iterateNewStmt(nodep); }
+    virtual void visit(AstAlwaysPublic* nodep) override {
         // CDC doesn't care about public variables
     }
-    virtual void visit(AstCFunc* nodep) VL_OVERRIDE { iterateNewStmt(nodep); }
-    virtual void visit(AstSenGate* nodep) VL_OVERRIDE {
-        // First handle the clock part will be handled in a minute by visit AstSenItem
-        // The logic gating term is dealt with as logic
-        iterateNewStmt(nodep);
-    }
-    virtual void visit(AstAssignAlias* nodep) VL_OVERRIDE { iterateNewStmt(nodep); }
-    virtual void visit(AstAssignW* nodep) VL_OVERRIDE { iterateNewStmt(nodep); }
+    virtual void visit(AstCFunc* nodep) override { iterateNewStmt(nodep); }
+    virtual void visit(AstAssignAlias* nodep) override { iterateNewStmt(nodep); }
+    virtual void visit(AstAssignW* nodep) override { iterateNewStmt(nodep); }
 
     // Math that shouldn't cause us to clear hazard
-    virtual void visit(AstConst*) VL_OVERRIDE {}
-    virtual void visit(AstReplicate* nodep) VL_OVERRIDE { iterateChildren(nodep); }
-    virtual void visit(AstConcat* nodep) VL_OVERRIDE { iterateChildren(nodep); }
-    virtual void visit(AstNot* nodep) VL_OVERRIDE { iterateChildren(nodep); }
-    virtual void visit(AstSel* nodep) VL_OVERRIDE {
+    virtual void visit(AstConst*) override {}
+    virtual void visit(AstReplicate* nodep) override { iterateChildren(nodep); }
+    virtual void visit(AstConcat* nodep) override { iterateChildren(nodep); }
+    virtual void visit(AstNot* nodep) override { iterateChildren(nodep); }
+    virtual void visit(AstSel* nodep) override {
         if (!VN_IS(nodep->lsbp(), Const)) setNodeHazard(nodep);
         iterateChildren(nodep);
     }
-    virtual void visit(AstNodeSel* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeSel* nodep) override {
         if (!VN_IS(nodep->bitp(), Const)) setNodeHazard(nodep);
         iterateChildren(nodep);
     }
 
     // Ignores
-    virtual void visit(AstInitial*) VL_OVERRIDE {}
-    virtual void visit(AstTraceInc*) VL_OVERRIDE {}
-    virtual void visit(AstCoverToggle*) VL_OVERRIDE {}
-    virtual void visit(AstNodeDType*) VL_OVERRIDE {}
+    virtual void visit(AstInitial*) override {}
+    virtual void visit(AstTraceDecl*) override {}
+    virtual void visit(AstCoverToggle*) override {}
+    virtual void visit(AstNodeDType*) override {}
 
     //--------------------
     // Default
-    virtual void visit(AstNodeMath* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeMath* nodep) override {
         setNodeHazard(nodep);
         iterateChildren(nodep);
     }
-    virtual void visit(AstNode* nodep) VL_OVERRIDE { iterateChildren(nodep); }
+    virtual void visit(AstNode* nodep) override { iterateChildren(nodep); }
 
 public:
     // CONSTRUCTORS
     explicit CdcVisitor(AstNode* nodep) {
-        m_logicVertexp = NULL;
-        m_scopep = NULL;
-        m_modp = NULL;
-        m_domainp = NULL;
-        m_inDly = false;
-        m_inSenItem = 0;
-        m_userGeneration = 0;
-        m_filelineWidth = 0;
-
         // Make report of all signal names and what clock edges they have
         string filename = v3Global.opt.makeDir() + "/" + v3Global.opt.prefix() + "__cdc.txt";
         m_ofp = V3File::new_ofstream(filename);
@@ -773,12 +745,12 @@ public:
         if (debug() >= 1) edgeReport();  // Not useful to users at the moment
         if (false) {
             *m_ofp << "\nDBG-test-dumper\n";
-            V3EmitV::verilogPrefixedTree(nodep, *m_ofp, "DBG ", 40, NULL, true);
+            V3EmitV::verilogPrefixedTree(nodep, *m_ofp, "DBG ", 40, nullptr, true);
             *m_ofp << endl;
         }
     }
-    virtual ~CdcVisitor() {
-        if (m_ofp) VL_DO_CLEAR(delete m_ofp, m_ofp = NULL);
+    virtual ~CdcVisitor() override {
+        if (m_ofp) VL_DO_CLEAR(delete m_ofp, m_ofp = nullptr);
     }
 };
 
