@@ -33,16 +33,13 @@
 #include "V3Hashed.h"
 
 #include <algorithm>
-#include <cstdarg>
-#include <iomanip>
 #include <list>
 #include <map>
-#include <vector>
-#include VL_INCLUDE_UNORDERED_SET
+#include <unordered_set>
 
 typedef std::list<AstNodeVarRef*> GateVarRefList;
 
-#define GATE_DEDUP_MAX_DEPTH 20
+constexpr int GATE_DEDUP_MAX_DEPTH = 20;
 
 //######################################################################
 
@@ -59,7 +56,7 @@ class GateGraphBaseVisitor {
 public:
     V3Graph* m_graphp;  // Graph this class is visiting
     explicit GateGraphBaseVisitor(V3Graph* graphp)
-        : m_graphp(graphp) {}
+        : m_graphp{graphp} {}
     virtual ~GateGraphBaseVisitor() {}
     virtual VNUser visit(GateLogicVertex* vertexp, VNUser vu = VNUser(0)) = 0;
     virtual VNUser visit(GateVarVertex* vertexp, VNUser vu = VNUser(0)) = 0;
@@ -71,19 +68,16 @@ public:
 
 class GateEitherVertex : public V3GraphVertex {
     AstScope* m_scopep;  // Scope vertex refers to
-    bool m_reducible;  // True if this node should be able to be eliminated
-    bool m_dedupable;  // True if this node should be able to be deduped
-    bool m_consumed;  // Output goes to something meaningful
+    bool m_reducible = true;  // True if this node should be able to be eliminated
+    bool m_dedupable = true;  // True if this node should be able to be deduped
+    bool m_consumed = false;  // Output goes to something meaningful
 public:
     GateEitherVertex(V3Graph* graphp, AstScope* scopep)
-        : V3GraphVertex(graphp)
-        , m_scopep(scopep)
-        , m_reducible(true)
-        , m_dedupable(true)
-        , m_consumed(false) {}
-    virtual ~GateEitherVertex() {}
+        : V3GraphVertex{graphp}
+        , m_scopep{scopep} {}
+    virtual ~GateEitherVertex() override {}
     // ACCESSORS
-    virtual string dotStyle() const { return m_consumed ? "" : "dotted"; }
+    virtual string dotStyle() const override { return m_consumed ? "" : "dotted"; }
     AstScope* scopep() const { return m_scopep; }
     bool reducible() const { return m_reducible; }
     bool dedupable() const { return m_dedupable; }
@@ -119,7 +113,7 @@ public:
     //          iterateInEdges() will stop traversing edges if one is deleted
     VNUser iterateCurrentOutEdges(GateGraphBaseVisitor& v, VNUser vu = VNUser(0)) {
         VNUser ret = VNUser(0);
-        V3GraphEdge* next_edgep = NULL;
+        V3GraphEdge* next_edgep = nullptr;
         for (V3GraphEdge* edgep = outBeginp(); edgep; edgep = next_edgep) {
             // Need to find the next edge before visiting in case the edge is deleted
             next_edgep = edgep->outNextp();
@@ -131,23 +125,19 @@ public:
 
 class GateVarVertex : public GateEitherVertex {
     AstVarScope* m_varScp;
-    bool m_isTop;
-    bool m_isClock;
-    AstNode* m_rstSyncNodep;  // Used as reset and not in SenItem, in clocked always
-    AstNode* m_rstAsyncNodep;  // Used as reset and in SenItem, in clocked always
+    bool m_isTop = false;
+    bool m_isClock = false;
+    AstNode* m_rstSyncNodep = nullptr;  // Used as reset and not in SenItem, in clocked always
+    AstNode* m_rstAsyncNodep = nullptr;  // Used as reset and in SenItem, in clocked always
 public:
     GateVarVertex(V3Graph* graphp, AstScope* scopep, AstVarScope* varScp)
-        : GateEitherVertex(graphp, scopep)
-        , m_varScp(varScp)
-        , m_isTop(false)
-        , m_isClock(false)
-        , m_rstSyncNodep(NULL)
-        , m_rstAsyncNodep(NULL) {}
-    virtual ~GateVarVertex() {}
+        : GateEitherVertex{graphp, scopep}
+        , m_varScp{varScp} {}
+    virtual ~GateVarVertex() override {}
     // ACCESSORS
     AstVarScope* varScp() const { return m_varScp; }
-    virtual string name() const { return (cvtToHex(m_varScp) + " " + varScp()->name()); }
-    virtual string dotColor() const { return "blue"; }
+    virtual string name() const override { return (cvtToHex(m_varScp) + " " + varScp()->name()); }
+    virtual string dotColor() const override { return "blue"; }
     bool isTop() const { return m_isTop; }
     void setIsTop() { m_isTop = true; }
     bool isClock() const { return m_isClock; }
@@ -168,29 +158,35 @@ public:
             setIsClock();
         }
     }
-    VNUser accept(GateGraphBaseVisitor& v, VNUser vu = VNUser(0)) { return v.visit(this, vu); }
+    virtual VNUser accept(GateGraphBaseVisitor& v, VNUser vu = VNUser(0)) override {
+        return v.visit(this, vu);
+    }
 };
 
 class GateLogicVertex : public GateEitherVertex {
     AstNode* m_nodep;
-    AstActive* m_activep;  // Under what active; NULL is ok (under cfunc or such)
+    AstActive* m_activep;  // Under what active; nullptr is ok (under cfunc or such)
     bool m_slow;  // In slow block
 public:
     GateLogicVertex(V3Graph* graphp, AstScope* scopep, AstNode* nodep, AstActive* activep,
                     bool slow)
-        : GateEitherVertex(graphp, scopep)
-        , m_nodep(nodep)
-        , m_activep(activep)
-        , m_slow(slow) {}
-    virtual ~GateLogicVertex() {}
+        : GateEitherVertex{graphp, scopep}
+        , m_nodep{nodep}
+        , m_activep{activep}
+        , m_slow{slow} {}
+    virtual ~GateLogicVertex() override {}
     // ACCESSORS
-    virtual string name() const { return (cvtToHex(m_nodep) + "@" + scopep()->prettyName()); }
-    virtual string dotColor() const { return "purple"; }
-    virtual FileLine* fileline() const { return nodep()->fileline(); }
+    virtual string name() const override {
+        return (cvtToHex(m_nodep) + "@" + scopep()->prettyName());
+    }
+    virtual string dotColor() const override { return "purple"; }
+    virtual FileLine* fileline() const override { return nodep()->fileline(); }
     AstNode* nodep() const { return m_nodep; }
     AstActive* activep() const { return m_activep; }
     bool slow() const { return m_slow; }
-    VNUser accept(GateGraphBaseVisitor& v, VNUser vu = VNUser(0)) { return v.visit(this, vu); }
+    virtual VNUser accept(GateGraphBaseVisitor& v, VNUser vu = VNUser(0)) override {
+        return v.visit(this, vu);
+    }
 };
 
 //######################################################################
@@ -199,14 +195,14 @@ public:
 class GateOkVisitor : public GateBaseVisitor {
 private:
     // RETURN STATE
-    bool m_isSimple;  // Set false when we know it isn't simple
+    bool m_isSimple = true;  // Set false when we know it isn't simple
     GateVarRefList m_rhsVarRefs;  // VarRefs on rhs of assignment
-    AstNode* m_substTreep;  // What to replace the variable with
+    AstNode* m_substTreep = nullptr;  // What to replace the variable with
     // STATE
     bool m_buffersOnly;  // Set when we only allow simple buffering, no equations (for clocks)
-    AstNodeVarRef* m_lhsVarRef;  // VarRef on lhs of assignment (what we're replacing)
+    AstNodeVarRef* m_lhsVarRef = nullptr;  // VarRef on lhs of assignment (what we're replacing)
     bool m_dedupe;  // Set when we use isGateDedupable instead of isGateOptimizable
-    int m_ops;  // Operation count
+    int m_ops = 0;  // Operation count
 
     // METHODS
     void clearSimple(const char* because) {
@@ -216,7 +212,7 @@ private:
         }
     }
     // VISITORS
-    virtual void visit(AstNodeVarRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeVarRef* nodep) override {
         ++m_ops;
         iterateChildren(nodep);
         // We only allow a LHS ref for the var being set, and a RHS ref for
@@ -240,7 +236,7 @@ private:
             m_rhsVarRefs.push_back(nodep);
         }
     }
-    virtual void visit(AstNodeAssign* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeAssign* nodep) override {
         m_substTreep = nodep->rhsp();
         if (!VN_IS(nodep->lhsp(), NodeVarRef)) {
             clearSimple("ASSIGN(non-VARREF)");
@@ -263,7 +259,7 @@ private:
         }
     }
     //--------------------
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         // *** Special iterator
         if (!m_isSimple) return;  // Fastpath
         if (++m_ops > v3Global.opt.gateStmts()) { clearSimple("--gate-stmts exceeded"); }
@@ -279,12 +275,8 @@ private:
 public:
     // CONSTRUCTORS
     GateOkVisitor(AstNode* nodep, bool buffersOnly, bool dedupe) {
-        m_isSimple = true;
-        m_substTreep = NULL;
         m_buffersOnly = buffersOnly;
-        m_lhsVarRef = NULL;
         m_dedupe = dedupe;
-        m_ops = 0;
         // Iterate
         iterate(nodep);
         // Check results
@@ -297,7 +289,7 @@ public:
         }
         if (debug() >= 9 && !m_isSimple) nodep->dumpTree(cout, "    gate!Ok: ");
     }
-    virtual ~GateOkVisitor() {}
+    virtual ~GateOkVisitor() override {}
     // PUBLIC METHODS
     bool isSimple() const { return m_isSimple; }
     AstNode* substTree() const { return m_substTreep; }
@@ -321,13 +313,13 @@ private:
 
     // STATE
     V3Graph m_graph;  // Scoreboard of var usages/dependencies
-    GateLogicVertex* m_logicVertexp;  // Current statement being tracked, NULL=ignored
-    AstScope* m_scopep;  // Current scope being processed
-    AstNodeModule* m_modp;  // Current module
-    AstActive* m_activep;  // Current active
-    bool m_activeReducible;  // Is activation block reducible?
-    bool m_inSenItem;  // Underneath AstSenItem; any varrefs are clocks
-    bool m_inSlow;  // Inside a slow structure
+    GateLogicVertex* m_logicVertexp = nullptr;  // Current statement being tracked, nullptr=ignored
+    AstScope* m_scopep = nullptr;  // Current scope being processed
+    AstNodeModule* m_modp = nullptr;  // Current module
+    AstActive* m_activep = nullptr;  // Current active
+    bool m_activeReducible = true;  // Is activation block reducible?
+    bool m_inSenItem = false;  // Underneath AstSenItem; any varrefs are clocks
+    bool m_inSlow = false;  // Inside a slow structure
     VDouble0 m_statSigs;  // Statistic tracking
     VDouble0 m_statRefs;  // Statistic tracking
     VDouble0 m_statDedupLogic;  // Statistic tracking
@@ -349,7 +341,7 @@ private:
             if (consumeReason) m_logicVertexp->setConsumed(consumeReason);
             if (VN_IS(nodep, SenItem)) m_logicVertexp->setConsumed("senItem");
             iterateChildren(nodep);
-            m_logicVertexp = NULL;
+            m_logicVertexp = nullptr;
         }
     }
 
@@ -388,7 +380,7 @@ private:
     void decomposeClkVectors();
 
     // VISITORS
-    virtual void visit(AstNetlist* nodep) VL_OVERRIDE {
+    virtual void visit(AstNetlist* nodep) override {
         iterateChildren(nodep);
         // if (debug() > 6) m_graph.dump();
         if (debug() > 6) m_graph.dumpDotFilePrefixed("gate_pre");
@@ -419,23 +411,22 @@ private:
         consumedMove();
         replaceAssigns();
     }
-    virtual void visit(AstNodeModule* nodep) VL_OVERRIDE {
-        AstNodeModule* origModp = m_modp;
+    virtual void visit(AstNodeModule* nodep) override {
+        VL_RESTORER(m_modp);
         {
             m_modp = nodep;
             m_activeReducible = true;
             iterateChildren(nodep);
         }
-        m_modp = origModp;
     }
-    virtual void visit(AstScope* nodep) VL_OVERRIDE {
+    virtual void visit(AstScope* nodep) override {
         UINFO(4, " SCOPE " << nodep << endl);
         m_scopep = nodep;
-        m_logicVertexp = NULL;
+        m_logicVertexp = nullptr;
         iterateChildren(nodep);
-        m_scopep = NULL;
+        m_scopep = nullptr;
     }
-    virtual void visit(AstActive* nodep) VL_OVERRIDE {
+    virtual void visit(AstActive* nodep) override {
         // Create required blocks and add to module
         UINFO(4, "  BLOCK  " << nodep << endl);
         m_activeReducible = !(nodep->hasClocked());  // Seq logic outputs aren't reducible
@@ -443,10 +434,10 @@ private:
         AstNode::user2ClearTree();
         iterateChildren(nodep);
         AstNode::user2ClearTree();
-        m_activep = NULL;
+        m_activep = nullptr;
         m_activeReducible = true;
     }
-    virtual void visit(AstNodeVarRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeVarRef* nodep) override {
         if (m_scopep) {
             UASSERT_OBJ(m_logicVertexp, nodep, "Var ref not under a logic block");
             AstVarScope* varscp = nodep->varScopep();
@@ -473,56 +464,53 @@ private:
             }
         }
     }
-    virtual void visit(AstAlways* nodep) VL_OVERRIDE {
-        iterateNewStmt(nodep, (nodep->isJustOneBodyStmt() ? NULL : "Multiple Stmts"), NULL);
+    virtual void visit(AstAlways* nodep) override {
+        iterateNewStmt(nodep, (nodep->isJustOneBodyStmt() ? nullptr : "Multiple Stmts"), nullptr);
     }
-    virtual void visit(AstAlwaysPublic* nodep) VL_OVERRIDE {
-        bool lastslow = m_inSlow;
-        m_inSlow = true;
-        iterateNewStmt(nodep, "AlwaysPublic", NULL);
-        m_inSlow = lastslow;
+    virtual void visit(AstAlwaysPublic* nodep) override {
+        VL_RESTORER(m_inSlow);
+        {
+            m_inSlow = true;
+            iterateNewStmt(nodep, "AlwaysPublic", nullptr);
+        }
     }
-    virtual void visit(AstCFunc* nodep) VL_OVERRIDE {
+    virtual void visit(AstCFunc* nodep) override {
         iterateNewStmt(nodep, "User C Function", "User C Function");
     }
-    virtual void visit(AstSenItem* nodep) VL_OVERRIDE {
-        // Note we look at only AstSenItems, not AstSenGate's
-        // The gating term of a AstSenGate is normal logic
+    virtual void visit(AstSenItem* nodep) override {
         m_inSenItem = true;
         if (m_logicVertexp) {  // Already under logic; presumably a SenGate
             iterateChildren(nodep);
         } else {  // Standalone item, probably right under a SenTree
-            iterateNewStmt(nodep, NULL, NULL);
+            iterateNewStmt(nodep, nullptr, nullptr);
         }
         m_inSenItem = false;
     }
-    virtual void visit(AstSenGate* nodep) VL_OVERRIDE {
-        // First handle the clock part will be handled in a minute by visit AstSenItem
-        // The logic gating term is dealt with as logic
-        iterateNewStmt(nodep, "Clock gater", "Clock gater");
+    virtual void visit(AstInitial* nodep) override {
+        VL_RESTORER(m_inSlow);
+        {
+            m_inSlow = true;
+            iterateNewStmt(nodep, (nodep->isJustOneBodyStmt() ? nullptr : "Multiple Stmts"),
+                           nullptr);
+        }
     }
-    virtual void visit(AstInitial* nodep) VL_OVERRIDE {
-        bool lastslow = m_inSlow;
-        m_inSlow = true;
-        iterateNewStmt(nodep, (nodep->isJustOneBodyStmt() ? NULL : "Multiple Stmts"), NULL);
-        m_inSlow = lastslow;
+    virtual void visit(AstAssignAlias* nodep) override {  //
+        iterateNewStmt(nodep, nullptr, nullptr);
     }
-    virtual void visit(AstAssignAlias* nodep) VL_OVERRIDE {  //
-        iterateNewStmt(nodep, NULL, NULL);
+    virtual void visit(AstAssignW* nodep) override {  //
+        iterateNewStmt(nodep, nullptr, nullptr);
     }
-    virtual void visit(AstAssignW* nodep) VL_OVERRIDE {  //
-        iterateNewStmt(nodep, NULL, NULL);
-    }
-    virtual void visit(AstCoverToggle* nodep) VL_OVERRIDE {
+    virtual void visit(AstCoverToggle* nodep) override {
         iterateNewStmt(nodep, "CoverToggle", "CoverToggle");
     }
-    virtual void visit(AstTraceInc* nodep) VL_OVERRIDE {
-        bool lastslow = m_inSlow;
-        m_inSlow = true;
-        iterateNewStmt(nodep, "Tracing", "Tracing");
-        m_inSlow = lastslow;
+    virtual void visit(AstTraceDecl* nodep) override {
+        VL_RESTORER(m_inSlow);
+        {
+            m_inSlow = true;
+            iterateNewStmt(nodep, "Tracing", "Tracing");
+        }
     }
-    virtual void visit(AstConcat* nodep) VL_OVERRIDE {
+    virtual void visit(AstConcat* nodep) override {
         UASSERT_OBJ(!(VN_IS(nodep->backp(), NodeAssign)
                       && VN_CAST(nodep->backp(), NodeAssign)->lhsp() == nodep),
                     nodep, "Concat on LHS of assignment; V3Const should have deleted it");
@@ -530,7 +518,7 @@ private:
     }
 
     //--------------------
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         iterateChildren(nodep);
         if (nodep->isOutputter() && m_logicVertexp) m_logicVertexp->setConsumed("outputter");
     }
@@ -539,16 +527,9 @@ public:
     // CONSTRUCTORS
     explicit GateVisitor(AstNode* nodep) {
         AstNode::user1ClearTree();
-        m_logicVertexp = NULL;
-        m_scopep = NULL;
-        m_modp = NULL;
-        m_activep = NULL;
-        m_activeReducible = true;
-        m_inSenItem = false;
-        m_inSlow = false;
         iterate(nodep);
     }
-    virtual ~GateVisitor() {
+    virtual ~GateVisitor() override {
         V3Stats::addStat("Optimizations, Gate sigs deleted", m_statSigs);
         V3Stats::addStat("Optimizations, Gate inputs replaced", m_statRefs);
         V3Stats::addStat("Optimizations, Gate sigs deduped", m_statDedupLogic);
@@ -685,7 +666,7 @@ void GateVisitor::optimizeSignals(bool allowMultiIn) {
                             // optimize it further
                             logicp->unlinkFrBack();
                             vvertexp->varScp()->valuep(logicp);
-                            logicp = NULL;
+                            logicp = nullptr;
                             // Mark the vertex so we don't mark it as being
                             // unconsumed in the next step
                             vvertexp->user(true);
@@ -703,7 +684,7 @@ bool GateVisitor::elimLogicOkOutputs(GateLogicVertex* consumeVertexp,
     // Return true if can optimize
     // Return false if the consuming logic has an output signal that the
     // replacement logic has as an input
-    typedef vl_unordered_set<AstVarScope*> VarScopeSet;
+    typedef std::unordered_set<AstVarScope*> VarScopeSet;
     // Use map to find duplicates between two lists
     VarScopeSet varscopes;
     // Replacement logic usually has shorter input list, so faster to build list based on it
@@ -797,7 +778,7 @@ void GateVisitor::consumedMove() {
         }
         if (GateLogicVertex* lvertexp = dynamic_cast<GateLogicVertex*>(vertexp)) {
             AstNode* nodep = lvertexp->nodep();
-            AstActive* oldactp = lvertexp->activep();  // NULL under cfunc
+            AstActive* oldactp = lvertexp->activep();  // nullptr under cfunc
             if (!lvertexp->consumed() && oldactp) {
                 // Eventually: Move the statement to a new active block
                 // with "tracing-on" sensitivity
@@ -858,7 +839,7 @@ private:
     void hashReplace(AstNode* oldp, AstNode* newp);
 
     // VISITORS
-    virtual void visit(AstNodeVarRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeVarRef* nodep) override {
         if (nodep->varScopep() == m_elimVarScp) {
             // Substitute in the new tree
             // It's possible we substitute into something that will be reduced more later,
@@ -884,11 +865,11 @@ private:
             VL_DO_DANGLING(nodep->deleteTree(), nodep);
         }
     }
-    virtual void visit(AstNode* nodep) VL_OVERRIDE { iterateChildren(nodep); }
+    virtual void visit(AstNode* nodep) override { iterateChildren(nodep); }
 
 public:
     // CONSTRUCTORS
-    virtual ~GateElimVisitor() {}
+    virtual ~GateElimVisitor() override {}
     GateElimVisitor(AstNode* nodep, AstVarScope* varscp, AstNode* replaceTreep,
                     GateDedupeVarVisitor* varVisp) {
         UINFO(9, "     elimvisitor " << nodep << endl);
@@ -905,7 +886,7 @@ public:
 
 void GateVisitor::optimizeElimVar(AstVarScope* varscp, AstNode* substp, AstNode* consumerp) {
     if (debug() >= 5) consumerp->dumpTree(cout, "    elimUsePre: ");
-    GateElimVisitor elimVisitor(consumerp, varscp, substp, NULL);
+    GateElimVisitor elimVisitor(consumerp, varscp, substp, nullptr);
     if (elimVisitor.didReplace()) {
         if (debug() >= 9) consumerp->dumpTree(cout, "    elimUseCns: ");
         // Caution: Can't let V3Const change our handle to consumerp, such as by
@@ -930,9 +911,9 @@ private:
     // NODE STATE
     // Ast*::user2p     -> parent AstNodeAssign* for this rhsp
     // Ast*::user3p     -> AstActive* of assign, for isSame() in test for duplicate
-    //                     Set to NULL if this assign's tree was later replaced
+    //                     Set to nullptr if this assign's tree was later replaced
     // Ast*::user5p     -> AstNode* of assign if condition, for isSame() in test for duplicate
-    //                     Set to NULL if this assign's tree was later replaced
+    //                     Set to nullptr if this assign's tree was later replaced
     // AstUser1InUse    m_inuser1;      (Allocated for use in GateVisitor)
     // AstUser2InUse    m_inuser2;      (Allocated for use in GateVisitor)
     AstUser3InUse m_inuser3;
@@ -945,7 +926,7 @@ private:
     VL_DEBUG_FUNC;  // Declare debug()
 
     void hash(AstNode* nodep) {
-        // !NULL && the object is hashable
+        // !nullptr && the object is hashable
         if (nodep && !nodep->sameHash().isIllegal()) m_hashed.hash(nodep);
     }
     bool sameHash(AstNode* node1p, AstNode* node2p) {
@@ -995,7 +976,8 @@ public:
     }
 
     AstNodeAssign* hashAndFindDupe(AstNodeAssign* assignp, AstNode* extra1p, AstNode* extra2p) {
-        // Legal for extra1p/2p to be NULL, we'll compare with other assigns with extras also NULL
+        // Legal for extra1p/2p to be nullptr, we'll compare with other assigns with extras also
+        // nullptr
         AstNode* rhsp = assignp->rhsp();
         rhsp->user2p(assignp);
         rhsp->user3p(extra1p);
@@ -1004,8 +986,8 @@ public:
         hash(extra1p);
         hash(extra2p);
 
-        V3Hashed::iterator inserted = m_hashed.hashAndInsert(rhsp);
-        V3Hashed::iterator dupit = m_hashed.findDuplicate(rhsp, this);
+        const auto inserted = m_hashed.hashAndInsert(rhsp);
+        const auto dupit = m_hashed.findDuplicate(rhsp, this);
         // Even though rhsp was just inserted, V3Hashed::findDuplicate doesn't
         // return anything in the hash that has the same pointer (V3Hashed.cpp::findDuplicate)
         // So dupit is either a different, duplicate rhsp, or the end of the hash.
@@ -1014,7 +996,7 @@ public:
             return VN_CAST(m_hashed.iteratorNodep(dupit)->user2p(), NodeAssign);
         }
         // Retain new inserted information
-        return NULL;
+        return nullptr;
     }
 
     void check() {
@@ -1052,13 +1034,13 @@ class GateDedupeVarVisitor : public GateBaseVisitor {
 private:
     // STATE
     GateDedupeHash m_ghash;  // Hash used to find dupes of rhs of assign
-    AstNodeAssign* m_assignp;  // Assign found for dedupe
-    AstNode* m_ifCondp;  // IF condition that assign is under
-    bool m_always;  // Assign is under an always
-    bool m_dedupable;  // Determined the assign to be dedupable
+    AstNodeAssign* m_assignp = nullptr;  // Assign found for dedupe
+    AstNode* m_ifCondp = nullptr;  // IF condition that assign is under
+    bool m_always = false;  // Assign is under an always
+    bool m_dedupable = true;  // Determined the assign to be dedupable
 
     // VISITORS
-    virtual void visit(AstNodeAssign* assignp) VL_OVERRIDE {
+    virtual void visit(AstNodeAssign* assignp) override {
         if (m_dedupable) {
             // I think we could safely dedupe an always block with multiple
             // non-blocking statements, but erring on side of caution here
@@ -1069,7 +1051,7 @@ private:
             }
         }
     }
-    virtual void visit(AstAlways* alwaysp) VL_OVERRIDE {
+    virtual void visit(AstAlways* alwaysp) override {
         if (m_dedupable) {
             if (!m_always) {
                 m_always = true;
@@ -1083,7 +1065,7 @@ private:
     //  always @(...)
     //    if (...)
     //       foo = ...; // or foo <= ...;
-    virtual void visit(AstNodeIf* ifp) VL_OVERRIDE {
+    virtual void visit(AstNodeIf* ifp) override {
         if (m_dedupable) {
             if (m_always && !m_ifCondp && !ifp->elsesp()) {
                 // we're under an always, this is the first IF, and there's no else
@@ -1095,25 +1077,20 @@ private:
         }
     }
 
-    virtual void visit(AstComment*) VL_OVERRIDE {}  // NOP
+    virtual void visit(AstComment*) override {}  // NOP
     //--------------------
-    virtual void visit(AstNode*) VL_OVERRIDE {  //
+    virtual void visit(AstNode*) override {  //
         m_dedupable = false;
     }
 
 public:
     // CONSTRUCTORS
-    GateDedupeVarVisitor() {
-        m_assignp = NULL;
-        m_ifCondp = NULL;
-        m_always = false;
-        m_dedupable = true;
-    }
+    GateDedupeVarVisitor() {}
     ~GateDedupeVarVisitor() {}
     // PUBLIC METHODS
     AstNodeVarRef* findDupe(AstNode* nodep, AstVarScope* consumerVarScopep, AstActive* activep) {
-        m_assignp = NULL;
-        m_ifCondp = NULL;
+        m_assignp = nullptr;
+        m_ifCondp = nullptr;
         m_always = false;
         m_dedupable = true;
         iterate(nodep);
@@ -1128,7 +1105,7 @@ public:
                 }
             }
         }
-        return NULL;
+        return nullptr;
     }
     void hashReplace(AstNode* oldp, AstNode* newp) { m_ghash.hashReplace(oldp, newp); }
 };
@@ -1150,9 +1127,9 @@ private:
     // AstUser2InUse            m_inuser2;      (Allocated for use in GateVisitor)
     VDouble0 m_numDeduped;  // Statistic tracking
     GateDedupeVarVisitor m_varVisitor;  // Looks for a dupe of the logic
-    int m_depth;  // Iteration depth
+    int m_depth = 0;  // Iteration depth
 
-    virtual VNUser visit(GateVarVertex* vvertexp, VNUser) {
+    virtual VNUser visit(GateVarVertex* vvertexp, VNUser) override {
         // Check that we haven't been here before
         if (m_depth > GATE_DEDUP_MAX_DEPTH)
             return VNUser(0);  // Break loops; before user2 set so hit this vertex later
@@ -1167,7 +1144,7 @@ private:
                 V3GraphEdge* edgep = vvertexp->inBeginp();
                 GateLogicVertex* lvertexp = static_cast<GateLogicVertex*>(edgep->fromp());
                 UASSERT_OBJ(vvertexp->dedupable(), vvertexp->varScp(),
-                            "GateLogicVertex* visit should have returned NULL "
+                            "GateLogicVertex* visit should have returned nullptr "
                             "if consumer var vertex is not dedupable.");
                 GateOkVisitor okVisitor(lvertexp->nodep(), false, true);
                 if (okVisitor.isSimple()) {
@@ -1181,7 +1158,7 @@ private:
                         GateLogicVertex* consumeVertexp
                             = dynamic_cast<GateLogicVertex*>(outedgep->top());
                         AstNode* consumerp = consumeVertexp->nodep();
-                        m_graphp->dumpDotFilePrefixed("gate_preelim");
+                        // if (debug() >= 9) m_graphp->dumpDotFilePrefixed("gate_preelim");
                         UINFO(9,
                               "elim src vtx" << lvertexp << " node " << lvertexp->nodep() << endl);
                         UINFO(9,
@@ -1207,7 +1184,7 @@ private:
                     AstNode* lvertexNodep = lvertexp->nodep();
                     lvertexNodep->unlinkFrBack();
                     vvertexp->varScp()->valuep(lvertexNodep);
-                    lvertexNodep = NULL;
+                    lvertexNodep = nullptr;
                     vvertexp->user(true);
                     lvertexp->user(true);
                 }
@@ -1218,8 +1195,8 @@ private:
     }
 
     // Given iterated logic, starting at vu which was consumer's GateVarVertex
-    // Returns a varref that has the same logic input; or NULL if none
-    virtual VNUser visit(GateLogicVertex* lvertexp, VNUser vu) {
+    // Returns a varref that has the same logic input; or nullptr if none
+    virtual VNUser visit(GateLogicVertex* lvertexp, VNUser vu) override {
         lvertexp->iterateInEdges(*this);
 
         GateVarVertex* consumerVvertexpp = static_cast<GateVarVertex*>(vu.toGraphVertex());
@@ -1238,8 +1215,7 @@ private:
 
 public:
     explicit GateDedupeGraphVisitor(V3Graph* graphp)
-        : GateGraphBaseVisitor(graphp)
-        , m_depth(0) {}
+        : GateGraphBaseVisitor{graphp} {}
     void dedupeTree(GateVarVertex* vvertexp) { vvertexp->accept(*this); }
     VDouble0 numDeduped() { return m_numDeduped; }
 };
@@ -1274,9 +1250,9 @@ void GateVisitor::dedupe() {
 class GateMergeAssignsGraphVisitor : public GateGraphBaseVisitor {
 private:
     // NODE STATE
-    AstNodeAssign* m_assignp;
-    AstActive* m_activep;
-    GateLogicVertex* m_logicvp;
+    AstNodeAssign* m_assignp = nullptr;
+    AstActive* m_activep = nullptr;
+    GateLogicVertex* m_logicvp = nullptr;
     VDouble0 m_numMergedAssigns;  // Statistic tracking
 
     // assemble two Sel into one if possible
@@ -1284,22 +1260,22 @@ private:
         AstVarRef* preVarRefp = VN_CAST(pre->fromp(), VarRef);
         AstVarRef* curVarRefp = VN_CAST(cur->fromp(), VarRef);
         if (!preVarRefp || !curVarRefp || !curVarRefp->same(preVarRefp)) {
-            return NULL;  // not the same var
+            return nullptr;  // not the same var
         }
         const AstConst* pstart = VN_CAST(pre->lsbp(), Const);
         const AstConst* pwidth = VN_CAST(pre->widthp(), Const);
         const AstConst* cstart = VN_CAST(cur->lsbp(), Const);
         const AstConst* cwidth = VN_CAST(cur->widthp(), Const);
-        if (!pstart || !pwidth || !cstart || !cwidth) return NULL;  // too complicated
+        if (!pstart || !pwidth || !cstart || !cwidth) return nullptr;  // too complicated
         if (cur->lsbConst() + cur->widthConst() == pre->lsbConst()) {
             return new AstSel(curVarRefp->fileline(), curVarRefp->cloneTree(false),
                               cur->lsbConst(), pre->widthConst() + cur->widthConst());
         } else {
-            return NULL;
+            return nullptr;
         }
     }
 
-    virtual VNUser visit(GateVarVertex* vvertexp, VNUser) {
+    virtual VNUser visit(GateVarVertex* vvertexp, VNUser) override {
         for (V3GraphEdge* edgep = vvertexp->inBeginp(); edgep;) {
             V3GraphEdge* oldedgep = edgep;
             edgep = edgep->inNextp();  // for recursive since the edge could be deleted
@@ -1372,18 +1348,14 @@ private:
         }
         return VNUser(0);
     }
-    virtual VNUser visit(GateLogicVertex* lvertexp, VNUser vu) {  //
+    virtual VNUser visit(GateLogicVertex*, VNUser vu) override {  //
         return VNUser(0);
     }
 
 public:
     explicit GateMergeAssignsGraphVisitor(V3Graph* graphp)
-        : GateGraphBaseVisitor(graphp) {
-        m_assignp = NULL;
-        m_activep = NULL;
-        m_logicvp = NULL;
-        m_numMergedAssigns = 0;
-        m_graphp = graphp;
+        : GateGraphBaseVisitor{graphp} {
+        m_graphp = graphp;  // In base
     }
     void mergeAssignsTree(GateVarVertex* vvertexp) { vvertexp->accept(*this); }
     VDouble0 numMergedAssigns() { return m_numMergedAssigns; }
@@ -1408,13 +1380,13 @@ void GateVisitor::mergeAssigns() {
 class GateConcatVisitor : public GateBaseVisitor {
 private:
     // STATE
-    AstVarScope* m_vscp;  // Varscope we're trying to find
-    int m_offset;  // Current offset of varscope
-    int m_found_offset;  // Found offset of varscope
-    bool m_found;  // Offset found
+    AstVarScope* m_vscp = nullptr;  // Varscope we're trying to find
+    int m_offset = 0;  // Current offset of varscope
+    int m_found_offset = 0;  // Found offset of varscope
+    bool m_found = false;  // Offset found
 
     // VISITORS
-    virtual void visit(AstNodeVarRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeVarRef* nodep) override {
         UINFO(9, "CLK DECOMP Concat search var (off = " << m_offset << ") - " << nodep << endl);
         if (nodep->varScopep() == m_vscp && !nodep->user2() && !m_found) {
             // A concatenation may use the same var multiple times
@@ -1426,23 +1398,18 @@ private:
         }
         m_offset += nodep->dtypep()->width();
     }
-    virtual void visit(AstConcat* nodep) VL_OVERRIDE {
+    virtual void visit(AstConcat* nodep) override {
         UINFO(9, "CLK DECOMP Concat search (off = " << m_offset << ") - " << nodep << endl);
         iterate(nodep->rhsp());
         iterate(nodep->lhsp());
     }
     //--------------------
-    virtual void visit(AstNode* nodep) VL_OVERRIDE { iterateChildren(nodep); }
+    virtual void visit(AstNode* nodep) override { iterateChildren(nodep); }
 
 public:
     // CONSTRUCTORS
-    GateConcatVisitor() {
-        m_vscp = NULL;
-        m_offset = 0;
-        m_found_offset = 0;
-        m_found = false;
-    }
-    virtual ~GateConcatVisitor() {}
+    GateConcatVisitor() {}
+    virtual ~GateConcatVisitor() override {}
     // PUBLIC METHODS
     bool concatOffset(AstConcat* concatp, AstVarScope* vscp, int& offsetr) {
         m_vscp = vscp;
@@ -1465,10 +1432,9 @@ class GateClkDecompState {
 public:
     int m_offset;
     AstVarScope* m_last_vsp;
-    GateClkDecompState(int offset, AstVarScope* vsp) {
-        m_offset = offset;
-        m_last_vsp = vsp;
-    }
+    GateClkDecompState(int offset, AstVarScope* vsp)
+        : m_offset{offset}
+        , m_last_vsp{vsp} {}
     virtual ~GateClkDecompState() {}
 };
 
@@ -1476,14 +1442,14 @@ class GateClkDecompGraphVisitor : public GateGraphBaseVisitor {
 private:
     // NODE STATE
     // AstVarScope::user2p      -> bool: already visited
-    int m_seen_clk_vectors;
-    AstVarScope* m_clk_vsp;
-    GateVarVertex* m_clk_vvertexp;
+    int m_seen_clk_vectors = 0;
+    AstVarScope* m_clk_vsp = nullptr;
+    GateVarVertex* m_clk_vvertexp = nullptr;
     GateConcatVisitor m_concat_visitor;
-    int m_total_seen_clk_vectors;
-    int m_total_decomposed_clk_vectors;
+    int m_total_seen_clk_vectors = 0;
+    int m_total_decomposed_clk_vectors = 0;
 
-    virtual VNUser visit(GateVarVertex* vvertexp, VNUser vu) {
+    virtual VNUser visit(GateVarVertex* vvertexp, VNUser vu) override {
         // Check that we haven't been here before
         AstVarScope* vsp = vvertexp->varScp();
         if (vsp->user2SetOnce()) return VNUser(0);
@@ -1500,7 +1466,7 @@ private:
         return VNUser(0);  // Unused
     }
 
-    virtual VNUser visit(GateLogicVertex* lvertexp, VNUser vu) {
+    virtual VNUser visit(GateLogicVertex* lvertexp, VNUser vu) override {
         GateClkDecompState* currState = reinterpret_cast<GateClkDecompState*>(vu.c());
         int clk_offset = currState->m_offset;
         if (const AstAssignW* assignp = VN_CAST(lvertexp->nodep(), AssignW)) {
@@ -1549,7 +1515,7 @@ private:
                     UINFO(9, "                   to - " << m_clk_vsp << endl);
                     AstNode* rhsp = assignp->rhsp();
                     rhsp->replaceWith(new AstVarRef(rhsp->fileline(), m_clk_vsp, false));
-                    for (V3GraphEdge* edgep = lvertexp->inBeginp(); edgep;) {
+                    while (V3GraphEdge* edgep = lvertexp->inBeginp()) {
                         VL_DO_DANGLING(edgep->unlinkDelete(), edgep);
                     }
                     new V3GraphEdge(m_graphp, m_clk_vvertexp, lvertexp, 1);
@@ -1566,14 +1532,8 @@ private:
 
 public:
     explicit GateClkDecompGraphVisitor(V3Graph* graphp)
-        : GateGraphBaseVisitor(graphp) {
-        m_seen_clk_vectors = 0;
-        m_clk_vsp = NULL;
-        m_clk_vvertexp = NULL;
-        m_total_seen_clk_vectors = 0;
-        m_total_decomposed_clk_vectors = 0;
-    }
-    virtual ~GateClkDecompGraphVisitor() {
+        : GateGraphBaseVisitor{graphp} {}
+    virtual ~GateClkDecompGraphVisitor() override {
         V3Stats::addStat("Optimizations, Clocker seen vectors", m_total_seen_clk_vectors);
         V3Stats::addStat("Optimizations, Clocker decomposed vectors",
                          m_total_decomposed_clk_vectors);
@@ -1613,7 +1573,7 @@ void GateVisitor::decomposeClkVectors() {
 class GateDeassignVisitor : public GateBaseVisitor {
 private:
     // VISITORS
-    virtual void visit(AstVarScope* nodep) VL_OVERRIDE {
+    virtual void visit(AstVarScope* nodep) override {
         if (AstNodeAssign* assp = VN_CAST(nodep->valuep(), NodeAssign)) {
             UINFO(5, " Removeassign " << assp << endl);
             AstNode* valuep = assp->rhsp();
@@ -1623,14 +1583,14 @@ private:
         }
     }
     // Speedups
-    virtual void visit(AstVar*) VL_OVERRIDE {}  // Accelerate
-    virtual void visit(AstActive*) VL_OVERRIDE {}  // Accelerate
-    virtual void visit(AstNode* nodep) VL_OVERRIDE { iterateChildren(nodep); }
+    virtual void visit(AstVar*) override {}  // Accelerate
+    virtual void visit(AstActive*) override {}  // Accelerate
+    virtual void visit(AstNode* nodep) override { iterateChildren(nodep); }
 
 public:
     // CONSTRUCTORS
     explicit GateDeassignVisitor(AstNode* nodep) { iterate(nodep); }
-    virtual ~GateDeassignVisitor() {}
+    virtual ~GateDeassignVisitor() override {}
 };
 
 //######################################################################
