@@ -31,25 +31,47 @@
 #include "V3Options.h"
 
 #include <string>
+#include <unordered_map>
 
 class AstNetlist;
+class V3HierBlockPlan;
 
 //======================================================================
-// Statics
+// Restorer
+
+/// Save a given variable's value on the stack, restoring it at
+/// end-of-stope.
+// Object must be named, or it will not persist until end-of-scope.
+// Constructor needs () or GCC 4.8 false warning.
+#define VL_RESTORER(var) const VRestorer<decltype(var)> restorer_##var(var);
+
+// Object used by VL_RESTORER.  This object must be an auto variable, not
+// allocated on the heap or otherwise.
+template <typename T> class VRestorer {
+    T& m_ref;  // Reference to object we're saving and restoring
+    const T m_saved;  // Value saved, for later restore
+
+public:
+    VRestorer(T& permr)
+        : m_ref(permr)
+        , m_saved(permr) {}
+    ~VRestorer() { m_ref = m_saved; }
+    VL_UNCOPYABLE(VRestorer);
+};
 
 //######################################################################
 
 class VWidthMinUsage {
 public:
-    enum en { LINT_WIDTH, MATCHES_WIDTH, VERILOG_WIDTH };
+    enum en : uint8_t { LINT_WIDTH, MATCHES_WIDTH, VERILOG_WIDTH };
     enum en m_e;
     inline VWidthMinUsage()
-        : m_e(LINT_WIDTH) {}
+        : m_e{LINT_WIDTH} {}
     // cppcheck-suppress noExplicitConstructor
     inline VWidthMinUsage(en _e)
-        : m_e(_e) {}
+        : m_e{_e} {}
     explicit inline VWidthMinUsage(int _e)
-        : m_e(static_cast<en>(_e)) {}
+        : m_e(static_cast<en>(_e)) {}  // Need () or GCC 4.8 false warning
     operator en() const { return m_e; }
 };
 inline bool operator==(const VWidthMinUsage& lhs, const VWidthMinUsage& rhs) {
@@ -68,40 +90,36 @@ inline bool operator==(VWidthMinUsage::en lhs, const VWidthMinUsage& rhs) {
 class V3Global {
     // Globals
     AstNetlist* m_rootp;  // Root of entire netlist
+    V3HierBlockPlan* m_hierPlanp;  // Hierarchical verilation plan, nullptr unless hier_block
     VWidthMinUsage m_widthMinUsage;  // What AstNode::widthMin() is used for
 
-    int m_debugFileNumber;  // Number to append to debug files created
-    bool m_assertDTypesResolved;  // Tree should have dtypep()'s
-    bool m_constRemoveXs;  // Const needs to strip any Xs
-    bool m_needC11;  // Need C++11
-    bool m_needHInlines;  // Need __Inlines file
-    bool m_needHeavy;  // Need verilated_heavy.h include
-    bool m_needTraceDumper;  // Need __Vm_dumperp in symbols
-    bool m_dpi;  // Need __Dpi include files
+    int m_debugFileNumber = 0;  // Number to append to debug files created
+    bool m_assertDTypesResolved = false;  // Tree should have dtypep()'s
+    bool m_constRemoveXs = false;  // Const needs to strip any Xs
+    bool m_needHeavy = false;  // Need verilated_heavy.h include
+    bool m_needTraceDumper = false;  // Need __Vm_dumperp in symbols
+    bool m_dpi = false;  // Need __Dpi include files
+    bool m_useParallelBuild = false;  // Use parallel build for model
+
+    // Memory address to short string mapping (for debug)
+    typedef std::unordered_map<const void*, std::string> PtrToIdMap;  // The map type
+    PtrToIdMap m_ptrToId;  // The actual 'address' <=> 'short string' bijection
 
 public:
     // Options
     V3Options opt;  // All options; let user see them directly
 
-public:
     // CONSTRUCTORS
     V3Global()
-        : m_rootp(NULL)  // created by makeInitNetlist() so static constructors run first
-        , m_widthMinUsage(VWidthMinUsage::LINT_WIDTH)
-        , m_debugFileNumber(0)
-        , m_assertDTypesResolved(false)
-        , m_constRemoveXs(false)
-        , m_needC11(false)
-        , m_needHInlines(false)
-        , m_needHeavy(false)
-        , m_needTraceDumper(false)
-        , m_dpi(false) {}
+        : m_rootp{nullptr}  // created by makeInitNetlist(} so static constructors run first
+        , m_hierPlanp{nullptr}  // Set via hierPlanp(V3HierBlockPlan*} when use hier_block
+        , m_widthMinUsage{VWidthMinUsage::LINT_WIDTH} {}
     AstNetlist* makeNetlist();
     void boot() {
         UASSERT(!m_rootp, "call once");
         m_rootp = makeNetlist();
     }
-    void clear();
+    void shutdown();  // Release allocated resorces
     // ACCESSORS (general)
     AstNetlist* rootp() const { return m_rootp; }
     VWidthMinUsage widthMinUsage() const { return m_widthMinUsage; }
@@ -109,7 +127,7 @@ public:
 
     // METHODS
     void readFiles();
-    void checkTree();
+    void checkTree() const;
     static void dumpCheckGlobalTree(const string& stagename, int newNumber = 0,
                                     bool doDump = true);
     void assertDTypesResolved(bool flag) { m_assertDTypesResolved = flag; }
@@ -121,18 +139,22 @@ public:
         if (newNumber) m_debugFileNumber = newNumber;
         char digits[100];
         sprintf(digits, "%03d", m_debugFileNumber);
-        return opt.makeDir() + "/" + opt.prefix() + "_" + digits + "_" + nameComment;
+        return opt.hierTopDataDir() + "/" + opt.prefix() + "_" + digits + "_" + nameComment;
     }
-    bool needC11() const { return m_needC11; }
-    void needC11(bool flag) { m_needC11 = flag; }
-    bool needHInlines() const { return m_needHInlines; }
-    void needHInlines(bool flag) { m_needHInlines = flag; }
     bool needHeavy() const { return m_needHeavy; }
     void needHeavy(bool flag) { m_needHeavy = flag; }
     bool needTraceDumper() const { return m_needTraceDumper; }
     void needTraceDumper(bool flag) { m_needTraceDumper = flag; }
     bool dpi() const { return m_dpi; }
     void dpi(bool flag) { m_dpi = flag; }
+    V3HierBlockPlan* hierPlanp() const { return m_hierPlanp; }
+    void hierPlanp(V3HierBlockPlan* plan) {
+        UASSERT(!m_hierPlanp, "call once");
+        m_hierPlanp = plan;
+    }
+    void useParallelBuild(bool flag) { m_useParallelBuild = flag; }
+    bool useParallelBuild() const { return m_useParallelBuild; }
+    const std::string& ptrToId(const void* p);
 };
 
 extern V3Global v3Global;
