@@ -21,7 +21,6 @@
 #include "V3Graph.h"
 
 #include <algorithm>
-#include <cstdarg>
 #include <list>
 #include <vector>
 
@@ -36,23 +35,20 @@ class GraphAcycVertex : public V3GraphVertex {
 protected:
     friend class GraphAcyc;
     V3ListEnt<GraphAcycVertex*> m_work;  // List of vertices with optimization work left
-    uint32_t m_storedRank;  // Rank held until commit to edge placement
-    bool m_onWorkList;  // True if already on list of work to do
-    bool m_deleted;  // True if deleted
+    uint32_t m_storedRank = 0;  // Rank held until commit to edge placement
+    bool m_onWorkList = false;  // True if already on list of work to do
+    bool m_deleted = false;  // True if deleted
 public:
     GraphAcycVertex(V3Graph* graphp, V3GraphVertex* origVertexp)
-        : V3GraphVertex(graphp)
-        , m_origVertexp(origVertexp)
-        , m_storedRank(0)
-        , m_onWorkList(false)
-        , m_deleted(false) {}
-    virtual ~GraphAcycVertex() {}
+        : V3GraphVertex{graphp}
+        , m_origVertexp{origVertexp} {}
+    virtual ~GraphAcycVertex() override {}
     V3GraphVertex* origVertexp() const { return m_origVertexp; }
     void setDelete() { m_deleted = true; }
     bool isDelete() const { return m_deleted; }
-    virtual string name() const { return m_origVertexp->name(); }
-    virtual string dotColor() const { return m_origVertexp->dotColor(); }
-    virtual FileLine* fileline() const { return m_origVertexp->fileline(); }
+    virtual string name() const override { return m_origVertexp->name(); }
+    virtual string dotColor() const override { return m_origVertexp->dotColor(); }
+    virtual FileLine* fileline() const override { return m_origVertexp->fileline(); }
 };
 
 //--------------------------------------------------------------------
@@ -70,19 +66,21 @@ private:
 public:
     GraphAcycEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top, int weight,
                   bool cutable = false)
-        : V3GraphEdge(graphp, fromp, top, weight, cutable) {}
-    virtual ~GraphAcycEdge() {}
+        : V3GraphEdge{graphp, fromp, top, weight, cutable} {}
+    virtual ~GraphAcycEdge() override {}
     // yellow=we might still cut it, else oldEdge: yellowGreen=made uncutable, red=uncutable
-    virtual string dotColor() const { return (cutable() ? "yellow" : origEdgep()->dotColor()); }
+    virtual string dotColor() const override {
+        return (cutable() ? "yellow" : origEdgep()->dotColor());
+    }
 };
 
 //--------------------------------------------------------------------
 
 struct GraphAcycEdgeCmp {
     inline bool operator()(const V3GraphEdge* lhsp, const V3GraphEdge* rhsp) const {
-        if (lhsp->weight() > rhsp->weight()) return 1;  // LHS goes first
-        if (lhsp->weight() < rhsp->weight()) return 0;  // RHS goes first
-        return 0;
+        if (lhsp->weight() > rhsp->weight()) return true;  // LHS goes first
+        if (lhsp->weight() < rhsp->weight()) return false;  // RHS goes first
+        return false;
     }
 };
 
@@ -105,7 +103,7 @@ private:
     V3List<GraphAcycVertex*> m_work;  // List of vertices with optimization work left
     std::vector<OrigEdgeList*> m_origEdgeDelp;  // List of deletions to do when done
     V3EdgeFuncP m_origEdgeFuncp;  // Function that says we follow this edge (in original graph)
-    uint32_t m_placeStep;  // Number that user() must be equal to to indicate processing
+    uint32_t m_placeStep = 0;  // Number that user() must be equal to to indicate processing
 
     static int debug() { return V3Graph::debug(); }
 
@@ -137,7 +135,7 @@ private:
     void addOrigEdgep(V3GraphEdge* toEdgep, V3GraphEdge* addEdgep) {
         // Add addEdge (or it's list) to list of edges that break edge represents
         // Note addEdge may already have a bunch of similar linked edge representations.  Yuk.
-        UASSERT(addEdgep, "Adding NULL");
+        UASSERT(addEdgep, "Adding nullptr");
         if (!toEdgep->userp()) {
             OrigEdgeList* oep = new OrigEdgeList;
             m_origEdgeDelp.push_back(oep);
@@ -187,16 +185,11 @@ private:
 
 public:
     // CONSTRUCTORS
-    GraphAcyc(V3Graph* origGraphp, V3EdgeFuncP edgeFuncp) {
-        m_origGraphp = origGraphp;
-        m_origEdgeFuncp = edgeFuncp;
-        m_placeStep = 0;
-    }
+    GraphAcyc(V3Graph* origGraphp, V3EdgeFuncP edgeFuncp)
+        : m_origGraphp{origGraphp}
+        , m_origEdgeFuncp{edgeFuncp} {}
     ~GraphAcyc() {
-        for (std::vector<OrigEdgeList*>::iterator it = m_origEdgeDelp.begin();
-             it != m_origEdgeDelp.end(); ++it) {
-            delete (*it);
-        }
+        for (OrigEdgeList* ip : m_origEdgeDelp) delete ip;
         m_origEdgeDelp.clear();
     }
     void main();
@@ -383,7 +376,7 @@ void GraphAcyc::simplifyDup(GraphAcycVertex* avertexp) {
     if (avertexp->isDelete()) return;
     // Clear marks
     for (V3GraphEdge* edgep = avertexp->outBeginp(); edgep; edgep = edgep->outNextp()) {
-        edgep->top()->userp(NULL);
+        edgep->top()->userp(nullptr);
     }
     // Mark edges and detect duplications
     for (V3GraphEdge *nextp, *edgep = avertexp->outBeginp(); edgep; edgep = nextp) {
@@ -480,10 +473,7 @@ void GraphAcyc::place() {
 
     // Process each edge in weighted order
     m_placeStep = 10;
-    for (std::vector<V3GraphEdge*>::iterator it = edges.begin(); it != edges.end(); ++it) {
-        V3GraphEdge* edgep = (*it);
-        placeTryEdge(edgep);
-    }
+    for (V3GraphEdge* edgep : edges) placeTryEdge(edgep);
 }
 
 void GraphAcyc::placeTryEdge(V3GraphEdge* edgep) {
