@@ -30,7 +30,6 @@
 #include "V3File.h"
 
 #include <algorithm>
-#include <cstdarg>
 #include <iomanip>
 #include <map>
 #include <memory>
@@ -61,7 +60,7 @@ private:
                 nodep,
                 "Node " << nodep->prettyTypeName()
                         << " in statement position but not marked stmt (node under function)");
-            V3Hash oldHash = m_lowerHash;
+            VL_RESTORER(m_lowerHash);
             {
                 m_lowerHash = nodep->sameHash();
                 UASSERT_OBJ(!m_lowerHash.isIllegal(), nodep,
@@ -75,19 +74,18 @@ private:
                 // Store the hash value
                 nodep->user4(m_lowerHash.fullValue());
                 // UINFO(9, "    hashnode "<<m_lowerHash<<"  "<<nodep<<endl);
+                thisHash = m_lowerHash;
             }
-            thisHash = m_lowerHash;
-            m_lowerHash = oldHash;
         }
         // Update what will become the above node's hash
         m_lowerHash += m_cacheInUser4 ? V3Hashed::nodeHash(nodep) : thisHash;
     }
 
     //--------------------
-    virtual void visit(AstVar*) VL_OVERRIDE {}
-    virtual void visit(AstTypedef*) VL_OVERRIDE {}
-    virtual void visit(AstParamTypeDType*) VL_OVERRIDE {}
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstVar*) override {}
+    virtual void visit(AstTypedef*) override {}
+    virtual void visit(AstParamTypeDType*) override {}
+    virtual void visit(AstNode* nodep) override {
         // Hash not just iterate
         nodeHashIterate(nodep);
     }
@@ -95,16 +93,16 @@ private:
 public:
     // CONSTRUCTORS
     explicit HashedVisitor(AstNode* nodep)
-        : m_cacheInUser4(true) {
+        : m_cacheInUser4{true} {
         nodeHashIterate(nodep);
         // UINFO(9,"  stmthash "<<hex<<V3Hashed::nodeHash(nodep)<<"  "<<nodep<<endl);
     }
     explicit HashedVisitor(const AstNode* nodep)
-        : m_cacheInUser4(false) {
+        : m_cacheInUser4{false} {
         nodeHashIterate(const_cast<AstNode*>(nodep));
     }
     V3Hash finalHash() const { return m_lowerHash; }
-    virtual ~HashedVisitor() {}
+    virtual ~HashedVisitor() override {}
 };
 
 //######################################################################
@@ -137,7 +135,7 @@ void V3Hashed::erase(iterator it) {
     UINFO(8, "   erase " << nodep << endl);
     UASSERT_OBJ(nodep->user4p(), nodep, "Called removeNode on non-hashed node");
     m_hashMmap.erase(it);
-    nodep->user4p(NULL);  // So we don't allow removeNode again
+    nodep->user4p(nullptr);  // So we don't allow removeNode again
 }
 
 void V3Hashed::check() {
@@ -152,14 +150,14 @@ void V3Hashed::dumpFilePrefixed(const string& nameComment, bool tree) {
 }
 
 void V3Hashed::dumpFile(const string& filename, bool tree) {
-    const vl_unique_ptr<std::ofstream> logp(V3File::new_ofstream(filename));
+    const std::unique_ptr<std::ofstream> logp(V3File::new_ofstream(filename));
     if (logp->fail()) v3fatal("Can't write " << filename);
 
     std::map<int, int> dist;
 
     V3Hash lasthash;
     int num_in_bucket = 0;
-    for (HashMmap::iterator it = begin(); 1; ++it) {
+    for (HashMmap::iterator it = begin(); true; ++it) {
         if (it == end() || lasthash != it->first) {
             if (it != end()) lasthash = it->first;
             if (num_in_bucket) {
