@@ -37,10 +37,10 @@ private:
     AstUser4InUse m_inuser4;
 
     // MEMBERS
-    uint32_t m_instrCount;  // Running count of instructions
+    uint32_t m_instrCount = 0;  // Running count of instructions
     const AstNode* m_startNodep;  // Start node of count
-    bool m_tracingCall;  // Iterating into a CCall to a CFunc
-    bool m_inCFunc;  // Inside AstCFunc
+    bool m_tracingCall = false;  // Iterating into a CCall to a CFunc
+    bool m_inCFunc = false;  // Inside AstCFunc
     bool m_assertNoDups;  // Check for duplicates
     std::ostream* m_osp;  // Dump file
 
@@ -56,8 +56,8 @@ private:
     public:
         // CONSTRUCTORS
         VisitBase(InstrCountVisitor* visitor, AstNode* nodep)
-            : m_nodep(nodep)
-            , m_visitor(visitor) {
+            : m_nodep{nodep}
+            , m_visitor{visitor} {
             m_savedCount = m_visitor->startVisitBase(nodep);
         }
         ~VisitBase() { m_visitor->endVisitBase(m_savedCount, m_nodep); }
@@ -69,15 +69,12 @@ private:
 public:
     // CONSTRUCTORS
     InstrCountVisitor(AstNode* nodep, bool assertNoDups, std::ostream* osp)
-        : m_instrCount(0)
-        , m_startNodep(nodep)
-        , m_tracingCall(false)
-        , m_inCFunc(false)
-        , m_assertNoDups(assertNoDups)
-        , m_osp(osp) {
+        : m_startNodep{nodep}
+        , m_assertNoDups{assertNoDups}
+        , m_osp{osp} {
         if (nodep) iterate(nodep);
     }
-    virtual ~InstrCountVisitor() {}
+    virtual ~InstrCountVisitor() override {}
 
     // METHODS
     uint32_t instrCount() const { return m_instrCount; }
@@ -119,7 +116,7 @@ private:
     }
 
     // VISITORS
-    virtual void visit(AstNodeSel* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeSel* nodep) override {
         // This covers both AstArraySel and AstWordSel
         //
         // If some vector is a bazillion dwords long, and we're selecting 1
@@ -130,7 +127,7 @@ private:
         VisitBase vb(this, nodep);
         iterateAndNextNull(nodep->bitp());
     }
-    virtual void visit(AstSel* nodep) VL_OVERRIDE {
+    virtual void visit(AstSel* nodep) override {
         // Similar to AstNodeSel above, a small select into a large vector
         // is not expensive. Count the cost of the AstSel itself (scales with
         // its width) and the cost of the lsbp() and widthp() nodes, but not
@@ -139,13 +136,13 @@ private:
         iterateAndNextNull(nodep->lsbp());
         iterateAndNextNull(nodep->widthp());
     }
-    virtual void visit(AstSliceSel* nodep) VL_OVERRIDE {
+    virtual void visit(AstSliceSel* nodep) override {  // LCOV_EXCL_LINE
         nodep->v3fatalSrc("AstSliceSel unhandled");
     }
-    virtual void visit(AstMemberSel* nodep) VL_OVERRIDE {
+    virtual void visit(AstMemberSel* nodep) override {  // LCOV_EXCL_LINE
         nodep->v3fatalSrc("AstMemberSel unhandled");
     }
-    virtual void visit(AstConcat* nodep) VL_OVERRIDE {
+    virtual void visit(AstConcat* nodep) override {
         // Nop.
         //
         // Ignore concat. The problem with counting concat is that when we
@@ -165,7 +162,7 @@ private:
         // the widths of the operands (ignored here).
         markCost(nodep);
     }
-    virtual void visit(AstNodeIf* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeIf* nodep) override {
         VisitBase vb(this, nodep);
         iterateAndNextNull(nodep->condp());
         uint32_t savedCount = m_instrCount;
@@ -190,7 +187,7 @@ private:
             if (nodep->ifsp()) nodep->ifsp()->user4(0);  // Don't dump it
         }
     }
-    virtual void visit(AstNodeCond* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeCond* nodep) override {
         // Just like if/else above, the ternary operator only evaluates
         // one of the two expressions, so only count the max.
         VisitBase vb(this, nodep);
@@ -215,7 +212,7 @@ private:
             if (nodep->expr2p()) nodep->expr2p()->user4(0);  // Don't dump it
         }
     }
-    virtual void visit(AstActive* nodep) VL_OVERRIDE {
+    virtual void visit(AstActive* nodep) override {
         // You'd think that the OrderLogicVertex's would be disjoint trees
         // of stuff in the AST, but it isn't so: V3Order makes an
         // OrderLogicVertex for each ACTIVE, and then also makes an
@@ -231,14 +228,14 @@ private:
         markCost(nodep);
         UASSERT_OBJ(nodep == m_startNodep, nodep, "Multiple actives, or not start node");
     }
-    virtual void visit(AstNodeCCall* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeCCall* nodep) override {
         VisitBase vb(this, nodep);
         iterateChildren(nodep);
         m_tracingCall = true;
         iterate(nodep->funcp());
         UASSERT_OBJ(!m_tracingCall, nodep, "visit(AstCFunc) should have cleared m_tracingCall.");
     }
-    virtual void visit(AstCFunc* nodep) VL_OVERRIDE {
+    virtual void visit(AstCFunc* nodep) override {
         // Don't count a CFunc other than by tracing a call or counting it
         // from the root
         UASSERT_OBJ(m_tracingCall || nodep == m_startNodep, nodep,
@@ -252,7 +249,7 @@ private:
         }
         m_inCFunc = saved_inCFunc;
     }
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         VisitBase vb(this, nodep);
         iterateChildren(nodep);
     }
@@ -269,23 +266,22 @@ private:
 
     // MEMBERS
     std::ostream* m_osp;  // Dump file
-    unsigned m_depth;  // Current tree depth for printing indent
+    unsigned m_depth = 0;  // Current tree depth for printing indent
 
 public:
     // CONSTRUCTORS
     InstrCountDumpVisitor(AstNode* nodep, std::ostream* osp)
-        : m_osp(osp)
-        , m_depth(0) {
-        // No check for NULL output, so...
+        : m_osp{osp} {
+        // No check for nullptr output, so...
         UASSERT_OBJ(osp, nodep, "Don't call if not dumping");
         if (nodep) iterate(nodep);
     }
-    virtual ~InstrCountDumpVisitor() {}
+    virtual ~InstrCountDumpVisitor() override {}
 
 private:
     // METHODS
-    string indent() { return string(m_depth, ':') + " "; }
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    string indent() const { return string(m_depth, ':') + " "; }
+    virtual void visit(AstNode* nodep) override {
         ++m_depth;
         if (unsigned costPlus1 = nodep->user4()) {
             *m_osp << "  " << indent() << "cost " << std::setw(6) << std::left << (costPlus1 - 1)
