@@ -27,7 +27,6 @@
 #include "V3Ast.h"
 
 #include <algorithm>
-#include <cstdarg>
 #include <map>
 #include <vector>
 
@@ -47,8 +46,6 @@ void V3LinkLevel::modSortByLevel() {
 
     // level() was computed for us in V3LinkCells
 
-    typedef std::vector<AstNodeModule*> ModVec;
-
     ModVec mods;  // Modules
     ModVec tops;  // Top level modules
     for (AstNodeModule* nodep = v3Global.rootp()->modulesp(); nodep;
@@ -64,8 +61,7 @@ void V3LinkLevel::modSortByLevel() {
                                        << "... Suggest see manual; fix the duplicates, or use "
                                           "--top-module to select top."
                                        << V3Error::warnContextNone());
-            for (ModVec::const_iterator it = tops.begin(); it != tops.end(); ++it) {
-                AstNode* alsop = *it;
+            for (AstNode* alsop : tops) {
                 std::cout << secp->warnMore() << "... Top module " << alsop->prettyNameQ() << endl
                           << alsop->warnContextSecondary();
             }
@@ -77,24 +73,20 @@ void V3LinkLevel::modSortByLevel() {
     // Reorder the netlist's modules to have modules in level sorted order
     stable_sort(mods.begin(), mods.end(), CmpLevel());  // Sort the vector
     UINFO(9, "modSortByLevel() sorted\n");  // Comment required for gcc4.6.3 / bug666
-    for (ModVec::const_iterator it = mods.begin(); it != mods.end(); ++it) {
-        AstNodeModule* nodep = *it;
+    for (AstNodeModule* nodep : mods) {
         nodep->clearIter();  // Because we didn't iterate to find the node
         // pointers, may have a stale m_iterp() needing cleanup
         nodep->unlinkFrBack();
     }
     UASSERT_OBJ(!v3Global.rootp()->modulesp(), v3Global.rootp(), "Unlink didn't work");
-    for (ModVec::const_iterator it = mods.begin(); it != mods.end(); ++it) {
-        AstNodeModule* nodep = *it;
-        v3Global.rootp()->addModulep(nodep);
-    }
+    for (AstNodeModule* nodep : mods) v3Global.rootp()->addModulep(nodep);
     UINFO(9, "modSortByLevel() done\n");  // Comment required for gcc4.6.3 / bug666
     V3Global::dumpCheckGlobalTree("cells", false, v3Global.opt.dumpTreeLevel(__FILE__) >= 3);
 }
 
 void V3LinkLevel::timescaling(const ModVec& mods) {
     // Timescale determination
-    AstNodeModule* modTimedp = NULL;
+    AstNodeModule* modTimedp = nullptr;
     VTimescale unit(VTimescale::NONE);
     // Use highest level module as default unit - already sorted in proper order
     for (ModVec::const_iterator it = mods.begin(); it != mods.end(); ++it) {
@@ -108,8 +100,7 @@ void V3LinkLevel::timescaling(const ModVec& mods) {
     if (unit.isNone()) unit = VTimescale(VTimescale::TS_DEFAULT);
     v3Global.rootp()->timeunit(unit);
 
-    for (ModVec::const_iterator it = mods.begin(); it != mods.end(); ++it) {
-        AstNodeModule* nodep = *it;
+    for (AstNodeModule* nodep : mods) {
         if (nodep->timeunit().isNone()) {
             if (modTimedp && !VN_IS(nodep, Iface)
                 && !(VN_IS(nodep, Package) && VN_CAST(nodep, Package)->isDollarUnit())) {
@@ -163,7 +154,7 @@ void V3LinkLevel::wrapTop(AstNetlist* rootp) {
             AstCell* cellp = new AstCell(modp->fileline(), modp->fileline(),
                                          // Could add __03a__03a="::" to prevent conflict
                                          // with module names/"v"
-                                         modp->name(), modp->name(), NULL, NULL, NULL);
+                                         modp->name(), modp->name(), nullptr, nullptr, nullptr);
             cellp->modp(modp);
             newmodp->addStmtp(cellp);
         }
@@ -177,7 +168,7 @@ void V3LinkLevel::wrapTopCell(AstNetlist* rootp) {
     UASSERT_OBJ(newmodp && newmodp->isTop(), rootp, "No TOP module found to insert under");
 
     // Find all duplicate signal names (if multitop)
-    typedef vl_unordered_set<std::string> NameSet;
+    typedef std::unordered_set<std::string> NameSet;
     NameSet ioNames;
     NameSet dupNames;
     // For all modules, skipping over new top
@@ -206,7 +197,7 @@ void V3LinkLevel::wrapTopCell(AstNetlist* rootp) {
         AstCell* cellp = new AstCell(
             newmodp->fileline(), newmodp->fileline(),
             (!v3Global.opt.l2Name().empty() ? v3Global.opt.l2Name() : oldmodp->name()),
-            oldmodp->name(), NULL, NULL, NULL);
+            oldmodp->name(), nullptr, nullptr, nullptr);
         cellp->modp(oldmodp);
         newmodp->addStmtp(cellp);
 
@@ -231,8 +222,9 @@ void V3LinkLevel::wrapTopCell(AstNetlist* rootp) {
                         varp->primaryIO(true);
                     }
                     if (varp->direction().isRefOrConstRef()) {
-                        varp->v3error("Unsupported: ref/const ref as primary input/output: "
-                                      << varp->prettyNameQ());
+                        varp->v3warn(E_UNSUPPORTED,
+                                     "Unsupported: ref/const ref as primary input/output: "
+                                         << varp->prettyNameQ());
                     }
                     if (varp->isIO() && v3Global.opt.systemC()) {
                         varp->sc(true);
