@@ -33,9 +33,7 @@
 #include "V3Ast.h"
 
 #include <algorithm>
-#include <cstdarg>
 #include <map>
-#include <vector>
 
 //######################################################################
 // Link state, as a visitor of each AstNode
@@ -49,12 +47,12 @@ private:
 
     // STATE
     // Below state needs to be preserved between each module call.
-    AstNodeModule* m_modp;  // Current module
-    AstClass* m_classp;  // Class we're inside
-    AstNodeFTask* m_ftaskp;  // Function or task we're inside
-    AstNodeCoverOrAssert* m_assertp;  // Current assertion
-    VLifetime m_lifetime;  // Propagating lifetime
-    int m_senitemCvtNum;  // Temporary signal counter
+    AstNodeModule* m_modp = nullptr;  // Current module
+    AstClass* m_classp = nullptr;  // Class we're inside
+    AstNodeFTask* m_ftaskp = nullptr;  // Function or task we're inside
+    AstNodeCoverOrAssert* m_assertp = nullptr;  // Current assertion
+    VLifetime m_lifetime = VLifetime::STATIC;  // Propagating lifetime
+    int m_senitemCvtNum = 0;  // Temporary signal counter
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
@@ -63,13 +61,13 @@ private:
     // TODO: Most of these visitors are here for historical reasons.
     // TODO: ExpectDecriptor can move to data type resolution, and the rest
     // TODO: could move to V3LinkParse to get them out of the way of elaboration
-    virtual void visit(AstNodeModule* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeModule* nodep) override {
         // Module: Create sim table for entire module and iterate
         UINFO(8, "MODULE " << nodep << endl);
         if (nodep->dead()) return;
-        AstNodeModule* origModp = m_modp;
-        VLifetime origLifetime = m_lifetime;
-        int origSenitemCvtNum = m_senitemCvtNum;
+        VL_RESTORER(m_modp);
+        VL_RESTORER(m_lifetime);
+        VL_RESTORER(m_senitemCvtNum);
         {
             m_modp = nodep;
             m_senitemCvtNum = 0;
@@ -77,23 +75,18 @@ private:
             if (m_lifetime.isNone()) m_lifetime = VLifetime::STATIC;
             iterateChildren(nodep);
         }
-        m_modp = origModp;
-        m_senitemCvtNum = origSenitemCvtNum;
-        m_lifetime = origLifetime;
     }
-    virtual void visit(AstClass* nodep) VL_OVERRIDE {
-        AstClass* origClassp = m_classp;
-        VLifetime origLifetime = m_lifetime;
+    virtual void visit(AstClass* nodep) override {
+        VL_RESTORER(m_classp);
+        VL_RESTORER(m_lifetime);
         {
             m_classp = nodep;
             m_lifetime = nodep->lifetime();
             if (m_lifetime.isNone()) m_lifetime = VLifetime::AUTOMATIC;
             iterateChildren(nodep);
         }
-        m_classp = origClassp;
-        m_lifetime = origLifetime;
     }
-    virtual void visit(AstInitial* nodep) VL_OVERRIDE {
+    virtual void visit(AstInitial* nodep) override {
         iterateChildren(nodep);
         // Initial assignments under function/tasks can just be simple
         // assignments without the initial
@@ -101,53 +94,67 @@ private:
             VL_DO_DANGLING(nodep->replaceWith(nodep->bodysp()->unlinkFrBackWithNext()), nodep);
         }
     }
-    virtual void visit(AstNodeCoverOrAssert* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeCoverOrAssert* nodep) override {
         if (m_assertp) nodep->v3error("Assert not allowed under another assert");
         m_assertp = nodep;
         iterateChildren(nodep);
-        m_assertp = NULL;
+        m_assertp = nullptr;
     }
-    virtual void visit(AstVar* nodep) VL_OVERRIDE {
+    virtual void visit(AstVar* nodep) override {
         iterateChildren(nodep);
         if (m_classp && !nodep->isParam()) nodep->varType(AstVarType::MEMBER);
-        if (m_classp && nodep->isParam()) nodep->v3error("Unsupported: class parameter");
+        if (m_classp && nodep->isParam())
+            nodep->v3warn(E_UNSUPPORTED, "Unsupported: class parameter");
         if (m_ftaskp) nodep->funcLocal(true);
-        if (nodep->lifetime().isNone()) nodep->lifetime(m_lifetime);
+        if (nodep->lifetime().isNone()) {
+            if (nodep->isFuncLocal() && nodep->isIO()) {
+                nodep->lifetime(VLifetime::AUTOMATIC);
+            } else {
+                nodep->lifetime(m_lifetime);
+            }
+        }
         if (nodep->isSigModPublic()) {
             nodep->sigModPublic(false);  // We're done with this attribute
             m_modp->modPublic(true);  // Avoid flattening if signals are exposed
         }
     }
 
-    virtual void visit(AstNodeVarRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeVarRef* nodep) override {
         // VarRef: Resolve its reference
         if (nodep->varp()) { nodep->varp()->usedParam(true); }
         iterateChildren(nodep);
     }
 
-    virtual void visit(AstNodeFTask* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeFTask* nodep) override {
         // NodeTask: Remember its name for later resolution
         // Remember the existing symbol table scope
         if (m_classp) nodep->classMethod(true);
+        // V3LinkDot moved the isExternDef into the class, the extern proto was
+        // checked to exist, and now isn't needed
+        nodep->isExternDef(false);
+        if (nodep->isExternProto()) {
+            VL_DO_DANGLING(nodep->unlinkFrBack()->deleteTree(), nodep);
+            return;
+        }
         VLifetime origLifetime = m_lifetime;
         {
-            if (!nodep->lifetime().isNone()) m_lifetime = nodep->lifetime();
+            m_lifetime = nodep->lifetime();
             if (m_lifetime.isNone()) m_lifetime = VLifetime::AUTOMATIC;
             m_ftaskp = nodep;
             iterateChildren(nodep);
         }
-        m_ftaskp = NULL;
+        m_ftaskp = nullptr;
         m_lifetime = origLifetime;
         if (nodep->dpiExport()) { nodep->scopeNamep(new AstScopeName(nodep->fileline())); }
     }
-    virtual void visit(AstNodeFTaskRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeFTaskRef* nodep) override {
         iterateChildren(nodep);
         if (nodep->taskp() && (nodep->taskp()->dpiContext() || nodep->taskp()->dpiExport())) {
             nodep->scopeNamep(new AstScopeName(nodep->fileline()));
         }
     }
 
-    virtual void visit(AstSenItem* nodep) VL_OVERRIDE {
+    virtual void visit(AstSenItem* nodep) override {
         // Remove bit selects, and bark if it's not a simple variable
         iterateChildren(nodep);
         if (nodep->isClocked()) {
@@ -169,8 +176,9 @@ private:
                     addwherep = addwherep->backp();
                 }
                 if (!VN_IS(addwherep, Always)) {  // Assertion perhaps?
-                    sensp->v3error("Unsupported: Non-single-bit pos/negedge clock statement under "
-                                   "some complicated block");
+                    sensp->v3warn(E_UNSUPPORTED,
+                                  "Unsupported: Non-single-bit pos/negedge clock statement under "
+                                  "some complicated block");
                     addwherep = m_modp;
                 }
                 addwherep->addNext(newvarp);
@@ -209,14 +217,11 @@ private:
             && !VN_IS(nodep->sensp(), EnumItemRef)  // V3Const will cleanup
             && !nodep->isIllegal()) {
             if (debug()) nodep->dumpTree(cout, "-tree: ");
-            nodep->v3error("Unsupported: Complex statement in sensitivity list");
+            nodep->v3warn(E_UNSUPPORTED, "Unsupported: Complex statement in sensitivity list");
         }
     }
-    virtual void visit(AstSenGate* nodep) VL_OVERRIDE {
-        nodep->v3fatalSrc("SenGates shouldn't be in tree yet");
-    }
 
-    virtual void visit(AstNodePreSel* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodePreSel* nodep) override {
         if (!nodep->attrp()) {
             iterateChildren(nodep);
             // Constification may change the fromp() to a constant, which will lose the
@@ -241,8 +246,8 @@ private:
             } else if (VN_IS(basefromp, Replicate)) {
                 // From {...}[...] syntax in IEEE 2017
                 if (basefromp) { UINFO(1, "    Related node: " << basefromp << endl); }
-                nodep->v3error("Unsupported: Select of concatenation");
-                nodep = NULL;
+                nodep->v3warn(E_UNSUPPORTED, "Unsupported: Select of concatenation");
+                nodep = nullptr;
             } else {
                 if (basefromp) { UINFO(1, "    Related node: " << basefromp << endl); }
                 nodep->v3fatalSrc("Illegal bit select; no signal/member being extracted from");
@@ -250,7 +255,7 @@ private:
         }
     }
 
-    virtual void visit(AstCaseItem* nodep) VL_OVERRIDE {
+    virtual void visit(AstCaseItem* nodep) override {
         // Move default caseItems to the bottom of the list
         // That saves us from having to search each case list twice, for non-defaults and defaults
         iterateChildren(nodep);
@@ -262,8 +267,18 @@ private:
         }
     }
 
-    virtual void visit(AstPragma* nodep) VL_OVERRIDE {
-        if (nodep->pragType() == AstPragmaType::PUBLIC_MODULE) {
+    virtual void visit(AstPragma* nodep) override {
+        if (nodep->pragType() == AstPragmaType::HIER_BLOCK) {
+            UASSERT_OBJ(m_modp, nodep, "HIER_BLOCK not under a module");
+            // If this is hierarchical mode which is to create protect-lib,
+            // sub modules do not have hier_block meta comment in the source code.
+            // But .vlt files may still mark a module which is actually a protect-lib wrapper
+            // hier_block. AstNodeModule::hierBlock() can be true only when --hierarchical is
+            // specified.
+            m_modp->hierBlock(v3Global.opt.hierarchical());
+            nodep->unlinkFrBack();
+            VL_DO_DANGLING(pushDeletep(nodep), nodep);
+        } else if (nodep->pragType() == AstPragmaType::PUBLIC_MODULE) {
             UASSERT_OBJ(m_modp, nodep, "PUBLIC_MODULE not under a module");
             m_modp->modPublic(true);
             nodep->unlinkFrBack();
@@ -289,11 +304,12 @@ private:
         // Check display arguments, return new format string
         string newFormat;
         bool inPct = false;
+        bool inIgnore = false;
         string fmt;
-        for (string::const_iterator it = format.begin(); it != format.end(); ++it) {
-            char ch = *it;
+        for (const char ch : format) {
             if (!inPct && ch == '%') {
                 inPct = true;
+                inIgnore = false;
                 fmt = ch;
             } else if (inPct && (isdigit(ch) || ch == '.' || ch == '-')) {
                 fmt += ch;
@@ -303,23 +319,27 @@ private:
                 switch (tolower(ch)) {
                 case '%':  // %% - just output a %
                     break;
+                case '*':
+                    inPct = true;
+                    inIgnore = true;
+                    break;
                 case 'm':  // %m - auto insert "name"
                     if (isScan) {
-                        nodep->v3error("Unsupported: %m in $fscanf");
+                        nodep->v3warn(E_UNSUPPORTED, "Unsupported: %m in $fscanf");
                         fmt = "";
                     }
                     break;
                 case 'l':  // %l - auto insert "library"
                     if (isScan) {
-                        nodep->v3error("Unsupported: %l in $fscanf");
+                        nodep->v3warn(E_UNSUPPORTED, "Unsupported: %l in $fscanf");
                         fmt = "";
                     }
                     if (m_modp) fmt = VString::quotePercent(m_modp->prettyName());
                     break;
                 default:  // Most operators, just move to next argument
-                    if (!V3Number::displayedFmtLegal(ch)) {
+                    if (!V3Number::displayedFmtLegal(ch, isScan)) {
                         nodep->v3error("Unknown $display-like format code: '%" << ch << "'");
-                    } else {
+                    } else if (!inIgnore) {
                         if (!argp) {
                             nodep->v3error("Missing arguments for $display-like format");
                         } else {
@@ -358,11 +378,20 @@ private:
                         } else if (inpercent) {
                             inpercent = 0;
                             switch (c) {
-                            case '0' ... '9':
+                            case '0':  // FALLTHRU
+                            case '1':  // FALLTHRU
+                            case '2':  // FALLTHRU
+                            case '3':  // FALLTHRU
+                            case '4':  // FALLTHRU
+                            case '5':  // FALLTHRU
+                            case '6':  // FALLTHRU
+                            case '7':  // FALLTHRU
+                            case '8':  // FALLTHRU
+                            case '9':  // FALLTHRU
                             case '.': inpercent = true; break;
                             case '%': break;
                             default:
-                                if (V3Number::displayedFmtLegal(c)) { skipCount++; }
+                                if (V3Number::displayedFmtLegal(c, isScan)) ++skipCount;
                             }
                         }
                     }
@@ -382,40 +411,45 @@ private:
 
     void expectDescriptor(AstNode* nodep, AstNodeVarRef* filep) {
         if (!filep) {
-            nodep->v3error("Unsupported: $fopen/$fclose/$f* descriptor must be a simple variable");
+            nodep->v3warn(E_UNSUPPORTED,
+                          "Unsupported: $fopen/$fclose/$f* descriptor must be a simple variable");
         }
         if (filep && filep->varp()) filep->varp()->attrFileDescr(true);
     }
 
-    virtual void visit(AstFOpen* nodep) VL_OVERRIDE {
+    virtual void visit(AstFOpen* nodep) override {
         iterateChildren(nodep);
         expectDescriptor(nodep, VN_CAST(nodep->filep(), NodeVarRef));
     }
-    virtual void visit(AstFClose* nodep) VL_OVERRIDE {
+    virtual void visit(AstFOpenMcd* nodep) override {
         iterateChildren(nodep);
         expectDescriptor(nodep, VN_CAST(nodep->filep(), NodeVarRef));
     }
-    virtual void visit(AstFError* nodep) VL_OVERRIDE {
+    virtual void visit(AstFClose* nodep) override {
         iterateChildren(nodep);
         expectDescriptor(nodep, VN_CAST(nodep->filep(), NodeVarRef));
     }
-    virtual void visit(AstFEof* nodep) VL_OVERRIDE {
+    virtual void visit(AstFError* nodep) override {
         iterateChildren(nodep);
         expectDescriptor(nodep, VN_CAST(nodep->filep(), NodeVarRef));
     }
-    virtual void visit(AstFRead* nodep) VL_OVERRIDE {
+    virtual void visit(AstFEof* nodep) override {
         iterateChildren(nodep);
         expectDescriptor(nodep, VN_CAST(nodep->filep(), NodeVarRef));
     }
-    virtual void visit(AstFScanF* nodep) VL_OVERRIDE {
+    virtual void visit(AstFRead* nodep) override {
+        iterateChildren(nodep);
+        expectDescriptor(nodep, VN_CAST(nodep->filep(), NodeVarRef));
+    }
+    virtual void visit(AstFScanF* nodep) override {
         iterateChildren(nodep);
         expectFormat(nodep, nodep->text(), nodep->exprsp(), true);
     }
-    virtual void visit(AstSScanF* nodep) VL_OVERRIDE {
+    virtual void visit(AstSScanF* nodep) override {
         iterateChildren(nodep);
         expectFormat(nodep, nodep->text(), nodep->exprsp(), true);
     }
-    virtual void visit(AstSFormatF* nodep) VL_OVERRIDE {
+    virtual void visit(AstSFormatF* nodep) override {
         iterateChildren(nodep);
         // Cleanup old-school displays without format arguments
         if (!nodep->hasFormat()) {
@@ -438,14 +472,14 @@ private:
         }
     }
 
-    virtual void visit(AstUdpTable* nodep) VL_OVERRIDE {
+    virtual void visit(AstUdpTable* nodep) override {
         UINFO(5, "UDPTABLE  " << nodep << endl);
         if (!v3Global.opt.bboxUnsup()) {
             // We don't warn until V3Inst, so that UDPs that are in libraries and
             // never used won't result in any warnings.
         } else {
             // Massive hack, just tie off all outputs so our analysis can proceed
-            AstVar* varoutp = NULL;
+            AstVar* varoutp = nullptr;
             for (AstNode* stmtp = m_modp->stmtsp(); stmtp; stmtp = stmtp->nextp()) {
                 if (AstVar* varp = VN_CAST(stmtp, Var)) {
                     if (varp->isReadOnly()) {
@@ -468,36 +502,28 @@ private:
         }
     }
 
-    virtual void visit(AstScCtor* nodep) VL_OVERRIDE {
+    virtual void visit(AstScCtor* nodep) override {
         // Constructor info means the module must remain public
         m_modp->modPublic(true);
         iterateChildren(nodep);
     }
-    virtual void visit(AstScDtor* nodep) VL_OVERRIDE {
+    virtual void visit(AstScDtor* nodep) override {
         // Destructor info means the module must remain public
         m_modp->modPublic(true);
         iterateChildren(nodep);
     }
-    virtual void visit(AstScInt* nodep) VL_OVERRIDE {
+    virtual void visit(AstScInt* nodep) override {
         // Special class info means the module must remain public
         m_modp->modPublic(true);
         iterateChildren(nodep);
     }
 
-    virtual void visit(AstNode* nodep) VL_OVERRIDE { iterateChildren(nodep); }
+    virtual void visit(AstNode* nodep) override { iterateChildren(nodep); }
 
 public:
     // CONSTRUCTORS
-    explicit LinkResolveVisitor(AstNetlist* rootp)
-        : m_lifetime(VLifetime::STATIC) {  // Static outside a module/class
-        m_classp = NULL;
-        m_ftaskp = NULL;
-        m_modp = NULL;
-        m_assertp = NULL;
-        m_senitemCvtNum = 0;
-        iterate(rootp);
-    }
-    virtual ~LinkResolveVisitor() {}
+    explicit LinkResolveVisitor(AstNetlist* rootp) { iterate(rootp); }
+    virtual ~LinkResolveVisitor() override {}
 };
 
 //######################################################################
@@ -508,40 +534,35 @@ public:
 class LinkBotupVisitor : public AstNVisitor {
 private:
     // STATE
-    AstNodeModule* m_modp;  // Current module
+    AstNodeModule* m_modp = nullptr;  // Current module
 
     // METHODS
     VL_DEBUG_FUNC;  // Declare debug()
 
     // VISITs
-    virtual void visit(AstNetlist* nodep) VL_OVERRIDE {
+    virtual void visit(AstNetlist* nodep) override {
         // Iterate modules backwards, in bottom-up order.
         iterateChildrenBackwards(nodep);
     }
-    virtual void visit(AstNodeModule* nodep) VL_OVERRIDE {
-        AstNodeModule* origModp = m_modp;
+    virtual void visit(AstNodeModule* nodep) override {
+        VL_RESTORER(m_modp);
         {
             m_modp = nodep;
             iterateChildren(nodep);
         }
-        m_modp = origModp;
     }
-    virtual void visit(AstCell* nodep) VL_OVERRIDE {
+    virtual void visit(AstCell* nodep) override {
         // Parent module inherits child's publicity
         if (nodep->modp()->modPublic()) m_modp->modPublic(true);
         //** No iteration for speed
     }
-    virtual void visit(AstNodeMath*) VL_OVERRIDE {}  // Accelerate
-    virtual void visit(AstNode* nodep) VL_OVERRIDE { iterateChildren(nodep); }
+    virtual void visit(AstNodeMath*) override {}  // Accelerate
+    virtual void visit(AstNode* nodep) override { iterateChildren(nodep); }
 
 public:
     // CONSTRUCTORS
-    explicit LinkBotupVisitor(AstNetlist* rootp) {
-        m_modp = NULL;
-        //
-        iterate(rootp);
-    }
-    virtual ~LinkBotupVisitor() {}
+    explicit LinkBotupVisitor(AstNetlist* rootp) { iterate(rootp); }
+    virtual ~LinkBotupVisitor() override {}
 };
 
 //######################################################################
