@@ -164,9 +164,11 @@ public:
         opCleanThis();
     }
     // Create from a verilog 32'hxxxx number.
-    V3Number(AstNode* nodep, const char* sourcep) { V3NumberCreate(nodep, sourcep, NULL); }
+    V3Number(AstNode* nodep, const char* sourcep) { V3NumberCreate(nodep, sourcep, nullptr); }
     class FileLined {};  // Fileline based errors, for parsing only, otherwise pass nodep
-    V3Number(FileLined, FileLine* fl, const char* sourcep) { V3NumberCreate(NULL, sourcep, fl); }
+    V3Number(FileLined, FileLine* fl, const char* sourcep) {
+        V3NumberCreate(nullptr, sourcep, fl);
+    }
     class VerilogStringLiteral {};  // For creator type-overload selection
     V3Number(VerilogStringLiteral, AstNode* nodep, const string& str);
     class String {};
@@ -175,11 +177,11 @@ public:
         setString(value);
     }
     explicit V3Number(const V3Number* nump, int width = 1) {
-        init(NULL, width);
+        init(nullptr, width);
         m_fileline = nump->fileline();
     }
     V3Number(const V3Number* nump, int width, uint32_t value) {
-        init(NULL, width);
+        init(nullptr, width);
         m_value[0] = value;
         opCleanThis();
         m_fileline = nump->fileline();
@@ -204,6 +206,7 @@ private:
 
 public:
     void v3errorEnd(std::ostringstream& sstr) const;
+    void v3errorEndFatal(std::ostringstream& sstr) const VL_ATTR_NORETURN;
     void width(int width, bool sized = true) {
         // Set width.  Only set m_width here, as we need to tweak vector size
         if (width) {
@@ -230,7 +233,7 @@ public:
     // ACCESSORS
     string ascii(bool prefixed = true, bool cleanVerilog = false) const;
     string displayed(AstNode* nodep, const string& vformat) const;
-    static bool displayedFmtLegal(char format);  // Is this a valid format letter?
+    static bool displayedFmtLegal(char format, bool isScan);  // Is this a valid format letter?
     int width() const { return m_width; }
     int widthMin() const;  // Minimum width that can represent this number (~== log2(num)+1)
     bool sized() const { return m_sized; }
@@ -266,10 +269,10 @@ public:
     bool isEqOne() const;
     bool isEqAllOnes(int optwidth = 0) const;
     bool isCaseEq(const V3Number& rhs) const;  // operator==
-    bool isLt(const V3Number& rhs) const;  // operator<
     bool isLtXZ(const V3Number& rhs) const;  // operator< with XZ compared
     void isSigned(bool ssigned) { m_signed = ssigned; }
-    bool isUnknown() const;
+    bool isAnyX() const;
+    bool isAnyXZ() const;
     bool isMsbXZ() const { return bitIsXZ(m_width); }
     uint32_t toUInt() const;
     vlsint32_t toSInt() const;
@@ -282,6 +285,8 @@ public:
     uint32_t toHash() const;
     uint32_t edataWord(int eword) const;
     uint8_t dataByte(int byte) const;
+    uint32_t countBits(const V3Number& ctrl) const;
+    uint32_t countBits(const V3Number& ctrl1, const V3Number& ctrl2, const V3Number& ctrl3) const;
     uint32_t countOnes() const;
     uint32_t
     mostSetBitP1() const;  // Highest bit set plus one, IE for 16 return 5, for 0 return 0.
@@ -312,6 +317,8 @@ public:
     V3Number& opRedAnd(const V3Number& lhs);
     V3Number& opRedXor(const V3Number& lhs);
     V3Number& opRedXnor(const V3Number& lhs);
+    V3Number& opCountBits(const V3Number& expr, const V3Number& ctrl1, const V3Number& ctrl2,
+                          const V3Number& ctrl3);
     V3Number& opCountOnes(const V3Number& lhs);
     V3Number& opIsUnknown(const V3Number& lhs);
     V3Number& opOneHot(const V3Number& lhs);
@@ -329,7 +336,6 @@ public:
     V3Number& opSelInto(const V3Number& lhs, int lsbval, int width);
     V3Number& opToLowerN(const V3Number& lhs);
     V3Number& opToUpperN(const V3Number& lhs);
-    V3Number& opCond(const V3Number& lhs, const V3Number& if1s, const V3Number& if0s);
     V3Number& opCaseEq(const V3Number& lhs, const V3Number& rhs);
     V3Number& opCaseNeq(const V3Number& lhs, const V3Number& rhs);
     V3Number& opWildEq(const V3Number& lhs, const V3Number& rhs);
@@ -381,7 +387,8 @@ public:
     V3Number& opLteS(const V3Number& lhs, const V3Number& rhs);  // Signed
 
     // "D" - double (aka real) math
-    V3Number& opIToRD(const V3Number& lhs);
+    V3Number& opIToRD(const V3Number& lhs, bool isSigned = false);
+    V3Number& opISToRD(const V3Number& lhs) { return opIToRD(lhs, true); }
     V3Number& opRToIS(const V3Number& lhs);
     V3Number& opRToIRoundS(const V3Number& lhs);
     V3Number& opRealToBits(const V3Number& lhs);
