@@ -55,8 +55,8 @@ public:
     V3TaskConnects* m_tconnects;
     // CONSTRUCTORS
     SimStackNode(AstFuncRef* funcp, V3TaskConnects* tconnects)
-        : m_funcp(funcp)
-        , m_tconnects(tconnects) {}
+        : m_funcp{funcp}
+        , m_tconnects{tconnects} {}
     ~SimStackNode() {}
 };
 
@@ -85,7 +85,7 @@ private:
     //    (and output for non-delayed assignments)
     //  AstVar(Scope)::user2()  -> AstCont*. Output value of variable (delayed assignments)
 
-    enum VarUsage { VU_NONE = 0, VU_LV = 1, VU_RV = 2, VU_LVDLY = 4 };
+    enum VarUsage : uint8_t { VU_NONE = 0, VU_LV = 1, VU_RV = 2, VU_LVDLY = 4 };
 
     // STATE
     // Major mode
@@ -93,7 +93,7 @@ private:
     bool m_scoped;  ///< Running with AstVarScopes instead of AstVars
     bool m_params;  ///< Doing parameter propagation
     // Checking:
-    string m_whyNotOptimizable;  ///< String explaining why not optimizable or NULL to optimize
+    string m_whyNotOptimizable;  ///< String explaining why not optimizable or nullptr to optimize
     AstNode* m_whyNotNodep;  ///< First node not optimizable
     bool m_anyAssignDly;  ///< True if found a delayed assignment
     bool m_anyAssignComb;  ///< True if found a non-delayed assignment
@@ -198,7 +198,7 @@ public:
             m_whyNotOptimizable += stack.str();
         }
     }
-    bool optimizable() const { return m_whyNotNodep == NULL; }
+    bool optimizable() const { return m_whyNotNodep == nullptr; }
     string whyNotMessage() const { return m_whyNotOptimizable; }
     AstNode* whyNotNodep() const { return m_whyNotNodep; }
 
@@ -305,12 +305,12 @@ public:
     V3Number* fetchNumberNull(AstNode* nodep) {
         AstConst* constp = fetchConstNull(nodep);
         if (constp) return &constp->num();
-        return NULL;
+        return nullptr;
     }
     V3Number* fetchOutNumberNull(AstNode* nodep) {
         AstConst* constp = fetchOutConstNull(nodep);
         if (constp) return &constp->num();
-        return NULL;
+        return nullptr;
     }
 
 private:
@@ -350,7 +350,7 @@ private:
         }
     }
 
-    AstNode* varOrScope(AstVarRef* nodep) {
+    AstNode* varOrScope(AstVarRef* nodep) const {
         AstNode* vscp;
         if (m_scoped) {
             vscp = nodep->varScopep();
@@ -378,15 +378,15 @@ private:
     }
 
     // VISITORS
-    virtual void visit(AstAlways* nodep) VL_OVERRIDE {
+    virtual void visit(AstAlways* nodep) override {
         if (jumpingOver(nodep)) return;
         checkNodeInfo(nodep);
         iterateChildren(nodep);
     }
-    virtual void visit(AstSenTree* nodep) VL_OVERRIDE {
+    virtual void visit(AstSenTree* nodep) override {
         // Sensitivities aren't inputs per se; we'll keep our tree under the same sens.
     }
-    virtual void visit(AstVarRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstVarRef* nodep) override {
         if (jumpingOver(nodep)) return;
         if (!optimizable()) return;  // Accelerate
         UASSERT_OBJ(nodep->varp(), nodep, "Unlinked");
@@ -423,7 +423,7 @@ private:
                 }
                 vscp->user1(vscp->user1() | VU_RV);
                 bool isConst = nodep->varp()->isParam() && nodep->varp()->valuep();
-                AstNode* valuep = isConst ? fetchValueNull(nodep->varp()->valuep()) : NULL;
+                AstNode* valuep = isConst ? fetchValueNull(nodep->varp()->valuep()) : nullptr;
                 if (isConst
                     && valuep) {  // Propagate PARAM constants for constant function analysis
                     if (!m_checkOnly && optimizable()) newValue(vscp, valuep);
@@ -452,7 +452,7 @@ private:
             }
         }
     }
-    virtual void visit(AstVarXRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstVarXRef* nodep) override {
         if (jumpingOver(nodep)) return;
         if (m_scoped) {
             badNodeType(nodep);
@@ -462,7 +462,7 @@ private:
                                     "allowed in constant functions");
         }
     }
-    virtual void visit(AstNodeFTask* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeFTask* nodep) override {
         if (jumpingOver(nodep)) return;
         if (!m_params) {
             badNodeType(nodep);
@@ -474,7 +474,7 @@ private:
         checkNodeInfo(nodep);
         iterateChildren(nodep);
     }
-    virtual void visit(AstNodeIf* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeIf* nodep) override {
         if (jumpingOver(nodep)) return;
         UINFO(5, "   IF " << nodep << endl);
         checkNodeInfo(nodep);
@@ -491,15 +491,15 @@ private:
             }
         }
     }
-    virtual void visit(AstConst* nodep) VL_OVERRIDE {
+    virtual void visit(AstConst* nodep) override {
         checkNodeInfo(nodep);
         if (!m_checkOnly && optimizable()) newValue(nodep, nodep);
     }
-    virtual void visit(AstInitArray* nodep) VL_OVERRIDE {
+    virtual void visit(AstInitArray* nodep) override {
         checkNodeInfo(nodep);
         if (!m_checkOnly && optimizable()) newValue(nodep, nodep);
     }
-    virtual void visit(AstEnumItemRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstEnumItemRef* nodep) override {
         checkNodeInfo(nodep);
         UASSERT_OBJ(nodep->itemp(), nodep, "Not linked");
         if (!m_checkOnly && optimizable()) {
@@ -508,11 +508,11 @@ private:
                 iterateAndNextNull(valuep);
                 if (optimizable()) newValue(nodep, fetchValue(valuep));
             } else {
-                clearOptimizable(nodep, "No value found for enum item");
+                clearOptimizable(nodep, "No value found for enum item");  // LCOV_EXCL_LINE
             }
         }
     }
-    virtual void visit(AstNodeUniop* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeUniop* nodep) override {
         if (!optimizable()) return;  // Accelerate
         checkNodeInfo(nodep);
         iterateChildren(nodep);
@@ -520,7 +520,7 @@ private:
             nodep->numberOperate(newConst(nodep)->num(), fetchConst(nodep->lhsp())->num());
         }
     }
-    virtual void visit(AstNodeBiop* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeBiop* nodep) override {
         if (!optimizable()) return;  // Accelerate
         checkNodeInfo(nodep);
         iterateChildren(nodep);
@@ -529,7 +529,7 @@ private:
                                  fetchConst(nodep->rhsp())->num());
         }
     }
-    virtual void visit(AstNodeTriop* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeTriop* nodep) override {
         if (!optimizable()) return;  // Accelerate
         checkNodeInfo(nodep);
         iterateChildren(nodep);
@@ -539,7 +539,18 @@ private:
                                  fetchConst(nodep->thsp())->num());
         }
     }
-    virtual void visit(AstLogAnd* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeQuadop* nodep) override {
+        if (!optimizable()) return;  // Accelerate
+        checkNodeInfo(nodep);
+        iterateChildren(nodep);
+        if (!m_checkOnly && optimizable()) {
+            nodep->numberOperate(newConst(nodep)->num(), fetchConst(nodep->lhsp())->num(),
+                                 fetchConst(nodep->rhsp())->num(),
+                                 fetchConst(nodep->thsp())->num(),
+                                 fetchConst(nodep->fhsp())->num());
+        }
+    }
+    virtual void visit(AstLogAnd* nodep) override {
         // Need to short circuit
         if (!optimizable()) return;  // Accelerate
         checkNodeInfo(nodep);
@@ -557,7 +568,7 @@ private:
             }
         }
     }
-    virtual void visit(AstLogOr* nodep) VL_OVERRIDE {
+    virtual void visit(AstLogOr* nodep) override {
         // Need to short circuit
         if (!optimizable()) return;  // Accelerate
         checkNodeInfo(nodep);
@@ -575,7 +586,7 @@ private:
             }
         }
     }
-    virtual void visit(AstLogIf* nodep) VL_OVERRIDE {
+    virtual void visit(AstLogIf* nodep) override {
         // Need to short circuit, same as (!A || B)
         if (!optimizable()) return;  // Accelerate
         checkNodeInfo(nodep);
@@ -594,7 +605,7 @@ private:
             }
         }
     }
-    virtual void visit(AstNodeCond* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeCond* nodep) override {
         // We could use above visit(AstNodeTriop), but need to do short circuiting.
         // It's also slower even O(n^2) to evaluate both sides when we
         // really only need to evaluate one side.
@@ -636,7 +647,7 @@ private:
         }
         if (!m_checkOnly && optimizable()) {
             AstNode* vscp = varOrScope(varrefp);
-            AstInitArray* initp = NULL;
+            AstInitArray* initp = nullptr;
             if (AstInitArray* vscpnump = VN_CAST(fetchOutValueNull(vscp), InitArray)) {
                 initp = vscpnump;
             } else if (AstInitArray* vscpnump = VN_CAST(fetchValueNull(vscp), InitArray)) {
@@ -665,7 +676,7 @@ private:
         }
     }
     void handleAssignSel(AstNodeAssign* nodep, AstSel* selp) {
-        AstVarRef* varrefp = NULL;
+        AstVarRef* varrefp = nullptr;
         V3Number lsb(nodep);
         iterateAndNextNull(nodep->rhsp());  // Value to assign
         handleAssignSelRecurse(nodep, selp, varrefp /*ref*/, lsb /*ref*/, 0);
@@ -673,7 +684,7 @@ private:
             UASSERT_OBJ(varrefp, nodep,
                         "Indicated optimizable, but no variable found on RHS of select");
             AstNode* vscp = varOrScope(varrefp);
-            AstConst* outconstp = NULL;
+            AstConst* outconstp = nullptr;
             if (AstConst* vscpnump = fetchOutConstNull(vscp)) {
                 outconstp = vscpnump;
             } else if (AstConst* vscpnump = fetchConstNull(vscp)) {
@@ -713,7 +724,7 @@ private:
         }
     }
 
-    virtual void visit(AstNodeAssign* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeAssign* nodep) override {
         if (jumpingOver(nodep)) return;
         if (!optimizable()) return;  // Accelerate
         if (VN_IS(nodep, AssignDly)) {
@@ -750,7 +761,7 @@ private:
         }
         m_inDlyAssign = false;
     }
-    virtual void visit(AstArraySel* nodep) VL_OVERRIDE {
+    virtual void visit(AstArraySel* nodep) override {
         checkNodeInfo(nodep);
         iterateChildren(nodep);
         if (AstInitArray* initp = VN_CAST(fetchValueNull(nodep->fromp()), InitArray)) {
@@ -767,11 +778,11 @@ private:
             clearOptimizable(nodep, "Array select of non-array");
         }
     }
-    virtual void visit(AstBegin* nodep) VL_OVERRIDE {
+    virtual void visit(AstBegin* nodep) override {
         checkNodeInfo(nodep);
         iterateChildren(nodep);
     }
-    virtual void visit(AstNodeCase* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeCase* nodep) override {
         if (jumpingOver(nodep)) return;
         UINFO(5, "   CASE " << nodep << endl);
         checkNodeInfo(nodep);
@@ -809,16 +820,20 @@ private:
         }
     }
 
-    virtual void visit(AstCaseItem* nodep) VL_OVERRIDE {
+    virtual void visit(AstCaseItem* nodep) override {
         // Real handling is in AstNodeCase
         if (jumpingOver(nodep)) return;
         checkNodeInfo(nodep);
         iterateChildren(nodep);
     }
 
-    virtual void visit(AstComment*) VL_OVERRIDE {}
+    virtual void visit(AstComment*) override {}
 
-    virtual void visit(AstJumpGo* nodep) VL_OVERRIDE {
+    virtual void visit(AstJumpBlock* nodep) override {
+        if (jumpingOver(nodep)) return;
+        iterateChildren(nodep);
+    }
+    virtual void visit(AstJumpGo* nodep) override {
         if (jumpingOver(nodep)) return;
         checkNodeInfo(nodep);
         if (!m_checkOnly) {
@@ -826,16 +841,18 @@ private:
             m_jumpp = nodep;
         }
     }
-    virtual void visit(AstJumpLabel* nodep) VL_OVERRIDE {
+    virtual void visit(AstJumpLabel* nodep) override {
+        // This only supports forward jumps. That's all we make at present,
+        // AstJumpGo::broken uses brokeExistsBelow() to check this.
         if (jumpingOver(nodep)) return;
         checkNodeInfo(nodep);
         iterateChildren(nodep);
         if (m_jumpp && m_jumpp->labelp() == nodep) {
             UINFO(5, "   JUMP DONE " << nodep << endl);
-            m_jumpp = NULL;
+            m_jumpp = nullptr;
         }
     }
-    virtual void visit(AstStop* nodep) VL_OVERRIDE {
+    virtual void visit(AstStop* nodep) override {
         if (jumpingOver(nodep)) return;
         if (m_params) {  // This message seems better than an obscure $stop
             // The spec says $stop is just ignored, it seems evil to ignore assertions
@@ -846,7 +863,7 @@ private:
         checkNodeInfo(nodep);
     }
 
-    virtual void visit(AstNodeFor* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeFor* nodep) override {
         // Doing lots of Whiles is slow, so only for parameters
         UINFO(5, "   FOR " << nodep << endl);
         if (!m_params) {
@@ -878,7 +895,7 @@ private:
         }
     }
 
-    virtual void visit(AstWhile* nodep) VL_OVERRIDE {
+    virtual void visit(AstWhile* nodep) override {
         // Doing lots of Whiles is slow, so only for parameters
         if (jumpingOver(nodep)) return;
         UINFO(5, "   WHILE " << nodep << endl);
@@ -918,7 +935,7 @@ private:
         }
     }
 
-    virtual void visit(AstFuncRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstFuncRef* nodep) override {
         if (jumpingOver(nodep)) return;
         if (!optimizable()) return;  // Accelerate
         UINFO(5, "   FUNCREF " << nodep << endl);
@@ -970,7 +987,7 @@ private:
         }
     }
 
-    virtual void visit(AstVar* nodep) VL_OVERRIDE {
+    virtual void visit(AstVar* nodep) override {
         if (jumpingOver(nodep)) return;
         if (!m_params) {
             badNodeType(nodep);
@@ -978,12 +995,12 @@ private:
         }
     }
 
-    virtual void visit(AstScopeName* nodep) VL_OVERRIDE {
+    virtual void visit(AstScopeName* nodep) override {
         if (jumpingOver(nodep)) return;
         // Ignore
     }
 
-    virtual void visit(AstSFormatF* nodep) VL_OVERRIDE {
+    virtual void visit(AstSFormatF* nodep) override {
         if (jumpingOver(nodep)) return;
         if (!optimizable()) return;  // Accelerate
         iterateChildren(nodep);
@@ -992,9 +1009,9 @@ private:
 
             string result;
             string format = nodep->text();
-            string::const_iterator pos = format.begin();
+            auto pos = format.cbegin();
             bool inPct = false;
-            for (; pos != format.end(); ++pos) {
+            for (; pos != format.cend(); ++pos) {
                 if (!inPct && pos[0] == '%') {
                     inPct = true;
                 } else if (!inPct) {  // Normal text
@@ -1002,7 +1019,7 @@ private:
                 } else {  // Format character
                     inPct = false;
 
-                    if (V3Number::displayedFmtLegal(tolower(pos[0]))) {
+                    if (V3Number::displayedFmtLegal(tolower(pos[0]), false)) {
                         AstNode* argp = nextArgp;
                         nextArgp = nextArgp->nextp();
                         AstConst* constp = fetchConstNull(argp);
@@ -1035,7 +1052,7 @@ private:
         }
     }
 
-    virtual void visit(AstDisplay* nodep) VL_OVERRIDE {
+    virtual void visit(AstDisplay* nodep) override {
         if (jumpingOver(nodep)) return;
         if (!optimizable()) return;  // Accelerate
         iterateChildren(nodep);
@@ -1057,7 +1074,7 @@ private:
     // These types are definitely not reducible
     //   AstCoverInc, AstFinish,
     //   AstRand, AstTime, AstUCFunc, AstCCall, AstCStmt, AstUCStmt
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         if (jumpingOver(nodep)) return;
         badNodeType(nodep);
     }
@@ -1083,13 +1100,13 @@ public:
     }
     void clear() {
         m_whyNotOptimizable = "";
-        m_whyNotNodep = NULL;
+        m_whyNotNodep = nullptr;
         m_anyAssignComb = false;
         m_anyAssignDly = false;
         m_inDlyAssign = false;
         m_instrCount = 0;
         m_dataCount = 0;
-        m_jumpp = NULL;
+        m_jumpp = nullptr;
 
         AstNode::user1ClearTree();
         AstNode::user2ClearTree();
@@ -1114,16 +1131,11 @@ public:
         setMode(false /*scoped*/, false /*checking*/, true /*params*/);
         mainGuts(nodep);
     }
-    virtual ~SimulateVisitor() {
-        for (ConstPile::iterator it = m_constAllps.begin(); it != m_constAllps.end(); ++it) {
-            for (ConstDeque::iterator it2 = it->second.begin(); it2 != it->second.end(); ++it2) {
-                delete (*it2);
-            }
-        }
-        for (std::deque<AstNode*>::iterator it = m_reclaimValuesp.begin();
-             it != m_reclaimValuesp.end(); ++it) {
-            delete (*it);
+    virtual ~SimulateVisitor() override {
+        for (const auto& i : m_constAllps) {
+            for (AstConst* i2p : i.second) delete i2p;
         }
+        for (AstNode* ip : m_reclaimValuesp) delete ip;
         m_reclaimValuesp.clear();
         m_constFreeps.clear();
         m_constAllps.clear();
