@@ -87,11 +87,10 @@
 #include "V3Graph.h"
 
 #include <algorithm>
-#include <cstdarg>
 #include <map>
 #include <vector>
-#include VL_INCLUDE_UNORDERED_MAP
-#include VL_INCLUDE_UNORDERED_SET
+#include <unordered_map>
+#include <unordered_set>
 
 //######################################################################
 // Support classes
@@ -101,14 +100,16 @@ class SplitNodeVertex : public V3GraphVertex {
 
 protected:
     SplitNodeVertex(V3Graph* graphp, AstNode* nodep)
-        : V3GraphVertex(graphp)
-        , m_nodep(nodep) {}
-    virtual ~SplitNodeVertex() {}
+        : V3GraphVertex{graphp}
+        , m_nodep{nodep} {}
+    virtual ~SplitNodeVertex() override {}
     // ACCESSORS
     // Do not make accessor for nodep(),  It may change due to
     // reordering a lower block, but we don't repair it
-    virtual string name() const { return cvtToHex(m_nodep) + ' ' + m_nodep->prettyTypeName(); }
-    virtual FileLine* fileline() const { return nodep()->fileline(); }
+    virtual string name() const override {
+        return cvtToHex(m_nodep) + ' ' + m_nodep->prettyTypeName();
+    }
+    virtual FileLine* fileline() const override { return nodep()->fileline(); }
 
 public:
     virtual AstNode* nodep() const { return m_nodep; }
@@ -117,50 +118,49 @@ public:
 class SplitPliVertex : public SplitNodeVertex {
 public:
     explicit SplitPliVertex(V3Graph* graphp, AstNode* nodep)
-        : SplitNodeVertex(graphp, nodep) {}
-    virtual ~SplitPliVertex() {}
-    virtual string name() const { return "*PLI*"; }
-    virtual string dotColor() const { return "green"; }
+        : SplitNodeVertex{graphp, nodep} {}
+    virtual ~SplitPliVertex() override {}
+    virtual string name() const override { return "*PLI*"; }
+    virtual string dotColor() const override { return "green"; }
 };
 
 class SplitLogicVertex : public SplitNodeVertex {
 public:
     SplitLogicVertex(V3Graph* graphp, AstNode* nodep)
-        : SplitNodeVertex(graphp, nodep) {}
-    virtual ~SplitLogicVertex() {}
-    virtual string dotColor() const { return "yellow"; }
+        : SplitNodeVertex{graphp, nodep} {}
+    virtual ~SplitLogicVertex() override {}
+    virtual string dotColor() const override { return "yellow"; }
 };
 
 class SplitVarStdVertex : public SplitNodeVertex {
 public:
     SplitVarStdVertex(V3Graph* graphp, AstNode* nodep)
-        : SplitNodeVertex(graphp, nodep) {}
-    virtual ~SplitVarStdVertex() {}
-    virtual string dotColor() const { return "skyblue"; }
+        : SplitNodeVertex{graphp, nodep} {}
+    virtual ~SplitVarStdVertex() override {}
+    virtual string dotColor() const override { return "skyblue"; }
 };
 
 class SplitVarPostVertex : public SplitNodeVertex {
 public:
     SplitVarPostVertex(V3Graph* graphp, AstNode* nodep)
-        : SplitNodeVertex(graphp, nodep) {}
-    virtual ~SplitVarPostVertex() {}
-    virtual string name() const { return string("POST ") + SplitNodeVertex::name(); }
-    virtual string dotColor() const { return "CadetBlue"; }
+        : SplitNodeVertex{graphp, nodep} {}
+    virtual ~SplitVarPostVertex() override {}
+    virtual string name() const override { return string("POST ") + SplitNodeVertex::name(); }
+    virtual string dotColor() const override { return "CadetBlue"; }
 };
 
 //######################################################################
 // Edge types
 
 class SplitEdge : public V3GraphEdge {
-    uint32_t m_ignoreInStep;  // Step number that if set to, causes this edge to be ignored
+    uint32_t m_ignoreInStep = 0;  // Step number that if set to, causes this edge to be ignored
     static uint32_t s_stepNum;  // Global step number
 protected:
-    enum { WEIGHT_NORMAL = 10 };
+    static constexpr int WEIGHT_NORMAL = 10;
     SplitEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top, int weight,
               bool cutable = CUTABLE)
-        : V3GraphEdge(graphp, fromp, top, weight, cutable)
-        , m_ignoreInStep(0) {}
-    virtual ~SplitEdge() {}
+        : V3GraphEdge{graphp, fromp, top, weight, cutable} {}
+    virtual ~SplitEdge() override {}
 
 public:
     // Iterator for graph functions
@@ -179,7 +179,7 @@ public:
         if (!oedgep) v3fatalSrc("Following edge of non-SplitEdge type");
         return (!oedgep->ignoreThisStep());
     }
-    virtual string dotStyle() const {
+    virtual string dotStyle() const override {
         return ignoreThisStep() ? "dotted" : V3GraphEdge::dotStyle();
     }
 };
@@ -188,37 +188,37 @@ uint32_t SplitEdge::s_stepNum = 0;
 class SplitPostEdge : public SplitEdge {
 public:
     SplitPostEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top)
-        : SplitEdge(graphp, fromp, top, WEIGHT_NORMAL) {}
-    virtual ~SplitPostEdge() {}
-    virtual bool followScoreboard() const { return false; }
-    virtual string dotColor() const { return "khaki"; }
+        : SplitEdge{graphp, fromp, top, WEIGHT_NORMAL} {}
+    virtual ~SplitPostEdge() override {}
+    virtual bool followScoreboard() const override { return false; }
+    virtual string dotColor() const override { return "khaki"; }
 };
 
 class SplitLVEdge : public SplitEdge {
 public:
     SplitLVEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top)
-        : SplitEdge(graphp, fromp, top, WEIGHT_NORMAL) {}
-    virtual ~SplitLVEdge() {}
-    virtual bool followScoreboard() const { return true; }
-    virtual string dotColor() const { return "yellowGreen"; }
+        : SplitEdge{graphp, fromp, top, WEIGHT_NORMAL} {}
+    virtual ~SplitLVEdge() override {}
+    virtual bool followScoreboard() const override { return true; }
+    virtual string dotColor() const override { return "yellowGreen"; }
 };
 
 class SplitRVEdge : public SplitEdge {
 public:
     SplitRVEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top)
-        : SplitEdge(graphp, fromp, top, WEIGHT_NORMAL) {}
-    virtual ~SplitRVEdge() {}
-    virtual bool followScoreboard() const { return true; }
-    virtual string dotColor() const { return "green"; }
+        : SplitEdge{graphp, fromp, top, WEIGHT_NORMAL} {}
+    virtual ~SplitRVEdge() override {}
+    virtual bool followScoreboard() const override { return true; }
+    virtual string dotColor() const override { return "green"; }
 };
 
 struct SplitScorebdEdge : public SplitEdge {
 public:
     SplitScorebdEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top)
-        : SplitEdge(graphp, fromp, top, WEIGHT_NORMAL) {}
-    virtual ~SplitScorebdEdge() {}
-    virtual bool followScoreboard() const { return true; }
-    virtual string dotColor() const { return "blue"; }
+        : SplitEdge{graphp, fromp, top, WEIGHT_NORMAL} {}
+    virtual ~SplitScorebdEdge() override {}
+    virtual bool followScoreboard() const override { return true; }
+    virtual string dotColor() const override { return "blue"; }
 };
 
 struct SplitStrictEdge : public SplitEdge {
@@ -226,10 +226,10 @@ struct SplitStrictEdge : public SplitEdge {
     // The only non-cutable edge type
 public:
     SplitStrictEdge(V3Graph* graphp, V3GraphVertex* fromp, V3GraphVertex* top)
-        : SplitEdge(graphp, fromp, top, WEIGHT_NORMAL, NOT_CUTABLE) {}
-    virtual ~SplitStrictEdge() {}
-    virtual bool followScoreboard() const { return true; }
-    virtual string dotColor() const { return "blue"; }
+        : SplitEdge{graphp, fromp, top, WEIGHT_NORMAL, NOT_CUTABLE} {}
+    virtual ~SplitStrictEdge() override {}
+    virtual bool followScoreboard() const override { return true; }
+    virtual string dotColor() const override { return "blue"; }
 };
 
 //######################################################################
@@ -262,7 +262,7 @@ protected:
     // CONSTRUCTORS
 public:
     SplitReorderBaseVisitor() { scoreboardClear(); }
-    virtual ~SplitReorderBaseVisitor() {
+    virtual ~SplitReorderBaseVisitor() override {
         V3Stats::addStat("Optimizations, Split always", m_statSplits);
     }
 
@@ -275,7 +275,7 @@ protected:
         m_inDly = false;
         m_graph.clear();
         m_stmtStackps.clear();
-        m_pliVertexp = NULL;
+        m_pliVertexp = nullptr;
         m_noReorderWhy = "";
         AstNode::user1ClearTree();
         AstNode::user2ClearTree();
@@ -340,19 +340,19 @@ protected:
     virtual void makeRvalueEdges(SplitVarStdVertex* vstdp) = 0;
 
     // VISITORS
-    virtual void visit(AstAlways* nodep) VL_OVERRIDE = 0;
-    virtual void visit(AstNodeIf* nodep) VL_OVERRIDE = 0;
+    virtual void visit(AstAlways* nodep) override = 0;
+    virtual void visit(AstNodeIf* nodep) override = 0;
 
     // We don't do AstNodeFor/AstWhile loops, due to the standard question
     // of what is before vs. after
 
-    virtual void visit(AstAssignDly* nodep) VL_OVERRIDE {
+    virtual void visit(AstAssignDly* nodep) override {
         m_inDly = true;
         UINFO(4, "    ASSIGNDLY " << nodep << endl);
         iterateChildren(nodep);
         m_inDly = false;
     }
-    virtual void visit(AstVarRef* nodep) VL_OVERRIDE {
+    virtual void visit(AstVarRef* nodep) override {
         if (!m_stmtStackps.empty()) {
             AstVarScope* vscp = nodep->varScopep();
             UASSERT_OBJ(vscp, nodep, "Not linked");
@@ -394,18 +394,16 @@ protected:
                     SplitVarPostVertex* vpostp
                         = reinterpret_cast<SplitVarPostVertex*>(vscp->user2p());
                     // Add edges
-                    for (VStack::iterator it = m_stmtStackps.begin(); it != m_stmtStackps.end();
-                         ++it) {
-                        new SplitLVEdge(&m_graph, vpostp, *it);
+                    for (SplitLogicVertex* vxp : m_stmtStackps) {
+                        new SplitLVEdge(&m_graph, vpostp, vxp);
                     }
                 } else {  // Nondelayed assignment
                     if (nodep->lvalue()) {
                         // Non-delay; need to maintain existing ordering
                         // with all consumers of the signal
                         UINFO(4, "     VARREFLV: " << nodep << endl);
-                        for (VStack::iterator it = m_stmtStackps.begin();
-                             it != m_stmtStackps.end(); ++it) {
-                            new SplitLVEdge(&m_graph, vstdp, *it);
+                        for (SplitLogicVertex* ivxp : m_stmtStackps) {
+                            new SplitLVEdge(&m_graph, vstdp, ivxp);
                         }
                     } else {
                         UINFO(4, "     VARREF:   " << nodep << endl);
@@ -416,7 +414,7 @@ protected:
         }
     }
 
-    virtual void visit(AstJumpGo* nodep) VL_OVERRIDE {
+    virtual void visit(AstJumpGo* nodep) override {
         // Jumps will disable reordering at all levels
         // This is overly pessimistic; we could treat jumps as barriers, and
         // reorder everything between jumps/labels, however jumps are rare
@@ -428,7 +426,7 @@ protected:
 
     //--------------------
     // Default
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         // **** SPECIAL default type that sets PLI_ORDERING
         if (!m_stmtStackps.empty() && !nodep->isPure()) {
             UINFO(9, "         NotSplittable " << nodep << endl);
@@ -445,14 +443,12 @@ class ReorderVisitor : public SplitReorderBaseVisitor {
     // CONSTRUCTORS
 public:
     explicit ReorderVisitor(AstNetlist* nodep) { iterate(nodep); }
-    virtual ~ReorderVisitor() {}
+    virtual ~ReorderVisitor() override {}
 
     // METHODS
 protected:
-    virtual void makeRvalueEdges(SplitVarStdVertex* vstdp) VL_OVERRIDE {
-        for (VStack::iterator it = m_stmtStackps.begin(); it != m_stmtStackps.end(); ++it) {
-            new SplitRVEdge(&m_graph, *it, vstdp);
-        }
+    virtual void makeRvalueEdges(SplitVarStdVertex* vstdp) override {
+        for (SplitLogicVertex* vxp : m_stmtStackps) new SplitRVEdge(&m_graph, vxp, vstdp);
     }
 
     void cleanupBlockGraph(AstNode* nodep) {
@@ -503,7 +499,7 @@ protected:
         m_graph.weaklyConnected(&SplitEdge::followScoreboard);
 
         // Add hard orderings between all nodes of same color, in the order they appeared
-        vl_unordered_map<uint32_t, SplitLogicVertex*> lastOfColor;
+        std::unordered_map<uint32_t, SplitLogicVertex*> lastOfColor;
         for (AstNode* nextp = nodep; nextp; nextp = nextp->nextp()) {
             SplitLogicVertex* vvertexp = reinterpret_cast<SplitLogicVertex*>(nextp->user3p());
             uint32_t color = vvertexp->color();
@@ -547,7 +543,7 @@ protected:
             UINFO(6, "   No changes\n");
         } else {
             AstNRelinker replaceHandle;  // Where to add the list
-            AstNode* newListp = NULL;
+            AstNode* newListp = nullptr;
             for (RankNodeMap::const_iterator it = rankMap.begin(); it != rankMap.end(); ++it) {
                 AstNode* nextp = it->second;
                 UINFO(6, "   New order: " << nextp << endl);
@@ -573,7 +569,7 @@ protected:
         // Save recursion state
         AstNode* firstp = nodep;  // We may reorder, and nodep is no longer first.
         void* oldBlockUser3 = nodep->user3p();  // May be overloaded in below loop, save it
-        nodep->user3p(NULL);
+        nodep->user3p(nullptr);
         UASSERT_OBJ(nodep->firstAbovep(), nodep,
                     "Node passed is in next list; should have processed all list at once");
         // Process it
@@ -604,7 +600,7 @@ protected:
         firstp->user3p(oldBlockUser3);
     }
 
-    virtual void visit(AstAlways* nodep) VL_OVERRIDE {
+    virtual void visit(AstAlways* nodep) override {
         UINFO(4, "   ALW   " << nodep << endl);
         if (debug() >= 9) nodep->dumpTree(cout, "   alwIn:: ");
         scoreboardClear();
@@ -612,7 +608,7 @@ protected:
         if (debug() >= 9) nodep->dumpTree(cout, "   alwOut: ");
     }
 
-    virtual void visit(AstNodeIf* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeIf* nodep) override {
         UINFO(4, "     IF " << nodep << endl);
         iterateAndNextNull(nodep->condp());
         processBlock(nodep->ifsp());
@@ -623,7 +619,7 @@ private:
     VL_UNCOPYABLE(ReorderVisitor);
 };
 
-typedef vl_unordered_set<uint32_t> ColorSet;
+typedef std::unordered_set<uint32_t> ColorSet;
 typedef std::vector<AstAlways*> AlwaysVec;
 
 class IfColorVisitor : public AstNVisitor {
@@ -633,7 +629,7 @@ class IfColorVisitor : public AstNVisitor {
     typedef std::vector<AstNodeIf*> IfStack;
     IfStack m_ifStack;  // Stack of nested if-statements we're currently processing
 
-    typedef vl_unordered_map<AstNodeIf*, ColorSet> IfColorMap;
+    typedef std::unordered_map<AstNodeIf*, ColorSet> IfColorMap;
     IfColorMap m_ifColors;  // Map each if-statement to the set of colors (split blocks)
     // that will get a copy of that if-statement
 
@@ -642,12 +638,12 @@ public:
     // Visit through *nodep and map each AstNodeIf within to the set of
     // colors it will participate in. Also find the whole set of colors.
     explicit IfColorVisitor(AstAlways* nodep) { iterate(nodep); }
-    virtual ~IfColorVisitor() {}
+    virtual ~IfColorVisitor() override {}
 
     // METHODS
     const ColorSet& colors() const { return m_colors; }
     const ColorSet& colors(AstNodeIf* nodep) const {
-        IfColorMap::const_iterator it = m_ifColors.find(nodep);
+        const auto it = m_ifColors.find(nodep);
         UASSERT_OBJ(it != m_ifColors.end(), nodep, "Node missing from split color() map");
         return it->second;
     }
@@ -668,13 +664,13 @@ private:
     }
 
 protected:
-    virtual void visit(AstNodeIf* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeIf* nodep) override {
         m_ifStack.push_back(nodep);
         trackNode(nodep);
         iterateChildren(nodep);
         m_ifStack.pop_back();
     }
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         trackNode(nodep);
         iterateChildren(nodep);
     }
@@ -690,7 +686,7 @@ class EmitSplitVisitor : public AstNVisitor {
     const IfColorVisitor* m_ifColorp;  // Digest of results of prior coloring
 
     // Map each color to our current place within the color's new always
-    typedef vl_unordered_map<uint32_t, AstNode*> LocMap;
+    typedef std::unordered_map<uint32_t, AstNode*> LocMap;
     LocMap m_addAfter;
 
     AlwaysVec* m_newBlocksp;  // Split always blocks we have generated
@@ -701,13 +697,13 @@ public:
     // and generates its split blocks, writing the split blocks
     // into *newBlocksp.
     EmitSplitVisitor(AstAlways* nodep, const IfColorVisitor* ifColorp, AlwaysVec* newBlocksp)
-        : m_origAlwaysp(nodep)
-        , m_ifColorp(ifColorp)
-        , m_newBlocksp(newBlocksp) {
+        : m_origAlwaysp{nodep}
+        , m_ifColorp{ifColorp}
+        , m_newBlocksp{newBlocksp} {
         UINFO(6, "  splitting always " << nodep << endl);
     }
 
-    virtual ~EmitSplitVisitor() {}
+    virtual ~EmitSplitVisitor() override {}
 
     // METHODS
     void go() {
@@ -717,7 +713,7 @@ public:
             // We don't need to clone m_origAlwaysp->sensesp() here;
             // V3Activate already moved it to a parent node.
             AstAlways* alwaysp
-                = new AstAlways(m_origAlwaysp->fileline(), VAlwaysKwd::ALWAYS, NULL, NULL);
+                = new AstAlways(m_origAlwaysp->fileline(), VAlwaysKwd::ALWAYS, nullptr, nullptr);
             // Put a placeholder node into stmtp to track our position.
             // We'll strip these out after the blocks are fully cloned.
             AstSplitPlaceholder* placeholderp = makePlaceholderp();
@@ -738,7 +734,7 @@ protected:
         return new AstSplitPlaceholder(m_origAlwaysp->fileline());
     }
 
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         // Anything that's not an if/else we assume is a leaf
         // (that is, something we won't split.) Don't visit further
         // into the leaf.
@@ -760,9 +756,9 @@ protected:
         m_addAfter[color] = clonedp;
     }
 
-    virtual void visit(AstNodeIf* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeIf* nodep) override {
         const ColorSet& colors = m_ifColorp->colors(nodep);
-        typedef vl_unordered_map<uint32_t, AstNodeIf*> CloneMap;
+        typedef std::unordered_map<uint32_t, AstNodeIf*> CloneMap;
         CloneMap clones;
 
         for (ColorSet::const_iterator color = colors.begin(); color != colors.end(); ++color) {
@@ -786,15 +782,11 @@ protected:
 
         iterateAndNextNull(nodep->ifsp());
 
-        for (ColorSet::const_iterator color = colors.begin(); color != colors.end(); ++color) {
-            m_addAfter[*color] = clones[*color]->elsesp();
-        }
+        for (const auto& color : colors) m_addAfter[color] = clones[color]->elsesp();
 
         iterateAndNextNull(nodep->elsesp());
 
-        for (ColorSet::const_iterator color = colors.begin(); color != colors.end(); ++color) {
-            m_addAfter[*color] = clones[*color];
-        }
+        for (const auto& color : colors) m_addAfter[color] = clones[color];
     }
 
 private:
@@ -802,20 +794,19 @@ private:
 };
 
 class RemovePlaceholdersVisitor : public AstNVisitor {
-    typedef vl_unordered_set<AstNode*> NodeSet;
+    typedef std::unordered_set<AstNode*> NodeSet;
     NodeSet m_removeSet;  // placeholders to be removed
 public:
     explicit RemovePlaceholdersVisitor(AstNode* nodep) {
         iterate(nodep);
-        for (NodeSet::const_iterator it = m_removeSet.begin(); it != m_removeSet.end(); ++it) {
-            AstNode* np = *it;
+        for (AstNode* np : m_removeSet) {
             np->unlinkFrBack();  // Without next
             VL_DO_DANGLING(np->deleteTree(), np);
         }
     }
-    virtual ~RemovePlaceholdersVisitor() {}
-    virtual void visit(AstSplitPlaceholder* nodep) VL_OVERRIDE { m_removeSet.insert(nodep); }
-    virtual void visit(AstNode* nodep) VL_OVERRIDE { iterateChildren(nodep); }
+    virtual ~RemovePlaceholdersVisitor() override {}
+    virtual void visit(AstSplitPlaceholder* nodep) override { m_removeSet.insert(nodep); }
+    virtual void visit(AstNode* nodep) override { iterateChildren(nodep); }
 
 private:
     VL_UNCOPYABLE(RemovePlaceholdersVisitor);
@@ -826,16 +817,15 @@ private:
     // Keys are original always blocks pending delete,
     // values are newly split always blocks pending insertion
     // at the same position as the originals:
-    typedef vl_unordered_map<AstAlways*, AlwaysVec> ReplaceMap;
+    typedef std::unordered_map<AstAlways*, AlwaysVec> ReplaceMap;
     ReplaceMap m_replaceBlocks;
 
     // AstNodeIf* whose condition we're currently visiting
-    AstNode* m_curIfConditional;
+    AstNode* m_curIfConditional = nullptr;
 
     // CONSTRUCTORS
 public:
-    explicit SplitVisitor(AstNetlist* nodep)
-        : m_curIfConditional(NULL) {
+    explicit SplitVisitor(AstNetlist* nodep) {
         iterate(nodep);
 
         // Splice newly-split blocks into the tree. Remove placeholders
@@ -854,11 +844,11 @@ public:
         }
     }
 
-    virtual ~SplitVisitor() {}
+    virtual ~SplitVisitor() override {}
 
     // METHODS
 protected:
-    virtual void makeRvalueEdges(SplitVarStdVertex* vstdp) VL_OVERRIDE {
+    virtual void makeRvalueEdges(SplitVarStdVertex* vstdp) override {
         // Each 'if' depends on rvalues in its own conditional ONLY,
         // not rvalues in the if/else bodies.
         for (VStack::const_iterator it = m_stmtStackps.begin(); it != m_stmtStackps.end(); ++it) {
@@ -933,7 +923,7 @@ protected:
         if (debug() >= 9) m_graph.dumpDotFilePrefixed("splitg_colored", false);
     }
 
-    virtual void visit(AstAlways* nodep) VL_OVERRIDE {
+    virtual void visit(AstAlways* nodep) override {
         // build the scoreboard
         scoreboardClear();
         scanBlock(nodep->bodysp());
@@ -965,11 +955,11 @@ protected:
             emitSplit.go();
         }
     }
-    virtual void visit(AstNodeIf* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeIf* nodep) override {
         UINFO(4, "     IF " << nodep << endl);
         m_curIfConditional = nodep;
         iterateAndNextNull(nodep->condp());
-        m_curIfConditional = NULL;
+        m_curIfConditional = nullptr;
         scanBlock(nodep->ifsp());
         scanBlock(nodep->elsesp());
     }
