@@ -20,12 +20,10 @@
 #include "V3Global.h"
 #include "V3Stats.h"
 #include "V3Ast.h"
-#include "V3File.h"
 
 // This visitor does not edit nodes, and is called at error-exit, so should use constant iterators
 #include "V3AstConstOnly.h"
 
-#include <cstdarg>
 #include <iomanip>
 #include <map>
 
@@ -45,7 +43,6 @@ private:
     bool m_fast;
 
     AstCFunc* m_cfuncp;  // Current CFUNC
-    VDouble0 m_statInstrLong;  // Instruction count
     bool m_counting;  // Currently counting
     double m_instrs;  // Current instr count (for determining branch direction)
     bool m_tracingCall;  // Iterating into a CCall to a CFunc
@@ -78,7 +75,7 @@ private:
     }
 
     // VISITORS
-    virtual void visit(AstNodeModule* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeModule* nodep) override {
         allNodes(nodep);
         if (!m_fast) {
             // Count all CFuncs below this module
@@ -87,7 +84,7 @@ private:
         // Else we recursively trace fast CFuncs from the top _eval
         // func, see visit(AstNetlist*)
     }
-    virtual void visit(AstVar* nodep) VL_OVERRIDE {
+    virtual void visit(AstVar* nodep) override {
         allNodes(nodep);
         iterateChildrenConst(nodep);
         if (m_counting && nodep->dtypep()) {
@@ -113,7 +110,7 @@ private:
             }
         }
     }
-    virtual void visit(AstVarScope* nodep) VL_OVERRIDE {
+    virtual void visit(AstVarScope* nodep) override {
         allNodes(nodep);
         iterateChildrenConst(nodep);
         if (m_counting) {
@@ -122,7 +119,7 @@ private:
             }
         }
     }
-    virtual void visit(AstNodeIf* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeIf* nodep) override {
         UINFO(4, "   IF i=" << m_instrs << " " << nodep << endl);
         allNodes(nodep);
         // Condition is part of cost allocated to PREVIOUS block
@@ -138,28 +135,24 @@ private:
             double ifInstrs = 0.0;
             double elseInstrs = 0.0;
             if (nodep->branchPred() != VBranchPred::BP_UNLIKELY) {  // Check if
-                double prevInstr = m_instrs;
-                bool prevCounting = m_counting;
+                VL_RESTORER(m_instrs);
+                VL_RESTORER(m_counting);
                 {
                     m_counting = false;
                     m_instrs = 0.0;
                     iterateAndNextConstNull(nodep->ifsp());
                     ifInstrs = m_instrs;
                 }
-                m_instrs = prevInstr;
-                m_counting = prevCounting;
             }
             if (nodep->branchPred() != VBranchPred::BP_LIKELY) {  // Check else
-                double prevInstr = m_instrs;
-                bool prevCounting = m_counting;
+                VL_RESTORER(m_instrs);
+                VL_RESTORER(m_counting);
                 {
                     m_counting = false;
                     m_instrs = 0.0;
                     iterateAndNextConstNull(nodep->elsesp());
                     elseInstrs = m_instrs;
                 }
-                m_instrs = prevInstr;
-                m_counting = prevCounting;
             }
             // Now collect the stats
             if (m_counting) {
@@ -172,9 +165,9 @@ private:
         }
     }
     // While's we assume evaluate once.
-    // virtual void visit(AstWhile* nodep) VL_OVERRIDE {
+    // virtual void visit(AstWhile* nodep) override {
 
-    virtual void visit(AstNodeCCall* nodep) VL_OVERRIDE {
+    virtual void visit(AstNodeCCall* nodep) override {
         allNodes(nodep);
         iterateChildrenConst(nodep);
         if (m_fast && !nodep->funcp()->entryPoint()) {
@@ -183,7 +176,7 @@ private:
             iterate(nodep->funcp());
         }
     }
-    virtual void visit(AstCFunc* nodep) VL_OVERRIDE {
+    virtual void visit(AstCFunc* nodep) override {
         if (m_fast) {
             if (!m_tracingCall && !nodep->entryPoint()) return;
             m_tracingCall = false;
@@ -191,13 +184,13 @@ private:
         m_cfuncp = nodep;
         allNodes(nodep);
         iterateChildrenConst(nodep);
-        m_cfuncp = NULL;
+        m_cfuncp = nullptr;
     }
-    virtual void visit(AstNode* nodep) VL_OVERRIDE {
+    virtual void visit(AstNode* nodep) override {
         allNodes(nodep);
         iterateChildrenConst(nodep);
     }
-    virtual void visit(AstNetlist* nodep) VL_OVERRIDE {
+    virtual void visit(AstNetlist* nodep) override {
         if (m_fast && nodep->evalp()) {
             m_instrs = 0;
             m_counting = true;
@@ -211,10 +204,10 @@ private:
 public:
     // CONSTRUCTORS
     StatsVisitor(AstNetlist* nodep, const string& stage, bool fast)
-        : m_stage(stage)
-        , m_fast(fast) {
+        : m_stage{stage}
+        , m_fast{fast} {
         UINFO(9, "Starting stats, fast=" << fast << endl);
-        m_cfuncp = NULL;
+        m_cfuncp = nullptr;
         m_counting = !m_fast;
         m_instrs = 0;
         m_tracingCall = false;
@@ -223,7 +216,7 @@ public:
         // Process
         iterate(nodep);
     }
-    virtual ~StatsVisitor() {
+    virtual ~StatsVisitor() override {
         // Done. Publish statistics
         V3Stats::addStat(m_stage, "Instruction count, TOTAL", m_statInstr);
         V3Stats::addStat(m_stage, "Instruction count, fast critical", m_statInstrFast);
