@@ -66,10 +66,6 @@ bool VString::wildmatch(const string& s, const string& p) {
     return wildmatch(s.c_str(), p.c_str());
 }
 
-bool VString::isWildcard(const string& p) {
-    return ((p.find('*') != string::npos) || (p.find('?') != string::npos));
-}
-
 string VString::dot(const string& a, const string& dot, const string& b) {
     if (b == "") return a;
     if (a == "") return b;
@@ -78,30 +74,47 @@ string VString::dot(const string& a, const string& dot, const string& b) {
 
 string VString::downcase(const string& str) {
     string out = str;
-    for (string::iterator pos = out.begin(); pos != out.end(); ++pos) *pos = tolower(*pos);
+    for (auto& cr : out) cr = tolower(cr);
     return out;
 }
 
 string VString::upcase(const string& str) {
     string out = str;
-    for (string::iterator pos = out.begin(); pos != out.end(); ++pos) *pos = toupper(*pos);
+    for (auto& cr : out) cr = toupper(cr);
     return out;
 }
 
-string VString::quotePercent(const string& str) {
+string VString::quoteAny(const string& str, char tgt, char esc) {
     string out;
-    for (string::const_iterator pos = str.begin(); pos != str.end(); ++pos) {
-        if (*pos == '%') out += '%';
-        out += *pos;
+    for (const char c : str) {
+        if (c == tgt) out += esc;
+        out += c;
     }
     return out;
 }
 
+string VString::quoteStringLiteralForShell(const string& str) {
+    string result;
+    const char dquote = '"';
+    const char escape = '\\';
+    result.push_back(dquote);  // Start quoted string
+    result.push_back(escape);
+    result.push_back(dquote);  // "
+    for (const char c : str) {
+        if (c == dquote || c == escape) result.push_back(escape);
+        result.push_back(c);
+    }
+    result.push_back(escape);
+    result.push_back(dquote);  // "
+    result.push_back(dquote);  // Terminate quoted string
+    return result;
+}
+
 string VString::spaceUnprintable(const string& str) {
     string out;
-    for (string::const_iterator pos = str.begin(); pos != str.end(); ++pos) {
-        if (isprint(*pos)) {
-            out += *pos;
+    for (const char c : str) {
+        if (isprint(c)) {
+            out += c;
         } else {
             out += ' ';
         }
@@ -112,19 +125,37 @@ string VString::spaceUnprintable(const string& str) {
 string VString::removeWhitespace(const string& str) {
     string out;
     out.reserve(str.size());
-    for (string::const_iterator pos = str.begin(); pos != str.end(); ++pos) {
-        if (!isspace(*pos)) out += *pos;
+    for (const char c : str) {
+        if (!isspace(c)) out += c;
     }
     return out;
 }
 
 bool VString::isWhitespace(const string& str) {
-    for (string::const_iterator pos = str.begin(); pos != str.end(); ++pos) {
-        if (!isspace(*pos)) return false;
+    for (const char c : str) {
+        if (!isspace(c)) return false;
     }
     return true;
 }
 
+double VString::parseDouble(const string& str, bool* successp) {
+    char* strgp = new char[str.size() + 1];
+    char* dp = strgp;
+    if (successp) *successp = true;
+    for (const char* sp = str.c_str(); *sp; ++sp) {
+        if (*sp != '_') *dp++ = *sp;
+    }
+    *dp++ = '\0';
+    char* endp = strgp;
+    double d = strtod(strgp, &endp);
+    size_t parsed_len = endp - strgp;
+    if (parsed_len != strlen(strgp)) {
+        if (successp) *successp = false;
+    }
+    VL_DO_DANGLING(delete[] strgp, strgp);
+    return d;
+}
+
 //######################################################################
 // VHashSha256
 
@@ -421,8 +452,7 @@ string VSpellCheck::bestCandidateInfo(const string& goal, EditDistance& distance
     string bestCandidate;
     size_t gLen = goal.length();
     distancer = LENGTH_LIMIT * 10;
-    for (Candidates::const_iterator it = m_candidates.begin(); it != m_candidates.end(); ++it) {
-        const string candidate = *it;
+    for (const string& candidate : m_candidates) {
         size_t cLen = candidate.length();
 
         // Min distance must be inserting/deleting to make lengths match
