@@ -23,8 +23,11 @@
 #include "V3AssertPre.h"
 #include "V3Begin.h"
 #include "V3Branch.h"
+#include "V3CCtors.h"
+#include "V3CUse.h"
 #include "V3Case.h"
 #include "V3Cast.h"
+#include "V3Cdc.h"
 #include "V3Changed.h"
 #include "V3Class.h"
 #include "V3Clean.h"
@@ -33,42 +36,40 @@
 #include "V3Const.h"
 #include "V3Coverage.h"
 #include "V3CoverageJoin.h"
-#include "V3CCtors.h"
-#include "V3CUse.h"
 #include "V3Dead.h"
 #include "V3Delayed.h"
 #include "V3Depth.h"
 #include "V3DepthBlock.h"
 #include "V3Descope.h"
 #include "V3EmitC.h"
-#include "V3EmitCMake.h"
 #include "V3EmitCMain.h"
+#include "V3EmitCMake.h"
 #include "V3EmitMk.h"
 #include "V3EmitV.h"
 #include "V3EmitXml.h"
 #include "V3Expand.h"
 #include "V3File.h"
-#include "V3Cdc.h"
 #include "V3Gate.h"
 #include "V3GenClk.h"
 #include "V3Graph.h"
+#include "V3HierBlock.h"
 #include "V3Inline.h"
 #include "V3Inst.h"
 #include "V3Life.h"
 #include "V3LifePost.h"
-#include "V3LinkCells.h"
 #include "V3LinkDot.h"
 #include "V3LinkJump.h"
+#include "V3LinkInc.h"
 #include "V3LinkLValue.h"
 #include "V3LinkLevel.h"
 #include "V3LinkParse.h"
 #include "V3LinkResolve.h"
 #include "V3Localize.h"
+#include "V3MergeCond.h"
 #include "V3Name.h"
 #include "V3Order.h"
 #include "V3Os.h"
 #include "V3Param.h"
-#include "V3Parse.h"
 #include "V3ParseSym.h"
 #include "V3Partition.h"
 #include "V3PreShell.h"
@@ -84,29 +85,46 @@
 #include "V3Stats.h"
 #include "V3String.h"
 #include "V3Subst.h"
+#include "V3TSP.h"
 #include "V3Table.h"
 #include "V3Task.h"
 #include "V3Trace.h"
 #include "V3TraceDecl.h"
 #include "V3Tristate.h"
-#include "V3TSP.h"
 #include "V3Undriven.h"
 #include "V3Unknown.h"
 #include "V3Unroll.h"
+#include "V3Waiver.h"
 #include "V3Width.h"
 
 #include <ctime>
-#include <sys/stat.h>
 
 V3Global v3Global;
 
+static void reportStatsIfEnabled() {
+    if (v3Global.opt.stats()) {
+        V3Stats::statsFinalAll(v3Global.rootp());
+        V3Stats::statsReport();
+    }
+}
+
 static void process() {
     // Sort modules by level so later algorithms don't need to care
     V3LinkLevel::modSortByLevel();
     V3Error::abortIfErrors();
+    if (v3Global.opt.debugExitParse()) {
+        cout << "--debug-exit-parse: Exiting after parse\n";
+        exit(0);
+    }
 
     // Convert parseref's to varrefs, and other directly post parsing fixups
     V3LinkParse::linkParse(v3Global.rootp());
+    if (v3Global.opt.debugExitUvm()) {
+        V3Error::abortIfErrors();
+        cout << "--debug-exit-uvm: Exiting after UVM-supported pass\n";
+        exit(0);
+    }
+
     // Cross-link signal names
     // Cross-link dotted hierarchical references
     V3LinkDot::linkDotPrimary(v3Global.rootp());
@@ -119,6 +137,8 @@ static void process() {
     V3LinkLValue::linkLValue(v3Global.rootp());
     // Convert return/continue/disable to jumps
     V3LinkJump::linkJump(v3Global.rootp());
+    // Convert --/++ to normal operations. Must be after LinkJump.
+    V3LinkInc::linkIncrements(v3Global.rootp());
     V3Error::abortIfErrors();
 
     if (v3Global.opt.stats()) V3Stats::statsStageAll(v3Global.rootp(), "Link");
@@ -133,6 +153,17 @@ static void process() {
     V3Dead::deadifyModules(v3Global.rootp());
     v3Global.checkTree();
 
+    // Create a hierarchical verilation plan
+    if (!v3Global.opt.lintOnly() && !v3Global.opt.xmlOnly() && v3Global.opt.hierarchical()) {
+        V3HierBlockPlan::createPlan(v3Global.rootp());
+        // If a plan is created, further analysis is not necessary.
+        // The actual Verilation will be done based on this plan.
+        if (v3Global.hierPlanp()) {
+            reportStatsIfEnabled();
+            return;
+        }
+    }
+
     // Calculate and check widths, edit tree to TRUNC/EXTRACT any width mismatches
     V3Width::width(v3Global.rootp());
 
@@ -187,12 +218,16 @@ static void process() {
         // Expand inouts, stage 2
         // Also simplify pin connections to always be AssignWs in prep for V3Unknown
         V3Tristate::tristateAll(v3Global.rootp());
+    }
 
+    if (!v3Global.opt.xmlOnly()) {
         // Move assignments from X into MODULE temps.
         // (Before flattening, so each new X variable is shared between all scopes of that module.)
         V3Unknown::unknownAll(v3Global.rootp());
         v3Global.constRemoveXs(true);
+    }
 
+    if (!(v3Global.opt.xmlOnly() && !v3Global.opt.flatten())) {
         // Module inlining
         // Cannot remove dead variables after this, as alias information for final
         // V3Scope's V3LinkDot is in the AstVar.
@@ -419,13 +454,18 @@ static void process() {
         V3Dead::deadifyAll(v3Global.rootp());
     }
 
-    if (!v3Global.opt.lintOnly() && !v3Global.opt.xmlOnly() && v3Global.opt.oReloop()) {
-        // Reform loops to reduce code size
-        // Must be after all Sel/array index based optimizations
-        V3Reloop::reloopAll(v3Global.rootp());
-    }
-
     if (!v3Global.opt.lintOnly() && !v3Global.opt.xmlOnly()) {
+        if (v3Global.opt.oMergeCond()) {
+            // Merge conditionals
+            V3MergeCond::mergeAll(v3Global.rootp());
+        }
+
+        if (v3Global.opt.oReloop()) {
+            // Reform loops to reduce code size
+            // Must be after all Sel/array index based optimizations
+            V3Reloop::reloopAll(v3Global.rootp());
+        }
+
         // Fix very deep expressions
         // Mark evaluation functions as member functions, if needed.
         V3Depth::depthAll(v3Global.rootp());
@@ -447,7 +487,7 @@ static void process() {
     if (!v3Global.opt.lintOnly() && !v3Global.opt.xmlOnly() && !v3Global.opt.dpiHdrOnly()) {
         // Create AstCUse to determine what class forward declarations/#includes needed in C
         // Must be before V3EmitC
-        V3CUse::cUseAll(v3Global.rootp());
+        V3CUse::cUseAll();
 
         // emitcInlines is first, as it may set needHInlines which other emitters read
         V3EmitC::emitcInlines();
@@ -481,10 +521,7 @@ static void process() {
     }
 
     // Statistics
-    if (v3Global.opt.stats()) {
-        V3Stats::statsFinalAll(v3Global.rootp());
-        V3Stats::statsReport();
-    }
+    reportStatsIfEnabled();
 
     if (!v3Global.opt.lintOnly() && !v3Global.opt.xmlOnly() && !v3Global.opt.dpiHdrOnly()) {
         // Makefile must be after all other emitters
@@ -502,8 +539,9 @@ static void verilate(const string& argString) {
     // Can we skip doing everything if times are ok?
     V3File::addSrcDepend(v3Global.opt.bin());
     if (v3Global.opt.skipIdentical().isTrue()
-        && V3File::checkTimes(
-            v3Global.opt.makeDir() + "/" + v3Global.opt.prefix() + "__verFiles.dat", argString)) {
+        && V3File::checkTimes(v3Global.opt.hierTopDataDir() + "/" + v3Global.opt.prefix()
+                                  + "__verFiles.dat",
+                              argString)) {
         UINFO(1, "--skip-identical: No change to any source files, exiting\n");
         return;
     }
@@ -516,9 +554,9 @@ static void verilate(const string& argString) {
     //--FRONTEND------------------
 
     // Cleanup
-    V3Os::unlinkRegexp(v3Global.opt.makeDir(), v3Global.opt.prefix() + "_*.tree");
-    V3Os::unlinkRegexp(v3Global.opt.makeDir(), v3Global.opt.prefix() + "_*.dot");
-    V3Os::unlinkRegexp(v3Global.opt.makeDir(), v3Global.opt.prefix() + "_*.txt");
+    V3Os::unlinkRegexp(v3Global.opt.hierTopDataDir(), v3Global.opt.prefix() + "_*.tree");
+    V3Os::unlinkRegexp(v3Global.opt.hierTopDataDir(), v3Global.opt.prefix() + "_*.dot");
+    V3Os::unlinkRegexp(v3Global.opt.hierTopDataDir(), v3Global.opt.prefix() + "_*.txt");
 
     // Internal tests (after option parsing as need debug() setting,
     // and after removing files as may make debug output)
@@ -542,17 +580,42 @@ static void verilate(const string& argString) {
 
     // Final steps
     V3Global::dumpCheckGlobalTree("final", 990, v3Global.opt.dumpTreeLevel(__FILE__) >= 3);
+
+    V3Error::abortIfErrors();
+
+    if (v3Global.opt.isWaiverOutput()) {
+        // Create waiver output, must be just before we exit on warnings
+        V3Waiver::write(v3Global.opt.waiverOutput());
+    }
+
     V3Error::abortIfWarnings();
 
+    if (v3Global.hierPlanp()) {  // This run is for just write a makefile
+        UASSERT(v3Global.opt.hierarchical(), "hierarchical must be set");
+        UASSERT(!v3Global.opt.hierChild(), "This must not be a hierarhcical-child run");
+        UASSERT(v3Global.opt.hierBlocks().empty(), "hierarchical-block must not be set");
+        if (v3Global.opt.gmake()) {
+            v3Global.hierPlanp()->writeCommandArgsFiles(false);
+            V3EmitMk::emitHierVerilation(v3Global.hierPlanp());
+        }
+        if (v3Global.opt.cmake()) {
+            v3Global.hierPlanp()->writeCommandArgsFiles(true);
+            V3EmitCMake::emit();
+        }
+    }
     if (v3Global.opt.makeDepend().isTrue()) {
-        V3File::writeDepend(v3Global.opt.makeDir() + "/" + v3Global.opt.prefix() + "__ver.d");
+        string filename = v3Global.opt.makeDir() + "/" + v3Global.opt.prefix();
+        filename += v3Global.opt.hierTop() ? "__hierVer.d" : "__ver.d";
+        V3File::writeDepend(filename);
     }
     if (v3Global.opt.protectIds()) {
-        VIdProtect::writeMapFile(v3Global.opt.makeDir() + "/" + v3Global.opt.prefix()
+        VIdProtect::writeMapFile(v3Global.opt.hierTopDataDir() + "/" + v3Global.opt.prefix()
                                  + "__idmap.xml");
     }
+
     if (v3Global.opt.skipIdentical().isTrue() || v3Global.opt.makeDepend().isTrue()) {
-        V3File::writeTimes(v3Global.opt.makeDir() + "/" + v3Global.opt.prefix() + "__verFiles.dat",
+        V3File::writeTimes(v3Global.opt.hierTopDataDir() + "/" + v3Global.opt.prefix()
+                               + "__verFiles.dat",
                            argString);
     }
 
@@ -565,30 +628,45 @@ static void verilate(const string& argString) {
     FileLine::deleteAllRemaining();
 }
 
+static string buildMakeCmd(const string& makefile, const string& target) {
+    const V3StringList& makeFlags = v3Global.opt.makeFlags();
+    const int jobs = v3Global.opt.buildJobs();
+    UASSERT(jobs >= 0, "-j option parser in V3Options.cpp filters out negative value");
+
+    std::ostringstream cmd;
+    cmd << v3Global.opt.getenvMAKE();
+    cmd << " -C " << v3Global.opt.makeDir();
+    cmd << " -f " << makefile;
+    if (jobs == 0) {
+        cmd << " -j";
+    } else if (jobs > 1) {
+        cmd << " -j " << jobs;
+    }
+    for (const string& flag : makeFlags) cmd << ' ' << flag;
+    if (!target.empty()) cmd << ' ' << target;
+
+    return cmd.str();
+}
+
 static void execBuildJob() {
     UASSERT(v3Global.opt.build(), "--build is not specified.");
     UASSERT(v3Global.opt.gmake(), "--build requires GNU Make.");
     UASSERT(!v3Global.opt.cmake(), "--build cannot use CMake.");
     UINFO(1, "Start Build\n");
 
-    const V3StringList& makeFlags = v3Global.opt.makeFlags();
-    const int jobs = v3Global.opt.buildJobs();
-    UASSERT(jobs >= 0, "-j option parser in V3Options.cpp filters out negative value");
-
-    std::stringstream cmd;
-    cmd << v3Global.opt.getenvMAKE();
-    cmd << " -C " << v3Global.opt.makeDir();
-    cmd << " -f " << v3Global.opt.prefix() << ".mk";
-    if (jobs == 0) {
-        cmd << " -j";
-    } else if (jobs > 1) {
-        cmd << " -j " << jobs;
-    }
-    for (V3StringList::const_iterator it = makeFlags.begin(); it != makeFlags.end(); ++it) {
-        cmd << ' ' << *it;
+    const string cmdStr = buildMakeCmd(v3Global.opt.prefix() + ".mk", "");
+    const int exit_code = V3Os::system(cmdStr);
+    if (exit_code != 0) {
+        v3error(cmdStr << " exitted with " << exit_code << std::endl);
+        exit(exit_code);
     }
+}
 
-    const std::string cmdStr = cmd.str();
+static void execHierVerilation() {
+    UASSERT(v3Global.hierPlanp(), "must be called only when plan exists");
+    const string makefile = v3Global.opt.prefix() + "_hier.mk ";
+    const string target = v3Global.opt.build() ? " hier_build" : " hier_verilation";
+    const string cmdStr = buildMakeCmd(makefile, target);
     const int exit_code = V3Os::system(cmdStr);
     if (exit_code != 0) {
         v3error(cmdStr << " exitted with " << exit_code << std::endl);
@@ -630,7 +708,14 @@ int main(int argc, char** argv, char** env) {
         UINFO(1, "Option --no-verilate: Skip Verilation\n");
     }
 
-    if (v3Global.opt.build()) execBuildJob();
+    if (v3Global.hierPlanp() && v3Global.opt.gmake()) {
+        execHierVerilation();  // execHierVerilation() takes care of --build too
+    } else if (v3Global.opt.build()) {
+        execBuildJob();
+    }
+
+    // Explicitly release resources
+    v3Global.shutdown();
 
     UINFO(1, "Done, Exiting...\n");
 }
