@@ -15,7 +15,7 @@ unsigned int main_time = 0;
 
 double sc_time_stamp() { return main_time; }
 
-VM_PREFIX* topp = NULL;
+VM_PREFIX* topp = nullptr;
 bool fail = false;
 
 void check(QData got, QData exp) {
@@ -32,11 +32,11 @@ int main(int argc, char* argv[]) {
 
     topp->a0 = 0;
     topp->eval();
-    check(topp->y, VL_ULL(0x0));
+    check(topp->y, 0x0ULL);
 
     topp->a0 = 15;
     topp->eval();
-    check(topp->y, VL_ULL(0x3c00000000));
+    check(topp->y, 0x3c00000000ULL);
 
     topp->final();
     if (!fail) {
