@@ -14,8 +14,6 @@
 
 #include "Vt_scope_map.h"
 
-using namespace std;
-
 unsigned long long main_time = 0;
 double sc_time_stamp() { return (double)main_time; }
 
@@ -45,24 +43,23 @@ int main(int argc, char** argv, char** env) {
         it->second->scopeDump();
 #endif
         VerilatedVarNameMap* varNameMap = it->second->varsp();
-        if (varNameMap == NULL) {
+        if (!varNameMap) {
             VL_PRINTF("%%Error: Bad varsp()\n");
             return -1;
         }
 
-        for (VerilatedVarNameMap::iterator varIt = varNameMap->begin(); varIt != varNameMap->end();
-             ++varIt) {
-            VerilatedVar* var = &varIt->second;
-            int varLeft = var->packed().left();
-            int varRight = var->packed().right();
+        for (const auto& varname : *varNameMap) {
+            const VerilatedVar* varp = &(varname.second);
+            int varLeft = varp->packed().left();
+            int varRight = varp->packed().right();
 
 #ifdef TEST_VERBOSE
-            VL_PRINTF("\tVar = %s\n", varIt->first);
-            VL_PRINTF("\t  Type = %d\n", var->vltype());
-            VL_PRINTF("\t  EntSize = %d\n", var->entSize());
-            VL_PRINTF("\t  Dims = %d\n", var->dims());
+            VL_PRINTF("\tVar = %s\n", varname.first);
+            VL_PRINTF("\t  Type = %d\n", varp->vltype());
+            VL_PRINTF("\t  EntSize = %d\n", varp->entSize());
+            VL_PRINTF("\t  Dims = %d\n", varp->dims());
             VL_PRINTF("\t  Range = %d:%d\n", varLeft, varRight);
-            VL_PRINTF("\t  Is RW = %d\n", var->isPublicRW());
+            VL_PRINTF("\t  Is RW = %d\n", varp->isPublicRW());
 #endif
 
             if (varRight != 0) {
@@ -73,7 +70,7 @@ int main(int argc, char** argv, char** env) {
             int varBits = varLeft + 1;
 
             // First expect an incrementing byte pattern
-            vluint8_t* varData = reinterpret_cast<vluint8_t*>(var->datap());
+            vluint8_t* varData = reinterpret_cast<vluint8_t*>(varp->datap());
             for (int i = 0; i < varBits / 8; i++) {
 #ifdef TEST_VERBOSE
                 VL_PRINTF("%02x ", varData[i]);
@@ -121,24 +118,23 @@ int main(int argc, char** argv, char** env) {
     for (VerilatedScopeNameMap::const_iterator it = scopeMapp->begin(); it != scopeMapp->end();
          ++it) {
         VerilatedVarNameMap* varNameMap = it->second->varsp();
-        if (varNameMap == NULL) {
+        if (!varNameMap) {
             VL_PRINTF("%%Error: Bad varsp()\n");
             return -1;
         }
 
-        for (VerilatedVarNameMap::iterator varIt = varNameMap->begin(); varIt != varNameMap->end();
-             ++varIt) {
-            VerilatedVar* var = &varIt->second;
-            int varLeft = var->packed().left();
+        for (const auto& varname : *varNameMap) {
+            const VerilatedVar* varp = &(varname.second);
+            int varLeft = varp->packed().left();
             int varBits = varLeft + 1;
-            vluint8_t* varData = reinterpret_cast<vluint8_t*>(var->datap());
+            vluint8_t* varData = reinterpret_cast<vluint8_t*>(varp->datap());
 
             // Check that all bits are high now
             for (int i = 0; i < varBits / 8; i++) {
                 vluint8_t expected = 0xff;
                 if (varData[i] != expected) {
                     VL_PRINTF("%%Error: Data mismatch (%s), got 0x%02x, expected 0x%02x\n",
-                              varIt->first, varData[i], expected);
+                              varname.first, varData[i], expected);
                     return -1;
                 }
             }
@@ -148,7 +144,7 @@ int main(int argc, char** argv, char** env) {
                 vluint8_t expected = ~(0xff << (varBits % 8));
                 if (got != expected) {
                     VL_PRINTF("%%Error: Data mismatch (%s), got 0x%02x, expected 0x%02x\n",
-                              varIt->first, got, expected);
+                              varname.first, got, expected);
                     return -1;
                 }
             }
